@@ -36,12 +36,12 @@ static const char pcb_acts_ExtFoo[] = "ExtFoo(arg1, arg2, [arg3]...)";
 static const char pcb_acth_ExtFoo[] = "Help text: short description of what the action does.";
 static fgw_error_t pcb_act_ExtFoo(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	pcb_message(PCB_MSG_ERROR, "PLEASE CONSIDER DEVELOPING A CORE PLUGIN INSTEAD!\n");
-	PCB_ACT_IRES(0);
+	rnd_message(RND_MSG_ERROR, "PLEASE CONSIDER DEVELOPING A CORE PLUGIN INSTEAD!\n");
+	RND_ACT_IRES(0);
 	return 0;
 }
 
-static const pcb_action_t ext_foo_action_list[] = {
+static const rnd_action_t ext_foo_action_list[] = {
 	{"ExtFoo", pcb_act_ExtFoo, pcb_acth_ExtFoo, pcb_acts_ExtFoo}
 };
 
@@ -50,15 +50,15 @@ int pplg_check_ver_ext_foo(int ver_needed) { return 0; }
 void pplg_uninit_ext_foo(void)
 {
 	fprintf(stderr, "EXT FOO uninit\n");
-	pcb_remove_actions_by_cookie(ext_foo_cookie);
+	rnd_remove_actions_by_cookie(ext_foo_cookie);
 }
 
 
 int pplg_init_ext_foo(void)
 {
-	PCB_API_CHK_VER; /* for external plugins this is CRITICAL */
+	RND_API_CHK_VER; /* for external plugins this is CRITICAL */
 
-	PCB_REGISTER_ACTIONS(ext_foo_action_list, ext_foo_cookie);
+	RND_REGISTER_ACTIONS(ext_foo_action_list, ext_foo_cookie);
 
 	fprintf(stderr, "EXT FOO init\n");
 	return 0;
