@@ -11,7 +11,7 @@
 #include "util/arg_auto_set.h"
 #include "Rev.h"
 
-#define version "2.3.1"
+#define version "2.4.0"
 
 #define LIBRND_ROOT         "../src_3rd/librnd-local/src/librnd"
 #define LIBRND_PLUGIN_ROOT  "../src_3rd/librnd-local/src/librnd/plugins"
@@ -62,6 +62,10 @@ int hook_custom_arg(const char *key, const char *value)
 		put("/local/pcb/dot_pcb_rnd", value);
 		return 1;
 	}
+	if (strcmp(key, "static") == 0) {
+		put("/local/pcb/want_static", strue);
+		return 1;
+	}
 	if (arg_auto_set(key, value, disable_libs) == 0) {
 		fprintf(stderr, "Error: unknown argument %s\n", key);
 		exit(1);
@@ -85,6 +89,7 @@ int hook_postinit()
 
 	put("/local/pcb/want_bison", sfalse);
 	put("/local/pcb/want_byaccic", sfalse);
+	put("/local/pcb/want_static", sfalse);
 	put("/local/pcb/dot_pcb_rnd", ".pcb-rnd");
 
 	put("/local/pcb/librnd_is_local", strue);
@@ -145,9 +150,6 @@ int hook_detect_target()
 	want_freetype2 = plug_is_enabled("import_ttf");
 	want_fuse = plug_is_enabled("export_vfs_fuse");
 
-/****** TODO #21: core depends on this plugin (yes, this is a bug) ******/
-	hook_custom_arg("buildin-lib_compat_help", NULL);
-
 	plugin_db_hidlib();
 
 	rnd_hook_detect_cc();
