@@ -61,6 +61,7 @@ void pcb_board_free(pcb_board_t * pcb)
 	free(pcb->PrintFilename);
 	pcb_ratspatch_destroy(pcb);
 	pcb_data_free(pcb->Data);
+	pcb_data_free(pcb->uilayer_data);
 
 	/* release font symbols */
 	pcb_fontkit_free(&pcb->fontkit);
@@ -84,6 +85,7 @@ pcb_board_t *pcb_board_new_(rnd_bool SetDefaultNames)
 	/* allocate memory, switch all layers on and copy resources */
 	ptr = calloc(1, sizeof(pcb_board_t));
 	ptr->Data = pcb_buffer_new(ptr);
+	ptr->uilayer_data = pcb_buffer_new(ptr);
 
 	for(i = 0; i < PCB_NUM_NETLISTS; i++)
 		pcb_netlist_init(&(ptr->netlist[i]));
@@ -328,48 +330,6 @@ void pcb_board_remove(pcb_board_t *Ptr)
 	free(Ptr);
 }
 
-/* sets a new line thickness */
-void pcb_board_set_line_width(rnd_coord_t Size)
-{
-	if (Size >= PCB_MIN_THICKNESS && Size <= PCB_MAX_THICKNESS) {
-		rnd_conf_set_design("design/line_thickness", "%$mS", Size);
-		if (conf_core.editor.auto_drc)
-			pcb_crosshair_grid_fit(PCB, pcb_crosshair.X, pcb_crosshair.Y);
-	}
-}
-
-/* sets a new via thickness */
-void pcb_board_set_via_size(rnd_coord_t Size, rnd_bool Force)
-{
-	if (Force || (Size <= PCB_MAX_PINORVIASIZE && Size >= PCB_MIN_PINORVIASIZE && Size >= conf_core.design.via_drilling_hole + PCB_MIN_PINORVIACOPPER)) {
-		rnd_conf_set_design("design/via_thickness", "%$mS", Size);
-	}
-}
-
-/* sets a new via drilling hole */
-void pcb_board_set_via_drilling_hole(rnd_coord_t Size, rnd_bool Force)
-{
-	if (Force || (Size <= PCB_MAX_PINORVIASIZE && Size >= PCB_MIN_PINORVIAHOLE && Size <= conf_core.design.via_thickness - PCB_MIN_PINORVIACOPPER)) {
-		rnd_conf_set_design("design/via_drilling_hole", "%$mS", Size);
-	}
-}
-
-/* sets a clearance width */
-void pcb_board_set_clearance(rnd_coord_t Width)
-{
-	if (Width <= PCB_MAX_THICKNESS) {
-		rnd_conf_set_design("design/clearance", "%$mS", Width);
-	}
-}
-
-/* sets a text scaling */
-void pcb_board_set_text_scale(int Scale)
-{
-	if (Scale <= PCB_MAX_TEXTSCALE && Scale >= PCB_MIN_TEXTSCALE) {
-		rnd_conf_set_design("design/text_scale", "%d", Scale);
-	}
-}
-
 /* sets or resets changed flag and redraws status */
 void pcb_board_set_changed_flag(pcb_board_t *pcb, rnd_bool New)
 {
