@@ -98,6 +98,8 @@ struct pcb_board_s {
 	int line_mod_merge_inhibit;
 	struct vtp0_s *line_mod_merge;
 
+	pcb_data_t *uilayer_data;                       /* parent of all UI layers */
+
 	/* random */
 	unsigned suppress_warn_missing_font:1;          /* do not warn for missing font; useful for alien formats not having embedded font */
 };
@@ -148,21 +150,6 @@ void pcb_board_resize(rnd_coord_t Width, rnd_coord_t Height, int undoable);
 /* free the board and remove its undo list */
 void pcb_board_remove(pcb_board_t *Ptr);
 
-/* sets a new line thickness */
-void pcb_board_set_line_width(rnd_coord_t Size);
-
-/* sets a new via thickness */
-void pcb_board_set_via_size(rnd_coord_t Size, rnd_bool Force);
-
-/* sets a new via drilling hole */
-void pcb_board_set_via_drilling_hole(rnd_coord_t Size, rnd_bool Force);
-
-/* sets a clearance width */
-void pcb_board_set_clearance(rnd_coord_t Width);
-
-/* sets a text scaling */
-void pcb_board_set_text_scale(int Scale);
-
 /* make sure the data of the board fits the PCB (move it out from negative,
    make the board large enough); returns -1 on error, the number of changes on success */
 int pcb_board_normalize(pcb_board_t *pcb);
