@@ -149,10 +149,15 @@ typedef struct {
 	} rc;
 
 	const struct {                       /* design: defaults of a new layout */
-		RND_CFT_COORD via_thickness;
-		RND_CFT_COORD via_drilling_hole;
+		/* drawing pen */
+		RND_CFT_COORD via_thickness;           /* [OBSOLETE] used to be via gemoetry for the via tool; replaced by via padstack prototype */
+		RND_CFT_COORD via_drilling_hole;       /* [OBSOLETE] used to be via gemoetry for the via tool; replaced by via padstack prototype */
 		RND_CFT_COORD line_thickness;
 		RND_CFT_COORD clearance;
+		RND_CFT_INTEGER via_proto;             /* prototype used for vias */
+		RND_CFT_INTEGER text_scale;            /* text scaling in % */
+		RND_CFT_COORD text_thickness;          /* override stroke font text thickness */
+		RND_CFT_INTEGER text_font_id;
 
 		/* limiting sizes (old DRC) settings */
 		RND_CFT_COORD bloat;                   /* core: size of box-style clearance around text objects; old plugins: gap between copper features on different nets */
@@ -171,9 +176,6 @@ typedef struct {
 			RND_CFT_BOOLEAN dummy;               /* placeholder, not used */
 		} drc_disable;
 
-		RND_CFT_INTEGER text_scale;            /* text scaling in % */
-		RND_CFT_COORD text_thickness;          /* override stroke font text thickness */
-		RND_CFT_INTEGER text_font_id;
 		RND_CFT_REAL poly_isle_area;           /* polygon min area */
 		RND_CFT_STRING fab_author;             /* Full name of author for FAB drawings */
 		RND_CFT_STRING initial_layer_stack;    /* deprecated. */
@@ -197,6 +199,7 @@ typedef struct {
 		RND_CFT_BOOLEAN black_current_group;   /* render all layers of the current group black, for maximum contrast */
 		RND_CFT_BOOLEAN smart_labels;          /* attempt to place terminal labels in a way they don't overlap (costs extra CPU cycles, may slow down on-screen rendering) */
 		RND_CFT_COORD label_thickness;         /* font thickness for labels (e.g. terminal labels); thinnest possible is 1nm */
+		RND_CFT_STRING render_script;          /* instructs the core about layer order in rendering */
 
 		struct {                           /* color */
 			RND_CFT_COLOR crosshair;             /* obsolete - DO NOT USE - kept for compatibility (use appearance/color/cross instead) */
@@ -221,6 +224,7 @@ typedef struct {
 			RND_CFT_COLOR mask;                  /* default mask layer color (when a new mask layer is created) */
 			RND_CFT_COLOR paste;                 /* default paste layer color (when a new paste layer is created) */
 			RND_CFT_COLOR element;               /* default silk layer color (when a new silk layer is created) */
+			RND_CFT_COLOR black_group;           /* 'black' color used by the 'black current group' feature (set it to white if you use dark background) */
 		} color;
 		struct {
 			RND_CFT_INTEGER cross_thick;      /* cross thickness in pixels - 0 means disable crosses */
