@@ -1046,7 +1046,7 @@ void pcb_crosshair_grid_fit(pcb_board_t *pcb, rnd_coord_t X, rnd_coord_t Y)
 	}
 
 	enfmode  = (rnd_conf.editor.mode == pcb_crosshair.tool_line) && pcb_crosshair.AttachedLine.State != PCB_CH_STATE_FIRST;
-	if (pcb_brave & PCB_BRAVE_ENFORCE_CLR_MOVE)
+	if (!(pcb_brave & PCB_BRAVE_NO_ENFORCE_CLR_MOVE))
 		enfmode |= (rnd_conf.editor.mode == pcb_crosshair.tool_move) && (pcb_crosshair.AttachedObject.Type == PCB_OBJ_LINE_POINT);
 	if (enfmode && conf_core.editor.auto_drc) {
 		rnd_coord_t thick;
@@ -1143,6 +1143,7 @@ void pcb_crosshair_pre_init(void)
 {
 	pcb_crosshair.tool_arrow = pcb_crosshair.tool_move = pcb_crosshair.tool_line = -1;
 	pcb_crosshair.tool_arc = pcb_crosshair.tool_poly = pcb_crosshair.tool_poly_hole = -1;
+	pcb_crosshair.tool_via = pcb_crosshair.tool_text = -1;
 }
 
 void pcb_crosshair_uninit(void)
