@@ -41,6 +41,7 @@
 #include "stub_draw.h"
 #include "layer_ui.h"
 #include <librnd/core/hid_inlines.h>
+#include <librnd/core/conf_hid.h>
 #include "funchash_core.h"
 
 #include "obj_pstk_draw.h"
@@ -162,22 +163,6 @@ void pcb_draw(void)
 	pcb_draw_invalidated.X2 = pcb_draw_invalidated.Y2 = -RND_COORD_MAX;
 }
 
-static void draw_everything_holes(pcb_draw_info_t *info, rnd_layergrp_id_t gid)
-{
-	int plated, unplated;
-	pcb_board_count_holes(PCB, &plated, &unplated, info->drawn_area);
-
-	if (plated && pcb_layer_gui_set_vlayer(PCB, PCB_VLY_PLATED_DRILL, 0, &info->xform_exporter)) {
-		pcb_draw_pstk_holes(info, gid, PCB_PHOLE_PLATED);
-		rnd_render->end_layer(rnd_render);
-	}
-
-	if (unplated && pcb_layer_gui_set_vlayer(PCB, PCB_VLY_UNPLATED_DRILL, 0, &info->xform_exporter)) {
-		pcb_draw_pstk_holes(info, gid, PCB_PHOLE_UNPLATED);
-		rnd_render->end_layer(rnd_render);
-	}
-}
-
 typedef struct {
 	rnd_layergrp_id_t top_fab, top_assy, bot_assy;
 	int top_fab_enable, top_assy_enable, bot_assy_enable;
@@ -289,6 +274,10 @@ static void draw_xor_marks(pcb_draw_info_t *info)
 		pcb_draw_pstk_marks(info);
 	}
 
+	rnd_hid_set_line_cap(pcb_draw_out.fgGC, rnd_cap_round);
+	rnd_hid_set_line_width(pcb_draw_out.fgGC, 0);
+	rnd_render->set_color(pcb_draw_out.fgGC, &conf_core.appearance.color.pin_name);
+
 	rnd_event(&PCB->hidlib, RND_EVENT_GUI_DRAW_OVERLAY_XOR, "pp", &pcb_draw_out.fgGC, info, NULL);
 
 	rnd_hid_set_draw_xor(pcb_draw_out.fgGC, 0);
@@ -338,215 +327,7 @@ static int has_auto(pcb_layergrp_t *grp)
 	return 0;
 }
 
-static void draw_everything(pcb_draw_info_t *info)
-{
-	rnd_layergrp_id_t backsilk_gid;
-	pcb_layergrp_t *backsilk_grp;
-	rnd_color_t old_silk_color[PCB_MAX_LAYERGRP];
-	int i, ngroups;
-	rnd_layergrp_id_t component, solder, gid, side_copper_grp;
-	/* This is the list of layer groups we will draw.  */
-	char do_group[PCB_MAX_LAYERGRP];
-	/* This is the reverse of the order in which we draw them.  */
-	rnd_layergrp_id_t drawn_groups[PCB_MAX_LAYERGRP];
-	rnd_bool paste_empty;
-	legacy_vlayer_t lvly;
-	rnd_xform_t tmp;
-
-	xform_setup(info, &tmp, NULL);
-	backsilk_gid = ((!info->xform->show_solder_side) ? pcb_layergrp_get_bottom_silk() : pcb_layergrp_get_top_silk());
-	backsilk_grp = pcb_get_layergrp(PCB, backsilk_gid);
-	if (backsilk_grp != NULL) {
-		rnd_cardinal_t n;
-		for(n = 0; n < backsilk_grp->len; n++) {
-			pcb_layer_t *ly = pcb_get_layer(PCB->Data, backsilk_grp->lid[n]);
-			if (ly != NULL) {
-				old_silk_color[n] = ly->meta.real.color;
-				ly->meta.real.color = conf_core.appearance.color.invisible_objects;
-			}
-		}
-	}
-
-	rnd_render->render_burst(rnd_render, RND_HID_BURST_START, info->drawn_area);
-
-	memset(do_group, 0, sizeof(do_group));
-	lvly.top_fab = -1;
-	lvly.top_assy = lvly.bot_assy = -1;
-	for (ngroups = 0, i = 0; i < pcb_max_layer(PCB); i++) {
-		pcb_layer_t *l = PCB_STACKLAYER(PCB, i);
-		rnd_layergrp_id_t group = pcb_layer_get_group(PCB, pcb_layer_stack[i]);
-		pcb_layergrp_t *grp = pcb_get_layergrp(PCB, group);
-		unsigned int gflg = 0;
-
-		if (grp != NULL)
-			gflg = grp->ltype;
-
-		if ((gflg & PCB_LYT_DOC) && (grp->purpi == F_fab)) {
-			lvly.top_fab = group;
-			lvly.top_fab_enable = has_auto(grp);
-		}
-
-		if ((gflg & PCB_LYT_DOC) && (grp->purpi == F_assy)) {
-			if (gflg & PCB_LYT_TOP) {
-				lvly.top_assy = group;
-				lvly.top_assy_enable = has_auto(grp);
-			}
-			else if (gflg & PCB_LYT_BOTTOM) {
-				lvly.bot_assy = group;
-				lvly.bot_assy_enable = has_auto(grp);
-			}
-		}
-
-		if ((gflg & PCB_LYT_SILK) || (gflg & PCB_LYT_DOC) || (gflg & PCB_LYT_MASK) || (gflg & PCB_LYT_PASTE) || (gflg & PCB_LYT_BOUNDARY) || (gflg & PCB_LYT_MECH)) /* do not draw silk, mask, paste and boundary here, they'll be drawn separately */
-			continue;
-
-		if (l->meta.real.vis && !do_group[group]) {
-			do_group[group] = 1;
-			drawn_groups[ngroups++] = group;
-		}
-	}
-
-	solder = component = -1;
-	pcb_layergrp_list(PCB, PCB_LYT_BOTTOM | PCB_LYT_COPPER, &solder, 1);
-	pcb_layergrp_list(PCB, PCB_LYT_TOP | PCB_LYT_COPPER, &component, 1);
-	side_copper_grp = info->xform->show_solder_side ? solder : component;
-
-
-	/*
-	 * first draw all 'invisible' stuff
-	 */
-	if ((info->xform_exporter != NULL) && !info->xform_exporter->check_planes && pcb_layer_gui_set_vlayer(PCB, PCB_VLY_INVISIBLE, 0, &info->xform_exporter)) {
-		pcb_layer_type_t side = PCB_LYT_INVISIBLE_SIDE();
-
-		pcb_draw_silk_doc(info, side, PCB_LYT_DOC, 0, 1);
-		pcb_draw_silk_doc(info, side, PCB_LYT_SILK, 0, 1);
-
-		rnd_render->set_drawing_mode(rnd_render, RND_HID_COMP_RESET, pcb_draw_out.direct, info->drawn_area);
-		rnd_render->set_drawing_mode(rnd_render, RND_HID_COMP_POSITIVE, pcb_draw_out.direct, info->drawn_area);
-		rnd_render->set_drawing_mode(rnd_render, RND_HID_COMP_FLUSH, pcb_draw_out.direct, info->drawn_area);
-		rnd_render->end_layer(rnd_render);
-	}
-
-	/* Draw far side doc and silks */
-	{
-		pcb_layer_type_t ivside = PCB_LYT_INVISIBLE_SIDE();
-
-		pcb_draw_silk_doc(info, ivside, PCB_LYT_SILK, 1, 0);
-		pcb_draw_silk_doc(info, ivside, PCB_LYT_DOC, 1, 0);
-	}
-
-	/* draw all layers in layerstack order */
-	for (i = ngroups - 1; i >= 0; i--) {
-		rnd_layergrp_id_t group = drawn_groups[i];
-
-		if (pcb_layer_gui_set_glayer(PCB, group, 0, &info->xform_exporter)) {
-			int is_current = 0;
-			rnd_layergrp_id_t cgrp = PCB_CURRLAYER(PCB)->meta.real.grp;
-
-			if ((cgrp == solder) || (cgrp == component)) {
-				/* current group is top or bottom: visibility depends on side we are looking at */
-				if (group == side_copper_grp)
-					is_current = 1;
-			}
-			else {
-				/* internal layer displayed on top: current group is solid, others are "invisible" */
-				if (group == cgrp)
-					is_current = 1;
-			}
-
-			pcb_draw_layer_grp(info, group, is_current);
-			rnd_render->end_layer(rnd_render);
-		}
-	}
-
-	if (conf_core.editor.check_planes && rnd_render->gui)
-		goto finish;
-
-	/* Draw padstacks below silk */
-	rnd_render->set_drawing_mode(rnd_render, RND_HID_COMP_RESET, pcb_draw_out.direct, info->drawn_area);
-	rnd_render->set_drawing_mode(rnd_render, RND_HID_COMP_POSITIVE, pcb_draw_out.direct, info->drawn_area);
-	if (rnd_render->gui) {
-		rnd_xform_t tmp;
-		xform_setup(info, &tmp, NULL);
-		pcb_draw_ppv(info, info->xform->show_solder_side ? solder : component);
-		info->xform = NULL; info->layer = NULL;
-	}
-	rnd_render->set_drawing_mode(rnd_render, RND_HID_COMP_FLUSH, pcb_draw_out.direct, info->drawn_area);
-
-	/* Draw the solder mask if turned on */
-	gid = pcb_layergrp_get_top_mask();
-	if ((gid >= 0) && (pcb_layer_gui_set_glayer(PCB, gid, 0, &info->xform_exporter))) {
-		pcb_draw_mask(info, PCB_COMPONENT_SIDE);
-		rnd_render->end_layer(rnd_render);
-	}
-
-	gid = pcb_layergrp_get_bottom_mask();
-	if ((gid >= 0) && (pcb_layer_gui_set_glayer(PCB, gid, 0, &info->xform_exporter))) {
-		pcb_draw_mask(info, PCB_SOLDER_SIDE);
-		rnd_render->end_layer(rnd_render);
-	}
-
-	/* Draw doc and silks */
-	{
-		pcb_layer_type_t vside = PCB_LYT_VISIBLE_SIDE();
-
-		pcb_draw_silk_doc(info, PCB_LYT_INTERN, PCB_LYT_SILK, 1, 0);
-		pcb_draw_silk_doc(info, PCB_LYT_INTERN, PCB_LYT_DOC, 1, 0);
-		pcb_draw_silk_doc(info, 0, PCB_LYT_DOC, 1, 0);
-		pcb_draw_silk_doc(info, vside, PCB_LYT_SILK, 1, 0);
-		pcb_draw_silk_doc(info, vside, PCB_LYT_DOC, 1, 0);
-	}
-
-	{ /* holes_after: draw holes after copper, silk and mask, to make sure it punches through everything. */
-		rnd_render->set_drawing_mode(rnd_render, RND_HID_COMP_RESET, pcb_draw_out.direct, info->drawn_area); 
-		rnd_render->set_drawing_mode(rnd_render, RND_HID_COMP_POSITIVE, pcb_draw_out.direct, info->drawn_area);
-		draw_everything_holes(info, side_copper_grp);
-		rnd_render->set_drawing_mode(rnd_render, RND_HID_COMP_FLUSH, pcb_draw_out.direct, info->drawn_area);
-	}
-
-	gid = pcb_layergrp_get_top_paste();
-	if (gid >= 0)
-		paste_empty = pcb_layergrp_is_empty(PCB, gid);
-	if ((gid >= 0) && (pcb_layer_gui_set_glayer(PCB, gid, paste_empty, &info->xform_exporter))) {
-		pcb_draw_paste(info, PCB_COMPONENT_SIDE);
-		rnd_render->end_layer(rnd_render);
-	}
-
-	gid = pcb_layergrp_get_bottom_paste();
-	if (gid >= 0)
-		paste_empty = pcb_layergrp_is_empty(PCB, gid);
-	if ((gid >= 0) && (pcb_layer_gui_set_glayer(PCB, gid, paste_empty, &info->xform_exporter))) {
-		pcb_draw_paste(info, PCB_SOLDER_SIDE);
-		rnd_render->end_layer(rnd_render);
-	}
-
-	pcb_draw_boundary_mech(info);
-
-	draw_virtual_layers(info, &lvly);
-	if ((rnd_render->gui) || (!info->xform_caller->omit_overlay)) {
-		rnd_xform_t tmp;
-		xform_setup(info, &tmp, NULL);
-		draw_rats(info, info->drawn_area);
-		draw_pins_and_pads(info, component, solder);
-	}
-	draw_ui_layers(info);
-
-	if (rnd_render->gui) {
-		rnd_xform_t tmp;
-		xform_setup(info, &tmp, NULL);
-		draw_xor_marks(info);
-	}
-
-	finish:;
-	if (backsilk_grp != NULL) {
-		rnd_cardinal_t n;
-		for(n = 0; n < backsilk_grp->len; n++) {
-			pcb_layer_t *ly = pcb_get_layer(PCB->Data, backsilk_grp->lid[n]);
-			if (ly != NULL)
-				ly->meta.real.color = old_silk_color[n];
-		}
-	}
-}
+#include "draw_everything.c"
 
 static void pcb_draw_pstks(pcb_draw_info_t *info, rnd_layergrp_id_t group, int is_current, pcb_layer_combining_t comb)
 {
@@ -683,11 +464,25 @@ static void pcb_draw_annotations(pcb_draw_info_t *info)
 
 static void xform_setup(pcb_draw_info_t *info, rnd_xform_t *dst, const pcb_layer_t *Layer)
 {
+	int copied = 0;
 	info->layer = Layer;
+
+	/* layer-specific transformations */
 	if ((Layer != NULL) && (!pcb_xform_is_nop(&Layer->meta.real.xform))) {
 		pcb_xform_copy(dst, &Layer->meta.real.xform);
 		info->xform = dst;
+		copied = 1;
 	}
+
+	/* ... plus only in GUI mode: per layer GUI modifications */
+	if ((Layer != NULL) && ((rnd_render == rnd_gui) || dst->add_gui_xform) && (!pcb_xform_is_nop(&Layer->meta.real.gui_xform))) {
+		if (copied)
+			pcb_xform_add(dst, &Layer->meta.real.gui_xform);
+		else
+			pcb_xform_copy(dst, &Layer->meta.real.gui_xform);
+		info->xform = dst;
+	}
+
 	if (info->xform_caller != NULL) {
 		if (info->xform == NULL) {
 			info->xform = dst;
@@ -733,7 +528,7 @@ void pcb_draw_layer(pcb_draw_info_t *info, const pcb_layer_t *Layer_)
 		if (!restore_color) {
 			orig_color = Layer->meta.real.color;
 			restore_color = 1;
-			Layer->meta.real.color = *rnd_color_black;
+			Layer->meta.real.color = conf_core.appearance.color.black_group;
 		}
 	}
 
@@ -1352,3 +1147,30 @@ void pcb_term_label_invalidate(rnd_coord_t x, rnd_coord_t y, double scale, rnd_b
 	pcb_label_invalidate(x, y, scale, vert, centered, label);
 }
 
+static const char draw_cookie[] = "core/draw";
+
+static void draw_chg_render_script(rnd_conf_native_t *cfg, int arr_idx)
+{
+	draw_everything_recompile = conf_core.appearance.render_script;
+}
+
+void pcb_draw_init(void)
+{
+	static rnd_conf_hid_callbacks_t cbs_mode;
+	static rnd_conf_hid_id_t draw_conf_id;
+	rnd_conf_native_t *n_rscr = rnd_conf_get_field("appearance/render_script");
+	draw_conf_id = rnd_conf_hid_reg(draw_cookie, NULL);
+
+	if (n_rscr != NULL) {
+		memset(&cbs_mode, 0, sizeof(rnd_conf_hid_callbacks_t));
+		cbs_mode.val_change_post = draw_chg_render_script;
+		rnd_conf_hid_set_cb(n_rscr, draw_conf_id, &cbs_mode);
+	}
+
+}
+
+void pcb_draw_uninit(void)
+{
+	rnd_event_unbind_allcookie(draw_cookie);
+	rnd_conf_hid_unreg(draw_cookie);
+}
