@@ -142,6 +142,7 @@ action_entry(TermID)
 action_entry(Text)
 action_entry(TextScale)
 action_entry(Thaw)
+action_entry(ThinDraw)
 action_entry(ToLayout)
 action_entry(ToggleAllDirections)
 action_entry(ToggleAutoDRC)
@@ -175,5 +176,6 @@ action_entry(Unselect)
 action_entry(UnFreeze)
 action_entry(Via)
 action_entry(Warp)
+action_entry(WireFrame)
 action_entry(Zoom)
 
