@@ -5,7 +5,7 @@
  *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *  Copyright (C) 1997, 1998, 1999, 2000, 2001 Harry Eaton
- *  Copyright (C) 2019, 2020 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2019, 2020, 2021 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -43,11 +43,13 @@
 #include <librnd/core/error.h>
 #include "undo.h"
 #include "funchash_core.h"
+#include <librnd/core/funchash_core.h>
 #include "change.h"
 
 #include "draw.h"
 #include "search.h"
 #include "find.h"
+#include "data_it.h"
 #include <librnd/core/actions.h>
 #include <librnd/core/hid_init.h>
 #include <librnd/core/compat_misc.h>
@@ -63,8 +65,10 @@
 #include <librnd/core/tool.h>
 #include "route_style.h"
 #include "tool_logic.h"
+#include "obj_text_op.h"
 
-#define CLONE_TYPES PCB_OBJ_LINE | PCB_OBJ_ARC | PCB_OBJ_POLY
+#define CLONE_TYPES \
+	PCB_OBJ_LINE | PCB_OBJ_ARC | PCB_OBJ_POLY | PCB_OBJ_PSTK | PCB_OBJ_TEXT
 
 /* --------------------------------------------------------------------------- */
 /* Toggle actions are kept for compatibility; new code should use the conf system instead */
@@ -504,7 +508,7 @@ static fgw_error_t pcb_act_MarkCrosshair(fgw_arg_t *res, int argc, fgw_arg_t *ar
 /* --------------------------------------------------------------------------- */
 
 static const char pcb_acts_RouteStyle[] = 
-	"RouteStyle(style_id|style_name|@current, [set|get|del], [trace-thickness|trace-clearance|text-thickness|text-scale|name], [value]])\n"
+	"RouteStyle(style_id|style_name|@current, [set|get|del], [trace-thickness|trace-clearance|text-thickness|text-scale|font|via-proto|name], [value]])\n"
 	"RouteStyle(new, [name])";
 static const char pcb_acth_RouteStyle[] = "Without second argument: copies the indicated routing style into the current pen; with second argument sets or gets a field of the routing style.";
 /* DOC: routestyle.html */
@@ -546,7 +550,7 @@ static fgw_error_t pcb_act_RouteStyle(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		}
 	}
 	else
-		number = pcb_route_style_lookup(&PCB->RouteStyle, conf_core.design.line_thickness, conf_core.design.via_thickness, conf_core.design.via_drilling_hole, conf_core.design.clearance, NULL)+1;
+		number = PCB_LOOKUP_ROUTE_STYLE_PEN(PCB)+1;
 
 	if ((number <= 0) || (number > vtroutestyle_len(&PCB->RouteStyle))) {
 		rnd_message(RND_MSG_ERROR, "RouteStyle: invalid route style name or index\n");
@@ -559,10 +563,7 @@ static fgw_error_t pcb_act_RouteStyle(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	switch(*cmd) {
 		case '\0':
 			do_select:;
-			pcb_board_set_line_width(rts->Thick);
-			pcb_board_set_via_size(rts->Diameter, rnd_true);
-			pcb_board_set_via_drilling_hole(rts->Hole, rnd_true);
-			pcb_board_set_clearance(rts->Clearance);
+			pcb_use_route_style(rts);
 			break;
 		case 'd': /* del */
 			res->type = FGW_INT;
@@ -573,20 +574,30 @@ static fgw_error_t pcb_act_RouteStyle(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 
 			if (strcmp(sfield, "trace-thickness") == 0) {
 				RND_ACT_CONVARG(4, FGW_COORD, RouteStyle, c = fgw_coord(&argv[4]));
-				pcb_route_style_change(PCB, number-1, &c, NULL, NULL, NULL, NULL, 1);
+				pcb_route_style_change(PCB, number-1, &c, NULL, NULL, NULL, NULL, NULL, 1);
 			}
 			else if (strcmp(sfield, "text-thickness") == 0) {
 				RND_ACT_CONVARG(4, FGW_COORD, RouteStyle, c = fgw_coord(&argv[4]));
-				pcb_route_style_change(PCB, number-1, NULL, &c, NULL, NULL, NULL, 1);
+				pcb_route_style_change(PCB, number-1, NULL, &c, NULL, NULL, NULL, NULL, 1);
 			}
 			else if (strcmp(sfield, "text-scale") == 0) {
 				int sc;
 				RND_ACT_CONVARG(4, FGW_COORD, RouteStyle, sc = argv[4].val.nat_int);
-				pcb_route_style_change(PCB, number-1, NULL, NULL, &sc, NULL, NULL, 1);
+				pcb_route_style_change(PCB, number-1, NULL, NULL, &sc, NULL, NULL, NULL, 1);
+			}
+			else if (strcmp(sfield, "font") == 0) {
+				long l;
+				RND_ACT_CONVARG(4, FGW_LONG, RouteStyle, l = argv[4].val.nat_long);
+				pcb_route_style_change(PCB, number-1, NULL, NULL, NULL, &l, NULL, NULL, 1);
 			}
 			else if (strcmp(sfield, "trace-clearance") == 0) {
 				RND_ACT_CONVARG(4, FGW_COORD, RouteStyle, c = fgw_coord(&argv[4]));
-				pcb_route_style_change(PCB, number-1, NULL, NULL, NULL, &c, NULL, 1);
+				pcb_route_style_change(PCB, number-1, NULL, NULL, NULL, NULL, &c, NULL, 1);
+			}
+			else if (strcmp(sfield, "via-proto") == 0) {
+				rnd_cardinal_t car;
+				RND_ACT_CONVARG(4, FGW_LONG, RouteStyle, car = argv[4].val.nat_long);
+				pcb_route_style_change(PCB, number-1, NULL, NULL, NULL, NULL, NULL, &car, 1);
 			}
 			else if (strcmp(sfield, "name") == 0) {
 				const char *new_name;
@@ -616,6 +627,10 @@ static fgw_error_t pcb_act_RouteStyle(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 				res->type = FGW_COORD;
 				fgw_coord(res) = rts->Clearance;
 			}
+			else if (strcmp(sfield, "via-proto") == 0) {
+				res->type = FGW_LONG;
+				res->val.nat_long = rts->via_proto;
+			}
 			else if (strcmp(sfield, "name") == 0) {
 				res->type = FGW_STR;
 				res->val.cstr = rts->name;
@@ -645,16 +660,18 @@ static fgw_error_t pcb_act_RouteStyle(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 static const char pcb_acts_SetSame[] = "SetSame()";
 static const char pcb_acth_SetSame[] = "Sets current layer and sizes to match indicated item.";
 /* DOC: setsame.html */
-static void set_same_(rnd_coord_t Thick, rnd_coord_t Diameter, rnd_coord_t Hole, rnd_coord_t Clearance, char *Name)
+static void set_same_(rnd_coord_t Thick, rnd_coord_t textt, int texts, pcb_font_id_t fid, rnd_coord_t Clearance, rnd_cardinal_t via_proto, char *Name)
 {
 	int known;
-	known = pcb_route_style_lookup(&PCB->RouteStyle, Thick, Diameter, Hole, Clearance, Name);
+	known = pcb_route_style_lookup_strict(&PCB->RouteStyle, Thick, textt, texts, fid, Clearance, via_proto, Name);
 	if (known < 0) {
 		/* unknown style, set properties */
-		if (Thick != -1)     { pcb_custom_route_style.Thick     = Thick;     rnd_conf_set_design("design/line_thickness", "%$mS", Thick); }
-		if (Clearance != -1) { pcb_custom_route_style.Clearance = Clearance; rnd_conf_set_design("design/clearance", "%$mS", Clearance); }
-		if (Diameter != -1)  { pcb_custom_route_style.Diameter  = Diameter;  rnd_conf_set_design("design/via_thickness", "%$mS", Diameter); }
-		if (Hole != -1)      { pcb_custom_route_style.Hole      = Hole;      rnd_conf_set_design("design/via_drilling_hole", "%$mS", Hole); }
+		if (Thick != -1)     { rnd_conf_set_design("design/line_thickness", "%$mS", Thick); }
+		if (Clearance != -1) { rnd_conf_set_design("design/clearance", "%$mS", Clearance); }
+		if (textt != -1)     { rnd_conf_set_design("design/text_thickness", "%$mS", textt); }
+		if (texts != -1)     { rnd_conf_set_design("design/text_scale", "%d", texts); }
+		if (fid != -1)       { rnd_conf_set_design("design/text_font_id", "%ld", (long)fid); }
+		if (via_proto != -1) { rnd_conf_set_design("design/via_proto", "%ld", (long)via_proto); }
 		PCB->pen_attr = NULL;
 	}
 	else
@@ -675,7 +692,7 @@ static fgw_error_t pcb_act_SetSame(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	switch (type) {
 	case PCB_OBJ_LINE:
 		rnd_hid_notify_crosshair_change(RND_ACT_HIDLIB, rnd_false);
-		set_same_(((pcb_line_t *) ptr2)->Thickness, -1, -1, ((pcb_line_t *) ptr2)->Clearance / 2, NULL);
+		set_same_(((pcb_line_t *)ptr2)->Thickness, -1, -1, -1, ((pcb_line_t *) ptr2)->Clearance / 2, -1, NULL);
 		layer = (pcb_layer_t *) ptr1;
 		if (rnd_conf.editor.mode != pcb_crosshair.tool_line)
 			rnd_tool_select_by_name(RND_ACT_HIDLIB, "line");
@@ -685,7 +702,7 @@ static fgw_error_t pcb_act_SetSame(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 
 	case PCB_OBJ_ARC:
 		rnd_hid_notify_crosshair_change(RND_ACT_HIDLIB, rnd_false);
-		set_same_(((pcb_arc_t *) ptr2)->Thickness, -1, -1, ((pcb_arc_t *) ptr2)->Clearance / 2, NULL);
+		set_same_(((pcb_arc_t *)ptr2)->Thickness, -1, -1, -1, ((pcb_arc_t *) ptr2)->Clearance / 2, -1, NULL);
 		layer = (pcb_layer_t *) ptr1;
 		if (rnd_conf.editor.mode != pcb_crosshair.tool_arc)
 			rnd_tool_select_by_name(RND_ACT_HIDLIB, "arc");
@@ -697,6 +714,27 @@ static fgw_error_t pcb_act_SetSame(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		layer = (pcb_layer_t *) ptr1;
 		break;
 
+	case PCB_OBJ_PSTK:
+		rnd_hid_notify_crosshair_change(RND_ACT_HIDLIB, rnd_false);
+		set_same_(-1, -1, -1, -1, -1, ((pcb_pstk_t *)ptr2)->proto, NULL);
+		if (rnd_conf.editor.mode != pcb_crosshair.tool_via)
+			rnd_tool_select_by_name(RND_ACT_HIDLIB, "via");
+		rnd_hid_notify_crosshair_change(RND_ACT_HIDLIB, rnd_true);
+		rnd_event(RND_ACT_HIDLIB, PCB_EVENT_ROUTE_STYLES_CHANGED, NULL);
+		break;
+
+	case PCB_OBJ_TEXT:
+	{
+		pcb_text_t *text = ptr2;
+		rnd_hid_notify_crosshair_change(RND_ACT_HIDLIB, rnd_false);
+		set_same_(-1, text->thickness, text->Scale, text->fid, -1, -1, NULL);
+		if (rnd_conf.editor.mode != pcb_crosshair.tool_text)
+			rnd_tool_select_by_name(RND_ACT_HIDLIB, "text");
+		rnd_hid_notify_crosshair_change(RND_ACT_HIDLIB, rnd_true);
+		rnd_event(RND_ACT_HIDLIB, PCB_EVENT_ROUTE_STYLES_CHANGED, NULL);
+		break;
+	}
+
 	default:
 		RND_ACT_IRES(-1);
 		return 0;
@@ -709,6 +747,120 @@ static fgw_error_t pcb_act_SetSame(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	return 0;
 }
 
+static int apply_pen(pcb_board_t *pcb, pcb_any_obj_t *obj)
+{
+	pcb_layer_t *layer = pcb_layer_get_real(obj->parent.layer);
+	int changed = 0;
+
+	switch(obj->type) {
+		case PCB_OBJ_LINE:
+			{
+				pcb_line_t *line = (pcb_line_t *)obj;
+				pcb_line_pre(line);
+				if (line->Thickness != conf_core.design.line_thickness) {
+					pcb_undo_add_obj_to_size(PCB_OBJ_LINE, layer, line, line);
+					line->Thickness = conf_core.design.line_thickness;
+					changed = 1;
+				}
+				if (line->Clearance != conf_core.design.clearance) {
+					pcb_undo_add_obj_to_clear_size(PCB_OBJ_LINE, layer, line, line);
+					line->Clearance = conf_core.design.clearance * 2;
+					changed = 1;
+				}
+				pcb_line_bbox(line);
+				pcb_line_post(line);
+			}
+			break;
+		case PCB_OBJ_ARC:
+			{
+				pcb_arc_t *arc = (pcb_arc_t *)obj;
+				pcb_arc_pre(arc);
+				if (arc->Thickness != conf_core.design.line_thickness) {
+					pcb_undo_add_obj_to_size(PCB_OBJ_ARC, layer, arc, arc);
+					arc->Thickness = conf_core.design.line_thickness;
+					changed = 1;
+				}
+				if (arc->Clearance != conf_core.design.clearance) {
+					pcb_undo_add_obj_to_clear_size(PCB_OBJ_ARC, layer, arc, arc);
+					arc->Clearance = conf_core.design.clearance * 2;
+					changed = 1;
+				}
+				pcb_arc_bbox(arc);
+				pcb_arc_post(arc);
+			}
+			break;
+
+		case PCB_OBJ_TEXT:
+			{
+				pcb_text_t src, *text = (pcb_text_t *)obj;
+				if ((text->fid != conf_core.design.text_font_id) || (text->Scale != conf_core.design.text_scale) || (text->thickness != conf_core.design.text_thickness)) {
+					src = *text;
+					src.fid = conf_core.design.text_font_id;
+					src.Scale = conf_core.design.text_scale;
+					src.thickness = conf_core.design.text_thickness;
+					pcb_text_chg_any(text, &src, 1);
+					changed = 1;
+				}
+			}
+			break;
+
+		case PCB_OBJ_PSTK:
+			{
+				pcb_pstk_t *pstk = (pcb_pstk_t *)obj;
+
+				if ((pstk->proto != conf_core.design.via_proto) || (pstk->Clearance != conf_core.design.clearance)) {
+					rnd_cardinal_t pr = conf_core.design.via_proto;
+					if (pcb_pstk_change_instance(pstk, &pr, &conf_core.design.clearance, NULL, NULL, NULL) == 0)
+						changed = 1;
+				}
+			}
+
+		default:
+			break;
+	}
+
+	return changed;
+}
+
+static const char pcb_acts_ApplyPen[] = "ApplyPen([selected|object])";
+static const char pcb_acth_ApplyPen[] = "Set properties of the selected object or object under the cursor to match current drawing style (\"pen\"), e.g. thickness, clearance.";
+/* DOC: applypen.html */
+static fgw_error_t pcb_act_ApplyPen(fgw_arg_t *res, int argc, fgw_arg_t *argv)
+{
+	rnd_coord_t x, y;
+	void *ptr1, *ptr2, *ptr3;
+	int op = F_Object, type, changed = 0;
+	pcb_any_obj_t *o;
+	pcb_data_it_t it;
+
+	RND_ACT_MAY_CONVARG(1, FGW_KEYWORD, ApplyPen, op = fgw_keyword(&argv[1]));
+
+	switch(op) {
+		case F_Object:
+			rnd_hid_get_coords("Click on an object to change", &x, &y, 0);
+
+			type = pcb_search_screen(x, y, CLONE_TYPES, &ptr1, &ptr2, &ptr3);
+			if (type != 0) {
+				pcb_undo_freeze_serial();
+				changed = apply_pen(PCB_ACT_BOARD, ptr2);
+				pcb_undo_unfreeze_serial();
+			}
+			break;
+		case F_Selected:
+			pcb_undo_freeze_serial();
+			for(o = pcb_data_first(&it, PCB_ACT_BOARD->Data, CLONE_TYPES); o != NULL; o = pcb_data_next(&it))
+				if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, o))
+					changed |= apply_pen(PCB_ACT_BOARD, o);
+			pcb_undo_unfreeze_serial();
+			break;
+	}
+
+	if (changed)
+		pcb_undo_inc_serial();
+	return 0;
+}
+
+
 static const char pcb_acts_EditLayer[] = "Editlayer([@layer], [name=text|auto=[0|1]|sub=[0|1])]\nEditlayer([@layer], attrib, key=value)";
 static const char pcb_acth_EditLayer[] = "Change a property or attribute of a layer. If the first argument starts with @, it is taken as the layer name to manipulate, else the action uses the current layer. Without arguments or if only a layer name is specified, interactive runs editing.";
 static fgw_error_t pcb_act_EditLayer(fgw_arg_t *res, int argc, fgw_arg_t *argv)
@@ -1493,7 +1645,7 @@ static fgw_error_t pcb_act_ChkRst(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	if (rst == NULL)
 		RND_ACT_IRES(-1);
 	else
-		RND_ACT_IRES(pcb_route_style_match(rst, conf_core.design.line_thickness, conf_core.design.via_thickness, conf_core.design.via_drilling_hole, conf_core.design.clearance, NULL));
+		RND_ACT_IRES(PCB_MATCH_ROUTE_STYLE_PEN(rst));
 	return 0;
 }
 
@@ -1576,11 +1728,44 @@ static fgw_error_t pcb_act_LayerVisReset(fgw_arg_t *res, int argc, fgw_arg_t *ar
 	return 0;
 }
 
+#define XFORM_BOOL(ly, how, field) \
+do { \
+	switch(how) { \
+		case F_Set:     ly->meta.real.gui_xform.field = 1; break; \
+		case F_Clear:   ly->meta.real.gui_xform.field = 0; break; \
+		case F_Toggle:  ly->meta.real.gui_xform.field = !ly->meta.real.gui_xform.field; break; \
+		default: return FGW_ERR_ARG_CONV; \
+	} \
+	rnd_event(RND_ACT_HIDLIB, PCB_EVENT_LAYERVIS_CHANGED, NULL); \
+} while(0)
+
+static const char pcb_acts_LayerXform[] = "LayerXform(layerid, toggle|set|clear, xformfield, [value])";
+static const char pcb_acth_LayerXform[] = "Reset layer visibility to safe defaults.";
+static fgw_error_t pcb_act_LayerXform(fgw_arg_t *res, int argc, fgw_arg_t *argv)
+{
+	pcb_layer_t *ly;
+	int how, what;
+
+	RND_ACT_CONVARG(1, FGW_LAYER, LayerXform, ly = fgw_layer(&argv[1]));
+	RND_ACT_CONVARG(2, FGW_KEYWORD, LayerXform, how = fgw_keyword(&argv[2]));
+	RND_ACT_CONVARG(3, FGW_KEYWORD, LayerXform, what = fgw_keyword(&argv[3]));
+
+	switch(what) {
+		case F_WireFrame: XFORM_BOOL(ly, how, wireframe); break;
+		case F_ThinDraw: XFORM_BOOL(ly, how, thin_draw); break;
+		default: return FGW_ERR_ARG_CONV;
+	}
+
+	RND_ACT_IRES(0);
+	return 0;
+}
+
 static rnd_action_t gui_action_list[] = {
 	{"Display", pcb_act_Display, pcb_acth_Display, pcb_acts_Display},
 	{"CycleDrag", pcb_act_CycleDrag, pcb_acth_CycleDrag, pcb_acts_CycleDrag},
 	{"MarkCrosshair", pcb_act_MarkCrosshair, pcb_acth_MarkCrosshair, pcb_acts_MarkCrosshair},
 	{"SetSame", pcb_act_SetSame, pcb_acth_SetSame, pcb_acts_SetSame},
+	{"ApplyPen", pcb_act_ApplyPen, pcb_acth_ApplyPen, pcb_acts_ApplyPen},
 	{"RouteStyle", pcb_act_RouteStyle, pcb_acth_RouteStyle, pcb_acts_RouteStyle},
 	{"SelectLayer", pcb_act_SelectLayer, pcb_acth_selectlayer, pcb_acts_selectlayer},
 	{"ChkLayer", pcb_act_ChkLayer, pcb_acth_chklayer, pcb_acts_chklayer},
@@ -1595,7 +1780,8 @@ static rnd_action_t gui_action_list[] = {
 	{"ChkRst", pcb_act_ChkRst, pcb_acth_chkrst, pcb_acts_chkrst},
 	{"BoardFlip", pcb_act_boardflip, pcb_acth_boardflip, pcb_acts_boardflip},
 	{"ClipInhibit", pcb_act_ClipInhibit, pcb_acth_ClipInhibit, pcb_acts_ClipInhibit},
-	{"LayerVisReset", pcb_act_LayerVisReset, pcb_acth_LayerVisReset, pcb_acts_LayerVisReset}
+	{"LayerVisReset", pcb_act_LayerVisReset, pcb_acth_LayerVisReset, pcb_acts_LayerVisReset},
+	{"LayerXform", pcb_act_LayerXform, pcb_acth_LayerXform, pcb_acts_LayerXform}
 };
 
 void pcb_gui_act_init2(void)
