@@ -122,6 +122,8 @@ static char *main_path_init(char *argv0)
 
 	if (haspath) {
 		bindir = rnd_lrealpath(argv0);
+		if (bindir == NULL)
+			bindir = rnd_strdup(argv0);
 		found_bindir = 1;
 	}
 	else {
@@ -150,6 +152,8 @@ static char *main_path_init(char *argv0)
 					printf("Found it:  \"%s\"\n", tmps);
 #endif
 					bindir = rnd_lrealpath(tmps);
+					if (bindir == NULL)
+						bindir = rnd_strdup(tmps);
 					found_bindir = 1;
 					free(tmps);
 					break;
@@ -296,6 +300,8 @@ extern void pcb_rats_patch_init2(void);
 extern void pcb_remove_act_init2(void);
 extern void pcb_select_act_init2(void);
 extern void pcb_undo_act_init2(void);
+extern void pcb_draw_init(void);
+extern void pcb_draw_uninit(void);
 
 void pcb_main_init_actions(void)
 {
@@ -329,6 +335,8 @@ void pcb_main_uninit(void)
 		rnd_log_last->seen = 1; /* ignore anything unseen before the uninit */
 
 	conf_core_uninit_pre();
+
+	pcb_draw_uninit();
 	pcb_brave_uninit();
 	pcb_polygon_uninit();
 
@@ -432,39 +440,17 @@ int main(int argc, char *argv[])
 
 	rnd_hidlib_init1(conf_core_init);
 
-	{
-		extern const char *lhtpers_early_end[];
-		extern void vti0_init();
-		extern volatile const unsigned char ureglex_nfa_str[];
-		extern int ureglex_strtree_exec();
-		volatile void *p0;
-		volatile void (*p1)();
-		volatile int (*p2)();
-		volatile size_t (*p3)();
-		unsigned char c;
-		volatile void *dummy;
-		
-TODO("librnd: remove this once librnd is not extern anymore");
-		rnd_plugin_add_dir("../src_3rd/librnd-local/src/plugins");
 
-		/* temporary workaround: make sure objects from .a are linked even
-		   if pcb-rnd code didn't need them (some plugins will) */
-		p0 = lhtpers_early_end; dummy = rnd_cast_f2d(p0);
-		p1 = vti0_init; dummy = rnd_cast_f2d(p1);
-		p2 = re_se_exec; dummy = rnd_cast_f2d(p2);
-		p2 = ureglex_strtree_exec; dummy = rnd_cast_f2d(p2);
-		p1 = pup_list_parse_pups; dummy = rnd_cast_f2d(p1);
-		p3 = vtl0_len; dummy = rnd_cast_f2d(p3);
-		c = ureglex_nfa_str[0];
-		
-		(void)c; (void)dummy;
-	}
+TODO("librnd: remove this once librnd is not extern anymore");
+	rnd_plugin_add_dir("../src_3rd/librnd-local/src/plugins");
 
 	pcb_event_init_app();
 	rnd_conf_set(RND_CFR_INTERNAL, "rc/path/exec_prefix", -1, exec_prefix, RND_POL_OVERWRITE);
 	free(exec_prefix);
 
+	pcb_uilayer_init();
 	pcb_layer_vis_init();
+	pcb_draw_init();
 	pcb_brave_init();
 	pcb_pixmap_init();
 
