@@ -59,6 +59,9 @@
 
 #include "obj_poly_draw_helper.c"
 
+TODO("ui_layer parent fix: remove this")
+#include "layer_ui.h"
+
 /*** allocation ***/
 
 void pcb_poly_reg(pcb_layer_t *layer, pcb_poly_t *poly)
@@ -66,10 +69,9 @@ void pcb_poly_reg(pcb_layer_t *layer, pcb_poly_t *poly)
 	polylist_append(&layer->Polygon, poly);
 	PCB_SET_PARENT(poly, layer, layer);
 
-	if (layer->parent_type == PCB_PARENT_UI)
-		return;
-
-	if (layer->parent_type == PCB_PARENT_DATA)
+	if (pcb_is_uilayer(layer))
+		pcb_obj_id_reg(layer->parent.data, poly);
+	else if (layer->parent_type == PCB_PARENT_DATA)
 		pcb_obj_id_reg(layer->parent.data, poly);
 }
 
@@ -80,7 +82,10 @@ void pcb_poly_unreg(pcb_poly_t *poly)
 	pcb_term_del_auto((pcb_any_obj_t *)poly);
 	assert(poly->parent_type == PCB_PARENT_LAYER);
 	polylist_remove(poly);
-	if (layer->parent_type != PCB_PARENT_UI) {
+	if (pcb_is_uilayer(layer)) {
+		pcb_obj_id_del(layer->parent.data, poly);
+	}
+	else {
 		assert(layer->parent_type == PCB_PARENT_DATA);
 		pcb_obj_id_del(layer->parent.data, poly);
 	}
@@ -117,19 +122,29 @@ void pcb_poly_free(pcb_poly_t *poly)
 	free(poly);
 }
 
+/* realloc new memory if necessary and clear it */
+RND_INLINE void pcb_poly_point_prealloc_(pcb_poly_t *Polygon, long num_pt, long step)
+{
+	if (num_pt >= Polygon->PointMax) {
+		long diff, old = Polygon->PointMax;
+
+		Polygon->PointMax = num_pt + step;
+		diff = Polygon->PointMax - old;
+		Polygon->Points = realloc(Polygon->Points, Polygon->PointMax * sizeof(rnd_point_t));
+		memset(Polygon->Points + old, 0, diff * sizeof(rnd_point_t));
+	}
+}
+
+void pcb_poly_point_prealloc(pcb_poly_t *Polygon, long num_pt)
+{
+	pcb_poly_point_prealloc_(Polygon, num_pt, 0);
+}
+
 /* gets the next slot for a point in a polygon struct, allocates memory if necessary */
 rnd_point_t *pcb_poly_point_alloc(pcb_poly_t *Polygon)
 {
-	rnd_point_t *points = Polygon->Points;
-
-	/* realloc new memory if necessary and clear it */
-	if (Polygon->PointN >= Polygon->PointMax) {
-		Polygon->PointMax += STEP_POLYGONPOINT;
-		points = (rnd_point_t *) realloc(points, Polygon->PointMax * sizeof(rnd_point_t));
-		Polygon->Points = points;
-		memset(points + Polygon->PointN, 0, STEP_POLYGONPOINT * sizeof(rnd_point_t));
-	}
-	return (points + Polygon->PointN++);
+	pcb_poly_point_prealloc_(Polygon, Polygon->PointN, STEP_POLYGONPOINT);
+	return (Polygon->Points + Polygon->PointN++);
 }
 
 /* gets the next slot for a point in a polygon struct, allocates memory if necessary */
@@ -1438,14 +1453,17 @@ void pcb_poly_draw_(pcb_draw_info_t *info, pcb_poly_t *polygon, int allow_term_g
 			rnd_hid_set_line_cap(pcb_draw_out.fgGC, rnd_cap_square);
 			for(n = head, i = 0; (n != head) || (i == 0); n = n->next, i++) {
 				rnd_coord_t x, y, r;
-				x = (n->prev->point[0] + n->point[0] + n->next->point[0]) / 3;
-				y = (n->prev->point[1] + n->point[1] + n->next->point[1]) / 3;
+				rnd_vector_t v;
 
-TODO("subc: check if x;y is within the poly, but use a cheaper method than the official")
-				r = PCB_DRAW_TERM_GFX_WIDTH;
-				rnd_hid_set_line_width(pcb_draw_out.fgGC, r);
-				rnd_hid_set_line_cap(pcb_draw_out.fgGC, rnd_cap_square);
-				rnd_render->draw_line(pcb_draw_out.fgGC, x, y, x, y);
+				v[0] = x = (n->prev->point[0] + n->point[0] + n->next->point[0]) / 3;
+				v[1] = y = (n->prev->point[1] + n->point[1] + n->next->point[1]) / 3;
+
+				if (rnd_poly_contour_inside(polygon->Clipped->contours, v)) {
+					r = PCB_DRAW_TERM_GFX_WIDTH;
+					rnd_hid_set_line_width(pcb_draw_out.fgGC, r);
+					rnd_hid_set_line_cap(pcb_draw_out.fgGC, rnd_cap_square);
+					rnd_render->draw_line(pcb_draw_out.fgGC, x, y, x, y);
+				}
 			}
 		}
 		else
