@@ -34,7 +34,6 @@
 #include "buffer.h"
 #include "board.h"
 #include "crosshair.h"
-#include "brave.h"
 #include "data.h"
 #include "data_it.h"
 #include <librnd/core/error.h>
@@ -170,8 +169,6 @@ pcb_layer_t *pcb_subc_layer_create(pcb_subc_t *sc, const char *name, pcb_layer_t
 	pcb_layer_t *dst = &sc->data->Layer[sc->data->LayerN++];
 
 	dst->is_bound = 1;
-	dst->meta.bound.user_specified = 0;
-	dst->meta.bound.user_lid = -1;
 	dst->meta.bound.type = type;
 	dst->comb = comb;
 	dst->meta.bound.stack_offs = stack_offs;
@@ -1132,7 +1129,7 @@ void *pcb_subc_op(pcb_data_t *Data, pcb_subc_t *sc, pcb_opfunc_t *opfunc, pcb_op
 {
 	int n;
 
-	if (!(pcb_brave & PCB_BRAVE_NOCLIPBATCH) && (Data != NULL))
+	if (Data != NULL)
 		pcb_data_clip_inhibit_inc(Data);
 	pcb_subc_part_changed_inhibit_inc(sc);
 
@@ -1261,7 +1258,7 @@ void *pcb_subc_op(pcb_data_t *Data, pcb_subc_t *sc, pcb_opfunc_t *opfunc, pcb_op
 		case PCB_SUBCOP_UNDO_NORMAL: break;
 	}
 
-	if (!(pcb_brave & PCB_BRAVE_NOCLIPBATCH) && (Data != NULL))
+	if (Data != NULL)
 		pcb_data_clip_inhibit_dec(Data, 0);
 
 	return sc;
