@@ -4,7 +4,7 @@
  *  pcb-rnd, interactive printed circuit board design
  *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
- *  Copyright (C) 2018,2019,2020 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2018,2019,2020,2021 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -55,6 +55,9 @@
 #include "obj_text_draw.h"
 #include "conf_core.h"
 
+TODO("ui_layer parent fix: remove this")
+#include "layer_ui.h"
+
 /*** allocation ***/
 
 void pcb_text_reg(pcb_layer_t *layer, pcb_text_t *text)
@@ -62,10 +65,9 @@ void pcb_text_reg(pcb_layer_t *layer, pcb_text_t *text)
 	textlist_append(&layer->Text, text);
 	PCB_SET_PARENT(text, layer, layer);
 
-	if (layer->parent_type == PCB_PARENT_UI)
-		return;
-
-	if (layer->parent_type == PCB_PARENT_DATA)
+	if (pcb_is_uilayer(layer))
+		pcb_obj_id_reg(layer->parent.data, text);
+	else if (layer->parent_type == PCB_PARENT_DATA)
 		pcb_obj_id_reg(layer->parent.data, text);
 }
 
@@ -74,7 +76,9 @@ void pcb_text_unreg(pcb_text_t *text)
 	pcb_layer_t *layer = text->parent.layer;
 	assert(text->parent_type == PCB_PARENT_LAYER);
 	textlist_remove(text);
-	if (layer->parent_type != PCB_PARENT_UI) {
+	if (pcb_is_uilayer(layer)) {
+		pcb_obj_id_del(layer->parent.data, text);
+	} else {
 		assert(layer->parent_type == PCB_PARENT_DATA);
 		pcb_obj_id_del(layer->parent.data, text);
 	}
@@ -123,6 +127,7 @@ typedef struct {
 	rnd_coord_t thickness;
 	rnd_coord_t clearance;
 	double rot;
+	pcb_font_id_t fid;
 } undo_text_geo_t;
 
 static int undo_text_geo_swap(void *udata)
@@ -143,6 +148,7 @@ static int undo_text_geo_swap(void *udata)
 	rnd_swap(rnd_coord_t, g->thickness, g->text->thickness);
 	rnd_swap(rnd_coord_t, g->clearance, g->text->clearance);
 	rnd_swap(double, g->rot, g->text->rot);
+	rnd_swap(pcb_font_id_t, g->fid, g->text->fid);
 
 	if (pcb != NULL)
 		pcb_text_bbox(pcb_font(pcb, g->text->fid, 1), g->text);
@@ -222,8 +228,8 @@ pcb_text_t *pcb_text_new_scaled(pcb_layer_t *Layer, pcb_font_t *PCBFont, rnd_coo
 
 pcb_text_t *pcb_text_new_by_bbox(pcb_layer_t *Layer, pcb_font_t *PCBFont, rnd_coord_t X, rnd_coord_t Y, rnd_coord_t bbw, rnd_coord_t bbh, rnd_coord_t anchx, rnd_coord_t anchy, double scxy, pcb_text_mirror_t mirror, double rot, rnd_coord_t thickness, const char *TextString, pcb_flag_t Flags)
 {
-	rnd_coord_t obw, obh, nbw, nbh, nanchx, nanchy;
-	double gsc, gscx, gscy, cs, sn, mx = 1, my = 1;
+	rnd_coord_t obw, obh, nbw, nbh;
+	double gsc, gscx, gscy, cs, sn, mx = 1, my = 1, nanchx, nanchy;
 	pcb_text_t *t;
 	
 	t = pcb_text_new_(Layer, PCBFont, 0, 0, 0, mirror, 100, 1, 1, thickness, TextString, Flags);
@@ -269,8 +275,8 @@ pcb_text_t *pcb_text_new_by_bbox(pcb_layer_t *Layer, pcb_font_t *PCBFont, rnd_co
 	cs = cos(rot*mx*my / RND_RAD_TO_DEG);
 	sn = sin(rot*mx*my / RND_RAD_TO_DEG);
 
-	t->X = X - (nanchx * cs * mx + nanchy * sn * my);
-	t->Y = Y - (nanchy * cs * my - nanchx * sn * mx);
+	t->X = rnd_round((double)X - (nanchx * cs * mx + nanchy * sn * my));
+	t->Y = rnd_round((double)Y - (nanchy * cs * my - nanchx * sn * mx));
 	t->rot = rot;
 
 	/*rnd_trace(" final: %ml %ml (%f %f -> %f) got:%f wanted:%f anch: %ml %ml -> %ml %ml\n", t->bbox_naked.X2 - t->bbox_naked.X1, t->bbox_naked.Y2 - t->bbox_naked.Y1, gscx, gscy, gsc, t->scale_x/t->scale_y, scxy, anchx, anchy, nanchx, nanchy);*/
@@ -294,6 +300,7 @@ static pcb_text_t *pcb_text_copy_meta(pcb_text_t *dst, pcb_text_t *src)
 pcb_text_t *pcb_text_dup(pcb_layer_t *dst, pcb_text_t *src)
 {
 	pcb_text_t *t = pcb_text_new_scaled(dst, pcb_font(PCB, src->fid, 1), src->X, src->Y, src->rot, text_mirror_bits(src), src->Scale, src->scale_x, src->scale_y, src->thickness, src->TextString, src->Flags);
+	t->clearance = src->clearance;
 	pcb_text_copy_meta(t, src);
 	return t;
 }
@@ -301,6 +308,7 @@ pcb_text_t *pcb_text_dup(pcb_layer_t *dst, pcb_text_t *src)
 pcb_text_t *pcb_text_dup_at(pcb_layer_t *dst, pcb_text_t *src, rnd_coord_t dx, rnd_coord_t dy)
 {
 	pcb_text_t *t = pcb_text_new_scaled(dst, pcb_font(PCB, src->fid, 1), src->X+dx, src->Y+dy, src->rot, text_mirror_bits(src), src->Scale, src->scale_x, src->scale_y, src->thickness, src->TextString, src->Flags);
+	t->clearance = src->clearance;
 	pcb_text_copy_meta(t, src);
 	return t;
 }
@@ -333,7 +341,10 @@ static int pcb_text_render_str_cb(void *ctx, gds_t *s, const char **input)
 		const pcb_attribute_list_t *attr = &text->Attributes;
 		path = key+2;
 		if ((path[0] == 'p') && (memcmp(path, "parent.", 7) == 0)) {
-			pcb_data_t *par = text->parent.layer->parent.data;
+			pcb_data_t *par = NULL;
+
+			if (text->parent.layer != NULL)
+				par = text->parent.layer->parent.data;
 
 			if (par == NULL)
 				attr = NULL;
@@ -599,6 +610,7 @@ void *pcb_textop_add_to_buffer(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t
 	pcb_layer_t *layer = &ctx->buffer.dst->Layer[pcb_layer_id(ctx->buffer.src, Layer)];
 	pcb_text_t *t = pcb_text_new_scaled(layer, pcb_font(PCB, Text->fid, 1), Text->X, Text->Y, Text->rot, text_mirror_bits(Text), Text->Scale, Text->scale_x, Text->scale_y, Text->thickness, Text->TextString, pcb_flag_mask(Text->Flags, ctx->buffer.extraflg));
 
+	t->clearance = Text->clearance;
 	pcb_text_copy_meta(t, Text);
 	if (ctx->buffer.keep_id) pcb_obj_change_id((pcb_any_obj_t *)t, Text->ID);
 	return t;
@@ -660,6 +672,37 @@ void *pcb_textop_change_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 	return NULL;
 }
 
+/* changes the clearance size of a text */
+void *pcb_textop_change_clear_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *text)
+{
+	pcb_layer_t *layer = text->parent.layer;
+	pcb_board_t *pcb = pcb_data_get_top(layer->parent.data);
+	rnd_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : text->clearance + ctx->chgsize.value;
+
+	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, text) || !PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, text))
+		return NULL;
+	if (value < 0)
+		value = 0;
+	value = MIN(PCB_MAX_THICKNESS, value);
+	if (!ctx->chgsize.is_absolute && ctx->chgsize.value < 0)
+		value = 0;
+	if (value != text->clearance) {
+		pcb_undo_add_obj_to_clear_size(PCB_OBJ_TEXT, Layer, text, text);
+		pcb_poly_restore_to_poly(ctx->chgsize.pcb->Data, PCB_OBJ_TEXT, Layer, text);
+		pcb_text_invalidate_erase(Layer, text);
+		rnd_r_delete_entry(Layer->text_tree, (rnd_box_t *)text);
+		text->clearance = value;
+		if (pcb != NULL)
+			pcb_text_bbox(pcb_font(pcb, text->fid, 1), text);
+		rnd_r_insert_entry(Layer->text_tree, (rnd_box_t *)text);
+		pcb_poly_clear_from_poly(ctx->chgsize.pcb->Data, PCB_OBJ_TEXT, Layer, text);
+		pcb_text_invalidate_draw(Layer, text);
+		return text;
+	}
+	return NULL;
+}
+
+
 /* changes the thickness of a text object */
 void *pcb_textop_change_2nd_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 {
@@ -770,6 +813,7 @@ void *pcb_textop_copy(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 											 Text->Y + ctx->copy.DeltaY, Text->rot, text_mirror_bits(Text), Text->Scale, Text->scale_x, Text->scale_y, Text->thickness, Text->TextString, pcb_flag_mask(Text->Flags, PCB_FLAG_FOUND));
 	if (ctx->copy.keep_id)
 		text->ID = Text->ID;
+	text->clearance = Text->clearance;
 	pcb_text_copy_meta(text, Text);
 	pcb_text_invalidate_draw(Layer, text);
 	pcb_undo_add_obj_to_create(PCB_OBJ_TEXT, Layer, text, text);
@@ -989,6 +1033,7 @@ void pcb_text_mirror_coords(pcb_text_t *text, rnd_coord_t y_offs, rnd_bool undoa
 	g->thickness = text->thickness;
 	g->clearance = text->clearance;
 	g->rot = text->rot;
+	g->fid = text->fid;
 
 	undo_text_geo_swap(g);
 	if (undoable) pcb_undo_inc_serial();
@@ -1155,6 +1200,30 @@ int pcb_text_chg_scale(pcb_text_t *text, double scx, rnd_bool absx, double scy,
 	g->thickness = text->thickness;
 	g->clearance = text->clearance;
 	g->rot = text->rot;
+	g->fid = text->fid;
+
+	undo_text_geo_swap(g);
+	if (undoable) pcb_undo_inc_serial();
+
+	return 0;
+}
+
+int pcb_text_chg_any(pcb_text_t *text, const pcb_text_t *src, rnd_bool undoable)
+{
+	undo_text_geo_t gtmp, *g = &gtmp;
+
+	if (undoable) g = pcb_undo_alloc(pcb_data_get_top(text->parent.layer->parent.data), &undo_text_geo, sizeof(undo_text_geo_t));
+
+	g->text = text;
+	g->X = src->X;
+	g->Y = src->Y;
+	g->Scale = src->Scale;
+	g->scale_x = src->scale_x;
+	g->scale_y = src->scale_y;
+	g->thickness = src->thickness;
+	g->clearance = src->clearance;
+	g->rot = src->rot;
+	g->fid = src->fid;
 
 	undo_text_geo_swap(g);
 	if (undoable) pcb_undo_inc_serial();
@@ -1555,10 +1624,23 @@ void pcb_text_draw_(pcb_draw_info_t *info, pcb_text_t *text, rnd_coord_t min_lin
 	}
 }
 
+rnd_coord_t pcb_text_min_thickness(const pcb_layer_t *layer)
+{
+	unsigned int flg = 0;
+
+	if ((!layer->is_bound) && (layer->meta.real.grp >= 0))
+		flg = pcb_layergrp_flags(PCB, layer->meta.real.grp);
+
+	if (flg & PCB_LYT_SILK)
+		return conf_core.design.min_slk;
+	else
+		return conf_core.design.min_wid;
+
+}
+
 static void pcb_text_draw(pcb_draw_info_t *info, pcb_text_t *text, int allow_term_gfx)
 {
 	int min_silk_line;
-	unsigned int flg = 0;
 	const pcb_layer_t *layer = info->layer != NULL ? info->layer : pcb_layer_get_real(text->parent.layer);
 
 	pcb_obj_noexport(info, text, return);
@@ -1586,13 +1668,7 @@ static void pcb_text_draw(pcb_draw_info_t *info, pcb_text_t *text, int allow_ter
 	else
 		rnd_render->set_color(pcb_draw_out.fgGC, &layer->meta.real.color);
 
-	if ((!layer->is_bound) && (layer->meta.real.grp >= 0))
-		flg = pcb_layergrp_flags(PCB, layer->meta.real.grp);
-
-	if (flg & PCB_LYT_SILK)
-		min_silk_line = conf_core.design.min_slk;
-	else
-		min_silk_line = conf_core.design.min_wid;
+	min_silk_line = pcb_text_min_thickness(layer);
 
 	pcb_text_draw_(info, text, min_silk_line, allow_term_gfx, PCB_TXT_TINY_CHEAP);
 }
