@@ -4,7 +4,7 @@
  *  pcb-rnd, interactive printed circuit board design
  *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996,2004,2006 Thomas Nau
- *  Copyright (C) 2016 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2016,2021 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,134 +40,17 @@
 #include "conf_core.h"
 #include "event.h"
 
-pcb_route_style_t pcb_custom_route_style;
-
 static const char rst_cookie[] = "core route style";
 
-static rnd_coord_t pcb_get_num(char **s, const char *default_unit)
-{
-	/* Read value */
-	rnd_coord_t ret_val = rnd_get_value_ex(*s, NULL, NULL, NULL, default_unit, NULL);
-	/* Advance pointer */
-	while (isalnum(**s) || **s == '.')
-		(*s)++;
-	return ret_val;
-}
-
-
-/* Serializes the route style list */
-char *pcb_route_string_make(vtroutestyle_t *styles)
-{
-	gds_t str;
-	int i;
-
-	gds_init(&str);
-	for (i = 0; i < vtroutestyle_len(styles); ++i) {
-		rnd_append_printf(&str, "%s,%mc,%mc,%mc,%mc", styles->array[i].name,
-																				 styles->array[i].Thick, styles->array[i].Diameter,
-																				 styles->array[i].Hole, styles->array[i].Clearance);
-		if (i > 0)
-			gds_append(&str, ':');
-	}
-	return str.array; /* this is the only allocation made, return this and don't uninit */
-}
-
-/* parses the routes definition string which is a colon separated list of
-   comma separated Name, Dimension, Dimension, Dimension, Dimension
-   e.g. Signal,20,40,20,10:Power,40,60,28,10:... */
-int pcb_route_string_parse1(char **str, pcb_route_style_t *routeStyle, const char *default_unit)
-{
-	char *s = *str;
-	char Name[256];
-	int i, len;
-
-	while (*s && isspace((int) *s))
-		s++;
-	for (i = 0; *s && *s != ','; i++)
-		Name[i] = *s++;
-	Name[i] = '\0';
-	len = strlen(Name);
-	if (len > sizeof(routeStyle->name)-1) {
-		memcpy(routeStyle->name, Name, sizeof(routeStyle->name)-1);
-		routeStyle->name[sizeof(routeStyle->name)-1] = '\0';
-		rnd_message(RND_MSG_WARNING, "Route style name '%s' too long, truncated to '%s'\n", Name, routeStyle->name);
-	}
-	else
-		strcpy(routeStyle->name, Name);
-	if (!isdigit((int) *++s))
-		goto error;
-	routeStyle->Thick = pcb_get_num(&s, default_unit);
-	while (*s && isspace((int) *s))
-		s++;
-	if (*s++ != ',')
-		goto error;
-	while (*s && isspace((int) *s))
-		s++;
-	if (!isdigit((int) *s))
-		goto error;
-	routeStyle->Diameter = pcb_get_num(&s, default_unit);
-	while (*s && isspace((int) *s))
-		s++;
-	if (*s++ != ',')
-		goto error;
-	while (*s && isspace((int) *s))
-		s++;
-	if (!isdigit((int) *s))
-		goto error;
-	routeStyle->Hole = pcb_get_num(&s, default_unit);
-	/* for backwards-compatibility, we use a 10-mil default
-	   for styles which omit the clearance specification. */
-	if (*s != ',')
-		routeStyle->Clearance = RND_MIL_TO_COORD(10);
-	else {
-		s++;
-		while (*s && isspace((int) *s))
-			s++;
-		if (!isdigit((int) *s))
-			goto error;
-		routeStyle->Clearance = pcb_get_num(&s, default_unit);
-		while (*s && isspace((int) *s))
-			s++;
-	}
-
-	*str = s;
-	return 0;
-	error:;
-		*str = s;
-		return -1;
-}
-
-int pcb_route_string_parse(char *s, vtroutestyle_t *styles, const char *default_unit)
-{
-	int n;
-
-	vtroutestyle_truncate(styles, 0);
-	for(n = 0;;n++) {
-		vtroutestyle_enlarge(styles, n+1);
-		if (pcb_route_string_parse1(&s, &styles->array[n], default_unit) != 0) {
-			n--;
-			break;
-		}
-		while (*s && isspace((int) *s))
-			s++;
-		if (*s == '\0')
-			break;
-		if (*s++ != ':') {
-			vtroutestyle_truncate(styles, 0);
-			return -1;
-		}
-	}
-	vtroutestyle_truncate(styles, n+1);
-	return 0;
-}
-
 void pcb_use_route_style(pcb_route_style_t * rst)
 {
 	rnd_conf_set_design("design/line_thickness", "%$mS", rst->Thick);
 	rnd_conf_set_design("design/text_scale", "%d", rst->texts <= 0 ? 100 : rst->texts);
 	rnd_conf_set_design("design/text_thickness", "%$mS", rst->textt);
-	rnd_conf_set_design("design/via_thickness", "%$mS", rst->Diameter);
-	rnd_conf_set_design("design/via_drilling_hole", "%$mS", rst->Hole);
+	if (rst->fid != -1)
+		rnd_conf_set_design("design/text_font_id", "%ld", rst->fid);
+	if (rst->via_proto_set)
+		rnd_conf_set_design("design/via_proto", "%ld", (long)rst->via_proto);
 	rnd_conf_set_design("design/clearance", "%$mS", rst->Clearance);
 	PCB->pen_attr = &rst->attr;
 }
@@ -181,29 +64,57 @@ int pcb_use_route_style_idx(vtroutestyle_t *styles, int idx)
 }
 
 #define cmp(a,b) (((a) != -1) && (coord_abs((a)-(b)) > 32))
+#define cmpi0(a,b) (((a) > 0) && (strict || ((b) > 0)) && ((a) != (b)))
+#define cmpi(a,b) (((a) != -1) && (strict || ((b) != -1)) && ((a) != (b)))
 #define cmps(a,b) (((a) != NULL) && (strcmp((a), (b)) != 0))
-int pcb_route_style_match(pcb_route_style_t *rst, rnd_coord_t Thick, rnd_coord_t Diameter, rnd_coord_t Hole, rnd_coord_t Clearance, char *Name)
+RND_INLINE int pcb_route_style_match_(pcb_route_style_t *rst, int strict, rnd_coord_t Thick, rnd_coord_t textt, int texts, pcb_font_id_t fid, rnd_coord_t Clearance, rnd_cardinal_t via_proto, char *Name)
 {
 	if (cmp(Thick, rst->Thick)) return 0;
-	if (cmp(Diameter, rst->Diameter)) return 0;
-	if (cmp(Hole, rst->Hole)) return 0;
+	if (cmp(textt, rst->textt)) return 0;
+	if (cmpi0(texts, rst->texts)) return 0;
+	if (cmpi(fid, rst->fid)) return 0;
+	if (cmpi(via_proto, rst->via_proto)) return 0;
 	if (cmp(Clearance, rst->Clearance)) return 0;
 	if (cmps(Name, rst->name)) return 0;
 	return 1;
 }
+
+int pcb_route_style_match(pcb_route_style_t *rst, rnd_coord_t Thick, rnd_coord_t textt, int texts, pcb_font_id_t fid, rnd_coord_t Clearance, rnd_cardinal_t via_proto, char *Name)
+{
+	return pcb_route_style_match_(rst, 0, Thick, textt, texts, fid, Clearance, via_proto, Name);
+}
+
 #undef cmp
+#undef cmpi
+#undef cmpi0
 #undef cmps
 
-int pcb_route_style_lookup(vtroutestyle_t *styles, rnd_coord_t Thick, rnd_coord_t Diameter, rnd_coord_t Hole, rnd_coord_t Clearance, char *Name)
+int pcb_route_style_lookup(vtroutestyle_t *styles, rnd_coord_t Thick, rnd_coord_t textt, int texts, pcb_font_id_t fid, rnd_coord_t Clearance, rnd_cardinal_t via_proto, char *Name)
 {
 	int n;
 	for (n = 0; n < vtroutestyle_len(styles); n++)
-		if (pcb_route_style_match(&styles->array[n], Thick, Diameter, Hole, Clearance, Name))
+		if (pcb_route_style_match_(&styles->array[n], 0, Thick, textt, texts, fid, Clearance, via_proto, Name))
 			return n;
 	return -1;
 }
 
 
+int pcb_route_style_lookup_strict(vtroutestyle_t *styles, rnd_coord_t Thick, rnd_coord_t textt, int texts, pcb_font_id_t fid, rnd_coord_t Clearance, rnd_cardinal_t via_proto, char *Name)
+{
+	int n;
+	for (n = 0; n < vtroutestyle_len(styles); n++)
+		if (pcb_route_style_match_(&styles->array[n], 1, Thick, textt, texts, fid, Clearance, via_proto, Name))
+			return n;
+	return -1;
+}
+
+int pcb_lookup_route_style_pen_bestfit(pcb_board_t *pcb)
+{
+	int res = PCB_LOOKUP_ROUTE_STYLE_PEN_STRICT(pcb);
+	if (res == -1) res = PCB_LOOKUP_ROUTE_STYLE_PEN(pcb);
+	return res;
+}
+
 int pcb_get_style_size(int funcid, rnd_coord_t * out, int type, int size_id)
 {
 	switch (funcid) {
@@ -224,10 +135,10 @@ int pcb_get_style_size(int funcid, rnd_coord_t * out, int type, int size_id)
 	case F_SelectedObjects:
 	case F_Selected:
 	case F_SelectedElements:
-		if (size_id == 0)
-			*out = conf_core.design.via_thickness;
-		else if (size_id == 1)
-			*out = conf_core.design.via_drilling_hole;
+		if ((size_id == 0) || (size_id == 1)) {
+			rnd_message(RND_MSG_ERROR, "pcb_get_style_size(Selected*) for size_id %d is not supported (padstack prototypes don't have diameter)\n", size_id);
+			*out = 0;
+		}
 		else
 			*out = conf_core.design.clearance;
 		break;
@@ -303,7 +214,7 @@ static const uundo_oper_t undo_rst = {
 };
 
 
-int pcb_route_style_change(pcb_board_t *pcb, int rstidx, rnd_coord_t *thick, rnd_coord_t *textt, int *texts, rnd_coord_t *clearance, rnd_cardinal_t *via_proto, rnd_bool undoable)
+int pcb_route_style_change(pcb_board_t *pcb, int rstidx, rnd_coord_t *thick, rnd_coord_t *textt, int *texts, pcb_font_id_t *fid, rnd_coord_t *clearance, rnd_cardinal_t *via_proto, rnd_bool undoable)
 {
 	undo_rst_t gtmp, *g = &gtmp;
 	pcb_route_style_t *rst = vtroutestyle_get(&pcb->RouteStyle, rstidx, 0);
@@ -320,6 +231,7 @@ int pcb_route_style_change(pcb_board_t *pcb, int rstidx, rnd_coord_t *thick, rnd
 	if (thick != NULL)      g->rst.Thick = *thick;
 	if (textt != NULL)      g->rst.textt = *textt;
 	if (texts != NULL)      g->rst.texts = *texts;
+	if (fid != NULL)        g->rst.fid = *fid;
 	if (clearance != NULL)  g->rst.Clearance = *clearance;
 	if (via_proto != NULL) {
 		g->rst.via_proto = *via_proto;
@@ -372,8 +284,7 @@ int pcb_route_style_new(pcb_board_t *pcb, const char *name, rnd_bool undoable)
 	g->rst.textt = conf_core.design.text_thickness;
 	g->rst.texts = conf_core.design.text_scale;
 	g->rst.Clearance = conf_core.design.clearance;
-	g->rst.Diameter = conf_core.design.via_thickness*2;
-	g->rst.Hole = conf_core.design.via_drilling_hole;
+	g->rst.fid = -1; /* leave font unset so saving in old lihata format won't trigger a warning */
 
 	undo_rst_swap(g);
 	if (undoable) pcb_undo_inc_serial();
