@@ -32,6 +32,7 @@
 #include <liblihata/tree.h>
 #include <stdarg.h>
 #include <librnd/config.h>
+#include <librnd/core/anyload.h>
 #include <librnd/core/globalconst.h>
 #include <librnd/core/color.h>
 #include <librnd/core/conf.h>
@@ -46,13 +47,15 @@
 #include <librnd/core/file_loaded.h>
 #include <librnd/core/hidlib.h>
 #include <librnd/core/fptr_cast.h>
+#include <librnd/core/safe_fs_dir.h>
 
 /* conf list node's name */
 const char *pcb_conf_list_name = "pcb-rnd-conf-v1";
 static const char *flcat = "conf";
 
+static const char conf_cookie_al[] = "core/conf/anyload";
+
 /* plugin config files and interns */
-static htsi_t conf_files;
 static htsi_t conf_interns;
 static int conf_files_inited = 0;
 
@@ -269,7 +272,8 @@ static int conf_load_plug_files(rnd_conf_role_t role, const char *dir)
 {
 	char path[RND_PATH_MAX], *fn;
 	int dlen, cnt = 0;
-	htsi_entry_t *e;
+	DIR *d;
+	struct dirent *e;
 
 	if (pcb_conf_plug_root[role] != NULL) {
 		lht_dom_uninit(pcb_conf_plug_root[role]);
@@ -278,13 +282,22 @@ static int conf_load_plug_files(rnd_conf_role_t role, const char *dir)
 
 	if (!conf_files_inited) return 0;
 
+	d = rnd_opendir(NULL, dir);
+	if (d == NULL) return 0;
+
 	dlen = strlen(dir);
 	memcpy(path, dir, dlen);
 	path[dlen] = '/';
 	fn = path+dlen+1;
 
-	for (e = htsi_first(&conf_files); e; e = htsi_next(&conf_files, e)) {
-		strcpy(fn, e->key);
+	for(e = rnd_readdir(d); e != NULL; e = rnd_readdir(d)) {
+		int len = strlen(e->d_name);
+
+		if ((e->d_name[0] == '.') || (len < 5) || strcmp(e->d_name + len - 5, ".conf") != 0)
+			continue;
+
+		strcpy(fn, e->d_name);
+
 		if (rnd_file_readable(path)) {
 			lht_doc_t *d = conf_load_plug_file(path, 0);
 			if (d != NULL) {
@@ -299,6 +312,8 @@ static int conf_load_plug_files(rnd_conf_role_t role, const char *dir)
 		}
 	}
 
+	rnd_closedir(d);
+
 	return cnt;
 }
 
@@ -2192,6 +2207,30 @@ int rnd_conf_print_native(rnd_conf_pfn pfn, void *ctx, const char * prefix, int
 }
 
 
+/****************/
+static int conf_anyload_subtree(const rnd_anyload_t *al, rnd_hidlib_t *hl, lht_node_t *root)
+{
+	lht_doc_t *doc;
+	int res;
+
+	/* copy the root to a new doc, keep file name for at least the root node */
+	doc = lht_dom_init();
+	doc->root = lht_dom_duptree(root);
+	lht_dom_loc_newfile(doc, root->file_name);
+	doc->root->file_name = doc->active_file;
+	doc->root->doc = doc;
+	res = conf_merge_plug(doc, RND_CFR_USER, doc->root->file_name);
+	if (!res) {
+		rnd_message(RND_MSG_ERROR, "menu anyload: failed to load menu patch from %s\n", doc->root->file_name);
+		return -1;
+	}
+	pcb_conf_lht_dirty[RND_CFR_USER]++;
+	rnd_anyload_conf_needs_update = 1;
+	return 0;
+}
+
+static rnd_anyload_t conf_anyload = {0};
+
 /****************/
 
 void rnd_conf_init(void)
@@ -2204,11 +2243,21 @@ void rnd_conf_init(void)
 	rnd_conf_reset(RND_CFR_binary, "<pcb_conf_init>");
 }
 
+void rnd_conf_init2(void)
+{
+	conf_anyload.load_subtree = conf_anyload_subtree;
+	conf_anyload.cookie = conf_cookie_al;
+	rnd_anyload_reg("^[a-z]*-rnd-conf-v[0-9]*$", &conf_anyload);
+	rnd_anyload_reg("^rnd-conf-v[0-9]*$", &conf_anyload);
+}
+
 void rnd_conf_uninit(void)
 {
 	int n;
 	htsp_entry_t *e;
 
+	rnd_anyload_unreg_by_cookie(conf_cookie_al);
+
 	rnd_conf_fields_foreach(e) {
 		if (strncmp(e->key, "plugins/", 8) == 0)
 			fprintf(stderr, "pcb-rnd conf ERROR: conf node '%s' is not unregistered\n", e->key);
