@@ -222,10 +222,9 @@ struct rnd_hid_s {
 	   HID supports.  In GUI mode, the print/export dialogs use this to
 	   set up the selectable options.  In command line mode, these are
 	   used to interpret command line options.  If n_ret_ is non-NULL,
-	   the number of attributes is stored there.
-	   The caller must not free the returned table but may modify the
-	   default values in it. */
-	rnd_export_opt_t *(*get_export_options)(rnd_hid_t *hid, int *n_ret);
+	   the number of attributes is stored there. Note: the table is read-only
+	   and persistent. */
+	const rnd_export_opt_t *(*get_export_options)(rnd_hid_t *hid, int *n_ret);
 
 	/* Exports (or print) the current PCB.  The options given represent
 	   the choices made from the options returned from
@@ -315,15 +314,11 @@ struct rnd_hid_s {
 	void (*fill_polygon_offs)(rnd_hid_gc_t gc, int n_coords, rnd_coord_t *x, rnd_coord_t *y, rnd_coord_t dx, rnd_coord_t dy);
 	void (*fill_rect)(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1, rnd_coord_t x2, rnd_coord_t y2);
 
-	void (*draw_pixmap)(rnd_hid_t *hid, rnd_coord_t cx, rnd_coord_t cy, rnd_coord_t sx, rnd_coord_t sy, const rnd_pixmap_t *pixmap);
+	void (*draw_pixmap)(rnd_hid_t *hid, rnd_coord_t cx, rnd_coord_t cy, rnd_coord_t sx, rnd_coord_t sy, rnd_pixmap_t *pixmap);
 	void (*uninit_pixmap)(rnd_hid_t *hid, rnd_pixmap_t *pixmap);
 
-	/* This is for the printer. If xval_ and yval_ are
-	   zero, a calibration page is printed with instructions for
-	   calibrating your printer.  After calibrating, nonzero xval_ and
-	   yval_ are passed according to the instructions.  Metric is nonzero
-	   if the user prefers metric units, else inches are used. */
-	void (*calibrate)(rnd_hid_t *hid, double xval, double yval);
+	/* OBSOLETE: used to be printer calibration */
+	void (*obsolete_calibrate)(rnd_hid_t *hid, double xval, double yval);
 
 
 	/*** GUI layout functions.  Not used or defined for print/export HIDs. ***/
