@@ -370,6 +370,8 @@ extern void rnd_tool_act_init2(void);
 extern void rnd_gui_act_init2(void);
 extern void rnd_main_act_init2(void);
 extern void rnd_menu_act_init2(void);
+extern void rnd_anyload_init2(void);
+extern void rnd_conf_init2(void);
 
 void rnd_hidlib_init2(const pup_buildin_t *buildins, const pup_buildin_t *local_buildins)
 {
@@ -413,6 +415,7 @@ void rnd_hidlib_init2(const pup_buildin_t *buildins, const pup_buildin_t *local_
 	rnd_gui_act_init2();
 	rnd_main_act_init2();
 	rnd_menu_act_init2();
+	rnd_conf_init2();
 
 	/* plugins: buildins */
 	pup_buildin_load(&rnd_pup, buildins);
@@ -423,6 +426,8 @@ void rnd_hidlib_init2(const pup_buildin_t *buildins, const pup_buildin_t *local_
 	rnd_conf_load_extra(NULL, NULL);
 	rnd_units_init();
 	rnd_funchash_init();
+
+	rnd_anyload_init2(); /* must be at the end because any init2() may register an anyload */
 }
 
 static void print_pup_err(pup_err_stack_t *entry, char *string)
@@ -452,6 +457,7 @@ void rnd_hidlib_init3_auto(void)
 
 
 extern void rnd_menu_uninit(void);
+extern void rnd_anyload_uninit(void);
 extern void rnd_hid_cfg_keys_uninit_module(void);
 
 
@@ -467,12 +473,13 @@ void rnd_hidlib_uninit(void)
 		rnd_conf_save_file(NULL, NULL, NULL, RND_CFR_USER, NULL);
 
 	rnd_hid_uninit();
-	rnd_events_uninit();
 	rnd_conf_uninit();
 	rnd_plugin_uninit();
 	rnd_actions_uninit();
 	rnd_dad_unit_uninit();
 	rnd_hid_cfg_keys_uninit_module();
+	rnd_anyload_uninit();
+	rnd_events_uninit();
 }
 
 /* parse arguments using the gui; if fails and fallback is enabled, try the next gui */
