@@ -92,9 +92,9 @@ static rnd_bool map_color(const rnd_color_t *inclr, pcb_gtk_color_t *color)
 	if (color->red || color->green || color->blue)
 		gdk_colormap_free_colors(colormap, color, 1);
 
-	color->red = (unsigned)inclr->r << 8;
-	color->green = (unsigned)inclr->g << 8;
-	color->blue = (unsigned)inclr->b << 8;
+	color->red = pcb_gtk_color8to16(inclr->r);
+	color->green = pcb_gtk_color8to16(inclr->g);
+	color->blue = pcb_gtk_color8to16(inclr->b);
 	gdk_color_alloc(colormap, color);
 
 	return TRUE;
@@ -582,6 +582,12 @@ static void ghid_gdk_set_color(rnd_hid_gc_t gc, const rnd_color_t *color)
 	}
 }
 
+/* gc width in pixels */
+#define GCWP(gc) ((gc->width < 0) ? -gc->width : Vz(gc->width))
+
+/* gc width in coords */
+#define GCWC(gc) ((gc->width < 0) ?  (-gc->width * ghidgui->port.view.coord_per_px) : gc->width)
+
 static void ghid_gdk_set_line_cap(rnd_hid_gc_t gc, rnd_cap_style_t style)
 {
 	switch (style) {
@@ -599,25 +605,18 @@ static void ghid_gdk_set_line_cap(rnd_hid_gc_t gc, rnd_cap_style_t style)
 		gc->join = GDK_JOIN_ROUND;
 	}
 	if (gc->pixel_gc)
-		gdk_gc_set_line_attributes(gc->pixel_gc, Vz(gc->width), GDK_LINE_SOLID, (GdkCapStyle) gc->cap, (GdkJoinStyle) gc->join);
+		gdk_gc_set_line_attributes(gc->pixel_gc, GCWP(gc), GDK_LINE_SOLID, (GdkCapStyle) gc->cap, (GdkJoinStyle) gc->join);
 }
 
 static void ghid_gdk_set_line_width(rnd_hid_gc_t gc, rnd_coord_t width)
 {
 	/* If width is negative then treat it as pixel width, otherwise it is world coordinates. */
-	if(width < 0) {
-		gc->width = -width;
-		width = -width;
-	}
-	else {
-		gc->width = width;
-		width = Vz(width);
-	}
+	gc->width = width;
 
 	if (gc->pixel_gc)
-		gdk_gc_set_line_attributes(gc->pixel_gc, width, GDK_LINE_SOLID, (GdkCapStyle) gc->cap, (GdkJoinStyle) gc->join);
+		gdk_gc_set_line_attributes(gc->pixel_gc, GCWP(gc), GDK_LINE_SOLID, (GdkCapStyle) gc->cap, (GdkJoinStyle) gc->join);
 	if (gc->clip_gc)
-		gdk_gc_set_line_attributes(gc->clip_gc, width, GDK_LINE_SOLID, (GdkCapStyle) gc->cap, (GdkJoinStyle) gc->join);
+		gdk_gc_set_line_attributes(gc->clip_gc, GCWP(gc), GDK_LINE_SOLID, (GdkCapStyle) gc->cap, (GdkJoinStyle) gc->join);
 }
 
 static void ghid_gdk_set_draw_xor(rnd_hid_gc_t gc, int xor_mask)
@@ -699,7 +698,7 @@ static void ghid_gdk_draw_line(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1,
 
 	/* optimization: draw a single dot if object is too small */
 	if ((gc->core_gc.width > 0) && (pcb_gtk_1dot(gc->width, x1, y1, x2, y2))) {
-		if (pcb_gtk_dot_in_canvas(gc->width, dx1, dy1)) {
+		if (pcb_gtk_dot_in_canvas(GCWP(gc), dx1, dy1)) {
 			USE_GC(gc);
 			gdk_draw_point(priv->out_pixel, priv->pixel_gc, dx1, dy1);
 			if (priv->out_clip != NULL)
@@ -711,7 +710,7 @@ static void ghid_gdk_draw_line(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1,
 	dx2 = Vx((double) x2);
 	dy2 = Vy((double) y2);
 
-	if (!pcb_line_clip(0, 0, ghidgui->port.view.canvas_width, ghidgui->port.view.canvas_height, &dx1, &dy1, &dx2, &dy2, gc->width / ghidgui->port.view.coord_per_px))
+	if (!pcb_line_clip(0, 0, ghidgui->port.view.canvas_width, ghidgui->port.view.canvas_height, &dx1, &dy1, &dx2, &dy2, GCWC(gc)))
 		return;
 
 	USE_GC(gc);
@@ -779,7 +778,7 @@ static void ghid_gdk_draw_rect(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1,
 
 	assert((curr_drawing_mode == RND_HID_COMP_POSITIVE) || (curr_drawing_mode == RND_HID_COMP_POSITIVE_XOR) || (curr_drawing_mode == RND_HID_COMP_NEGATIVE));
 
-	lw = gc->width;
+	lw = GCWC(gc);
 	w = ghidgui->port.view.canvas_width * ghidgui->port.view.coord_per_px;
 	h = ghidgui->port.view.canvas_height * ghidgui->port.view.coord_per_px;
 
@@ -794,7 +793,7 @@ static void ghid_gdk_draw_rect(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1,
 
 	/* optimization: draw a single dot if object is too small */
 	if (pcb_gtk_1dot(gc->width, x1, y1, x2, y2)) {
-		if (pcb_gtk_dot_in_canvas(gc->width, sx1, sy1)) {
+		if (pcb_gtk_dot_in_canvas(GCWP(gc), sx1, sy1)) {
 			USE_GC(gc);
 			gdk_draw_point(priv->out_pixel, priv->pixel_gc, sx1, sy1);
 		}
@@ -1028,7 +1027,7 @@ static void ghid_gdk_fill_rect(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1,
 
 	assert((curr_drawing_mode == RND_HID_COMP_POSITIVE) || (curr_drawing_mode == RND_HID_COMP_POSITIVE_XOR) || (curr_drawing_mode == RND_HID_COMP_NEGATIVE));
 
-	lw = gc->width;
+	lw = GCWC(gc);
 	w = ghidgui->port.view.canvas_width * ghidgui->port.view.coord_per_px;
 	h = ghidgui->port.view.canvas_height * ghidgui->port.view.coord_per_px;
 
@@ -1043,7 +1042,7 @@ static void ghid_gdk_fill_rect(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1,
 
 	/* optimization: draw a single dot if object is too small */
 	if (pcb_gtk_1dot(gc->width, x1, y1, x2, y2)) {
-		if (pcb_gtk_dot_in_canvas(gc->width, sx1, sy1)) {
+		if (pcb_gtk_dot_in_canvas(GCWP(gc), sx1, sy1)) {
 			USE_GC_NOPEN(gc);
 			gdk_draw_point(priv->out_pixel, priv->pixel_gc, sx1, sy1);
 		}
@@ -1387,11 +1386,13 @@ static void ghid_gdk_drawing_area_configure_hook(void *vport)
 static void ghid_gdk_screen_update(void)
 {
 	render_priv_t *priv = ghidgui->port.render_priv;
-	GdkWindow *window = gtkc_widget_get_window(ghidgui->port.drawing_area);
+	GdkWindow *window;
 
-	if (priv->base_pixel == NULL)
+	if ((priv->base_pixel == NULL) || (ghidgui->port.drawing_area == NULL))
 		return;
 
+	window = gtkc_widget_get_window(ghidgui->port.drawing_area);
+
 	gdk_draw_drawable(window, priv->bg_gc, priv->base_pixel, 0, 0, 0, 0, ghidgui->port.view.canvas_width, ghidgui->port.view.canvas_height);
 	show_crosshair(TRUE);
 }
