@@ -277,16 +277,9 @@ static void ShowCrosshair(rnd_bool show)
 	lesstif_notify_mark_change(&lesstif_hid, rnd_true);
 }
 
-/* This is the size of the current PCB work area.  */
-/* Use ltf_hidlib->size_x, ltf_hidlib->size_y.  */
-/* static int pcb_width, pcb_height; */
-static int use_private_colormap = 0;
-static int stdin_listen = 0;
-static char *background_image_file = 0;
-
-rnd_export_opt_t lesstif_attribute_list[] = {
+const rnd_export_opt_t lesstif_attribute_list[] = {
 	{"install", "Install private colormap",
-		RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, &use_private_colormap},
+		RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_colormap 0
 
 /* %start-doc options "22 lesstif GUI Options"
@@ -297,7 +290,7 @@ Listen for actions on stdin.
 %end-doc
 */
 	{"listen", "Listen on standard input for actions",
-		RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, &stdin_listen},
+		RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_listen 1
 
 /* %start-doc options "22 lesstif GUI Options"
@@ -310,12 +303,14 @@ automatically scaled to fit the canvas.
 %end-doc
 */
 	{"bg-image", "Background Image",
-		RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, &background_image_file},
+		RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_bg_image 2
 };
 
-TODO("menu: pcb-menu should be generic and not depend on the HID")
 
+#define NUM_OPTIONS ((sizeof(lesstif_attribute_list) / sizeof(lesstif_attribute_list[0])))
+
+rnd_hid_attr_val_t ltf_values[NUM_OPTIONS];
 
 static int lesstif_direct = 0;
 static rnd_composite_op_t lesstif_drawing_mode = 0;
@@ -856,7 +851,7 @@ static void LoadBackgroundImage(const char *fn)
 
 /* ---------------------------------------------------------------------- */
 
-static rnd_export_opt_t *lesstif_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *lesstif_get_export_options(rnd_hid_t *hid, int *n)
 {
 	if (n != NULL)
 		*n = sizeof(lesstif_attribute_list) / sizeof(rnd_export_opt_t);
@@ -1590,8 +1585,8 @@ static void lesstif_do_export(rnd_hid_t *hid, rnd_hid_attr_val_t *options)
 	ltf_topwin_make_bottom();
 
 
-	if ((background_image_file != NULL) && (*background_image_file != '\0'))
-		LoadBackgroundImage(background_image_file);
+	if ((ltf_values[HA_bg_image].str != NULL) && (*ltf_values[HA_bg_image].str != '\0'))
+		LoadBackgroundImage(ltf_values[HA_bg_image].str);
 
 	XtRealizeWidget(appwidget);
 	pcb_ltf_winplace(display, XtWindow(appwidget), "top", 640, 480);
@@ -1720,7 +1715,7 @@ static int lesstif_parse_arguments(rnd_hid_t *hid, int *argc, char ***argv)
 
 	for (ha = rnd_hid_attr_nodes; ha; ha = ha->next)
 		for (i = 0; i < ha->n; i++) {
-			rnd_export_opt_t *a = ha->opts + i;
+			const rnd_export_opt_t *a = ha->opts + i;
 			switch (a->type) {
 			case RND_HATT_INTEGER:
 			case RND_HATT_COORD:
@@ -1756,7 +1751,8 @@ static int lesstif_parse_arguments(rnd_hid_t *hid, int *argc, char ***argv)
 
 	for (ha = rnd_hid_attr_nodes; ha; ha = ha->next)
 		for (i = 0; i < ha->n; i++) {
-			rnd_export_opt_t *a = ha->opts + i;
+			const rnd_export_opt_t *a = ha->opts + i;
+			rnd_hid_attr_val_t *val = &ltf_values[i];
 			XrmOptionDescRec *o = new_options + acount;
 			char *tmpopt, *tmpres;
 			XtResource *r = new_resources + rcount;
@@ -1798,35 +1794,35 @@ static int lesstif_parse_arguments(rnd_hid_t *hid, int *argc, char ***argv)
 				r->resource_type = XtRInt;
 				r->default_type = XtRInt;
 				r->resource_size = sizeof(int);
-				r->default_addr = &(a->default_val.lng);
+				r->default_addr = &(val->lng);
 				rcount++;
 				break;
 			case RND_HATT_COORD:
 				r->resource_type = XmStrCast(XtRPCBCoord);
 				r->default_type = XmStrCast(XtRPCBCoord);
 				r->resource_size = sizeof(rnd_coord_t);
-				r->default_addr = &(a->default_val.crd);
+				r->default_addr = &(val->crd);
 				rcount++;
 				break;
 			case RND_HATT_REAL:
 				r->resource_type = XmStrCast(XtRDouble);
 				r->default_type = XmStrCast(XtRDouble);
 				r->resource_size = sizeof(double);
-				r->default_addr = &(a->default_val.dbl);
+				r->default_addr = &(val->dbl);
 				rcount++;
 				break;
 			case RND_HATT_STRING:
 				r->resource_type = XtRString;
 				r->default_type = XtRString;
 				r->resource_size = sizeof(char *);
-				r->default_addr = (char *) a->default_val.str;
+				r->default_addr = (char *) val->str;
 				rcount++;
 				break;
 			case RND_HATT_BOOL:
 				r->resource_type = XtRBoolean;
 				r->default_type = XtRInt;
 				r->resource_size = sizeof(int);
-				r->default_addr = &(a->default_val.lng);
+				r->default_addr = &(val->lng);
 				rcount++;
 				break;
 			default:
@@ -1872,42 +1868,33 @@ static int lesstif_parse_arguments(rnd_hid_t *hid, int *argc, char ***argv)
 	rcount = 0;
 	for (ha = rnd_hid_attr_nodes; ha; ha = ha->next)
 		for (i = 0; i < ha->n; i++) {
-			rnd_export_opt_t *a = ha->opts + i;
+			const rnd_export_opt_t *a = ha->opts + i;
 			val_union *v = new_values + rcount;
+			rnd_hid_attr_val_t *backup = NULL;
+			if (ha->hid != NULL)
+				backup = ha->hid->argument_array;
+
+			assert(backup != NULL);
+
 			switch (a->type) {
 			case RND_HATT_INTEGER:
-				if (a->value)
-					*(int *) a->value = v->i;
-				else
-					a->default_val.lng = v->i;
+				backup[i].lng = v->i;
 				rcount++;
 				break;
 			case RND_HATT_COORD:
-				if (a->value)
-					*(rnd_coord_t *) a->value = v->c;
-				else
-					a->default_val.crd = v->c;
+				backup[i].crd = v->c;
 				rcount++;
 				break;
 			case RND_HATT_BOOL:
-				if (a->value)
-					*(char *) a->value = v->i;
-				else
-					a->default_val.lng = v->i;
+				backup[i].lng = v->i;
 				rcount++;
 				break;
 			case RND_HATT_REAL:
-				if (a->value)
-					*(double *) a->value = v->f;
-				else
-					a->default_val.dbl = v->f;
+				backup[i].dbl = v->f;
 				rcount++;
 				break;
 			case RND_HATT_STRING:
-				if (a->value)
-					*(char **) a->value = v->s;
-				else
-					a->default_val.str = v->s;
+				backup[i].str = v->s;
 				rcount++;
 				break;
 			default:
@@ -1918,13 +1905,13 @@ static int lesstif_parse_arguments(rnd_hid_t *hid, int *argc, char ***argv)
 	rnd_hid_parse_command_line(argc, argv);
 
 	/* redefine lesstif_colormap, if requested via "-install" */
-	if (use_private_colormap) {
+	if (ltf_values[HA_colormap].lng) {
 		lesstif_colormap = XCopyColormapAndFree(display, lesstif_colormap);
 		XtVaSetValues(appwidget, XtNcolormap, lesstif_colormap, NULL);
 	}
 
 	/* listen on standard input for actions */
-	if (stdin_listen) {
+	if (ltf_values[HA_listen].lng) {
 		XtAppAddInput(app_context, rnd_fileno(stdin), (XtPointer) XtInputReadMask, lesstif_listener_cb, NULL);
 	}
 	return 0;
@@ -2632,11 +2619,6 @@ static void lesstif_fill_rect(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1, r
 		XFillRectangle(display, mask_bitmap, mask_gc, x1, y1, x2 - x1 + 1, y2 - y1 + 1);
 }
 
-static void lesstif_calibrate(rnd_hid_t *hid, double xval, double yval)
-{
-	CRASH("lesstif_calibrate");
-}
-
 static int lesstif_shift_is_pressed(rnd_hid_t *hid)
 {
 	return shift_pressed;
@@ -3060,7 +3042,6 @@ int pplg_init_hid_lesstif(void)
 	lesstif_hid.fill_polygon_offs = lesstif_fill_polygon_offs;
 	lesstif_hid.fill_rect = lesstif_fill_rect;
 
-	lesstif_hid.calibrate = lesstif_calibrate;
 	lesstif_hid.shift_is_pressed = lesstif_shift_is_pressed;
 	lesstif_hid.control_is_pressed = lesstif_control_is_pressed;
 	lesstif_hid.mod1_is_pressed = lesstif_mod1_is_pressed;
@@ -3122,6 +3103,8 @@ int pplg_init_hid_lesstif(void)
 
 	lesstif_hid.get_dad_hidlib = ltf_attr_get_dad_hidlib;
 
+	lesstif_hid.argument_array = ltf_values;
+
 	rnd_hid_register_hid(&lesstif_hid);
 	if (lesstif_conf_id < 0)
 		lesstif_conf_id = rnd_conf_hid_reg(lesstif_cookie, &ccb);
@@ -3134,7 +3117,7 @@ static int lesstif_attrs_regd = 0;
 static void lesstif_reg_attrs(void)
 {
 	if (!lesstif_attrs_regd)
-		rnd_export_register_opts(lesstif_attribute_list, sizeof(lesstif_attribute_list)/sizeof(lesstif_attribute_list[0]), lesstif_cookie, 0);
+		rnd_export_register_opts2(&lesstif_hid, lesstif_attribute_list, sizeof(lesstif_attribute_list)/sizeof(lesstif_attribute_list[0]), lesstif_cookie, 0);
 	lesstif_attrs_regd = 1;
 }
 
