@@ -37,14 +37,13 @@ static gint ghid_port_window_enter_cb(GtkWidget *widget, GdkEventCrossing *ev, v
 
 	/* See comment in ghid_port_window_leave_cb() */
 
-	if (ev->mode != GDK_CROSSING_NORMAL && ev->detail != GDK_NOTIFY_NONLINEAR)
+	if (ev->mode != GDK_CROSSING_NORMAL && ev->mode != GDK_CROSSING_UNGRAB && ev->detail != GDK_NOTIFY_NONLINEAR)
 		return FALSE;
 
-	if (!gctx->topwin.cmd.command_entry_status_line_active) {
-		out->view.has_entered = TRUE;
-		force_update = 1; /* force a redraw for the crosshair */
-		gtk_widget_grab_focus(out->drawing_area); /* Make sure drawing area has keyboard focus when we are in it. */
-	}
+	out->view.has_entered = TRUE;
+	force_update = 1; /* force a redraw for the crosshair */
+	if (!gctx->topwin.cmd.command_entry_status_line_active)
+		gtk_widget_grab_focus(out->drawing_area); /* Make sure drawing area has keyboard focus when we are in it (don't steal keys from the command line, tho) */
 
 	/* Following expression is true if a you open a menu from the menu bar,
 	   move the mouse to the viewport and click on it. This closes the menu
@@ -243,11 +242,6 @@ int gtkhid_parse_arguments(rnd_hid_t *hid, int *argc, char ***argv)
 	return 0;
 }
 
-static void ghid_calibrate(rnd_hid_t *hid, double xval, double yval)
-{
-	printf("ghid_calibrate() -- not implemented\n");
-}
-
 static void ghid_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y, int action)
 {
 	pcb_gtk_t *gctx = hid->hid_data;
@@ -488,6 +482,8 @@ static int ghid_open_popup(rnd_hid_t *hid, const char *menupath)
 		return 1;
 	}
 
+	gctx->port.view.panning = 0; /* corner case: on a popup gtk won't deliver button releases because the popup takes focus; it's safer to turn off panning */
+
 	gtk_widget_grab_focus(gctx->port.drawing_area);
 	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time());
 	gtk_window_set_transient_for(GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(menu))), GTK_WINDOW(gtk_widget_get_toplevel(gctx->port.drawing_area)));
@@ -613,7 +609,7 @@ static void ghid_uninit_pixmap_(rnd_hid_t *hid, rnd_pixmap_t *pxm)
 	free(pxm->hid_data);
 }
 
-static void ghid_draw_pixmap(rnd_hid_t *hid, rnd_coord_t cx, rnd_coord_t cy, rnd_coord_t sx, rnd_coord_t sy, const rnd_pixmap_t *pixmap)
+static void ghid_draw_pixmap(rnd_hid_t *hid, rnd_coord_t cx, rnd_coord_t cy, rnd_coord_t sx, rnd_coord_t sy, rnd_pixmap_t *pixmap)
 {
 	pcb_gtk_t *gctx = hid->hid_data;
 
@@ -652,7 +648,6 @@ void ghid_glue_hid_init(rnd_hid_t *dst)
 	dst->iterate = ghid_iterate;
 	dst->parse_arguments = gtkhid_parse_arguments;
 
-	dst->calibrate = ghid_calibrate;
 	dst->shift_is_pressed = ghid_shift_is_pressed;
 	dst->control_is_pressed = ghid_control_is_pressed;
 	dst->mod1_is_pressed = ghid_mod1_is_pressed;
