@@ -85,7 +85,7 @@ int qparse4(const char *input, char **argv_ret[], unsigned int *argv_allocated,
 	char *buff;
 	size_t buff_len, buff_used;
 	char **argv;
-	int num_fparens = 0;
+	int num_fparens = 0, implicit_empty = 0;
 
 	if (argv_allocated == NULL) {
 		argv           = NULL;
@@ -121,6 +121,7 @@ int qparse4(const char *input, char **argv_ret[], unsigned int *argv_allocated,
 				}
 				break;
 			case qp_normal:
+				implicit_empty = 0;
 				switch (*s) {
 					case '"':
 						if (flg & QPARSE_DOUBLE_QUOTE)
@@ -135,8 +136,11 @@ int qparse4(const char *input, char **argv_ret[], unsigned int *argv_allocated,
 							qpush(*s);
 						break;
 					case ')':
-						if ((flg & QPARSE_PAREN_FUNC) && (num_fparens == 1))
-							goto stop;
+						if ((flg & QPARSE_PAREN_FUNC) && (num_fparens == 1)) {
+							qnext();
+							s++;
+							goto stop2;
+						}
 						qpush(*s);  /* plain ')', don't care */
 						break;
 					case '(':
@@ -157,8 +161,11 @@ int qparse4(const char *input, char **argv_ret[], unsigned int *argv_allocated,
 						break;
 
 					case ';':
-						if (flg & QPARSE_TERM_SEMICOLON)
-							goto stop;
+						if (flg & QPARSE_TERM_SEMICOLON) {
+							qnext();
+							s++;
+							goto stop2;
+						}
 						qpush(*s);  /* plain ';', don't care */
 						break;
 
@@ -175,6 +182,7 @@ int qparse4(const char *input, char **argv_ret[], unsigned int *argv_allocated,
 					case '\t':
 						if (flg & QPARSE_MULTISEP)
 							while(isspace(s[1])) s++;
+						implicit_empty = 1;
 						qnext();
 						break;
 					default:
@@ -238,6 +246,14 @@ int qparse4(const char *input, char **argv_ret[], unsigned int *argv_allocated,
 
 	qnext();
 
+	stop2:;
+
+	/* Corner case: input has stray whitespace at the end - that shouldn't be
+	   an extra empty argv; but if it's explicit (quoted or function syntax),
+	   keep it! */
+	if (implicit_empty)
+		argc--;
+
 	if (buffer == NULL) {
 		if (buff != NULL)
 			free(buff);
