@@ -34,6 +34,7 @@
 #include <librnd/core/hid_inlines.h>
 #include <librnd/core/hid_dad.h>
 #include <librnd/core/hid_dad_tree.h>
+#include <librnd/core/compat_fs.h>
 #include <librnd/core/safe_fs.h>
 #include "obj_pstk.h"
 #include "obj_pstk_inlines.h"
@@ -185,14 +186,16 @@ static void pstklib_expose(rnd_hid_attribute_t *attrib, rnd_hid_preview_t *prv,
 	y2 = ctx->drawbox.Y2;
 
 	grid = ctx->dlg[ctx->wgrid].val.crd;
-	for(x = 0; x < x2; x += grid)
-		rnd_render->draw_line(gc, x, y1, x, y2);
-	for(x = -grid; x > x1; x -= grid)
-		rnd_render->draw_line(gc, x, y1, x, y2);
-	for(y = 0; y < y2; y += grid)
-		rnd_render->draw_line(gc, x1, y, x2, y);
-	for(y = -grid; y > y1; y -= grid)
-		rnd_render->draw_line(gc, x1, y, x2, y);
+	if (grid > 0) {
+		for(x = 0; x < x2; x += grid)
+			rnd_render->draw_line(gc, x, y1, x, y2);
+		for(x = -grid; x > x1; x -= grid)
+			rnd_render->draw_line(gc, x, y1, x, y2);
+		for(y = 0; y < y2; y += grid)
+			rnd_render->draw_line(gc, x1, y, x2, y);
+		for(y = -grid; y > y1; y -= grid)
+			rnd_render->draw_line(gc, x1, y, x2, y);
+	}
 
 	/* draw the mark only */
 	for(n = 0; n < pcb_proto_num_layers; n++)
@@ -462,6 +465,144 @@ static void pstklib_load(void *hid_ctx, void *caller_data, rnd_hid_attribute_t *
 	rnd_gui->attr_dlg_set_value(ctx->dlg_hid_ctx, ctx->wprev, &hv);
 }
 
+static void pstklib_ccopy(void *hid_ctx, void *caller_data, rnd_hid_attribute_t *attr)
+{
+	pstk_lib_ctx_t *ctx = caller_data;
+	pcb_data_t *data = get_data(ctx, ctx->subc_id, NULL);
+	rnd_hid_row_t *row = rnd_dad_tree_get_selected(&ctx->dlg[ctx->wlist]);
+	pcb_pstk_proto_t *proto;
+	FILE *f;
+	char *tmp_fn;
+	int res;
+
+
+	if (data == NULL)
+		return;
+
+	if (row == NULL) {
+		rnd_message(RND_MSG_ERROR, "First select a prototype to copy\n");
+		return;
+	}
+
+	proto = pcb_pstk_get_proto_(data, strtol(row->cell[0], NULL, 10));
+	if (proto == NULL)
+		return;
+
+	tmp_fn = rnd_tempfile_name_new("pstk_copy");
+	if (tmp_fn == NULL) {
+		rnd_message(RND_MSG_ERROR, "Failed to create temporary file\n");
+		return;
+	}
+	f = rnd_fopen(&ctx->pcb->hidlib, tmp_fn, "w");
+	if (f == NULL) {
+		rnd_message(RND_MSG_ERROR, "Failed to open temporary file %s (write)\n", tmp_fn);
+		rnd_tempfile_unlink(tmp_fn);
+		return;
+	}
+
+	res = pcb_write_padstack(f, proto, "lihata");
+	fclose(f);
+	if (res == 0) {
+		long size = rnd_file_size(&ctx->pcb->hidlib, tmp_fn);
+		void *buf;
+		if ((size > 0) && ((buf = malloc(size)) != NULL)) {
+			f = rnd_fopen(&ctx->pcb->hidlib, tmp_fn, "r");
+			if (f != NULL) {
+				if (fread(buf, size, 1, f) == 1) {
+					if (rnd_gui->clip_set(rnd_gui, RND_HID_CLIPFMT_TEXT, buf, size) != 0)
+						rnd_message(RND_MSG_ERROR, "Failed to write the clipboard\n");
+				}
+				else
+					rnd_message(RND_MSG_ERROR, "Failed to read padstack proto from temp file %s\n", tmp_fn);
+			}
+			else
+				rnd_message(RND_MSG_ERROR, "Failed to open temporary file %s (read)\n", tmp_fn);
+			free(buf);
+			fclose(f);
+		}
+		else
+			rnd_message(RND_MSG_ERROR, "Failed to render the padstack prototype or to allocate memory\n");
+	}
+	else
+		rnd_message(RND_MSG_ERROR, "Failed to write padstack proto into temporary file %s\n", tmp_fn);
+	rnd_tempfile_unlink(tmp_fn);
+}
+
+static void pstklib_cpaste(void *hid_ctx, void *caller_data, rnd_hid_attribute_t *attr)
+{
+	pstk_lib_ctx_t *ctx = caller_data;
+	pcb_data_t *data = get_data(ctx, ctx->subc_id, NULL);
+	rnd_hid_row_t *row = rnd_dad_tree_get_selected(&ctx->dlg[ctx->wlist]);
+	pcb_pstk_proto_t *proto;
+	char *tmp_fn;
+	void *buf;
+	FILE *f;
+	rnd_hid_attr_val_t hv;
+	size_t size;
+	int res, res2;
+	rnd_hid_clipfmt_t fmt;
+
+	if (data == NULL)
+		return;
+
+	if (row == NULL) {
+		rnd_message(RND_MSG_ERROR, "First select a prototype to paste into\n");
+		return;
+	}
+
+	proto = pcb_pstk_get_proto_(data, strtol(row->cell[0], NULL, 10));
+	if (proto == NULL)
+		return;
+
+	tmp_fn = rnd_tempfile_name_new("pstk_paste");
+	if (tmp_fn == NULL) {
+		rnd_message(RND_MSG_ERROR, "Failed to create temporary file\n");
+		return;
+	}
+	f = rnd_fopen(&ctx->pcb->hidlib, tmp_fn, "w");
+	if (f == NULL) {
+		rnd_message(RND_MSG_ERROR, "Failed to open temporary file %s (write)\n", tmp_fn);
+		rnd_tempfile_unlink(tmp_fn);
+		return;
+	}
+
+	res = rnd_gui->clip_get(rnd_gui, &fmt, &buf, &size);
+	if (res == 0) {
+		res2 = fwrite(buf, size, 1, f);
+		rnd_gui->clip_free(rnd_gui, fmt, buf, size);
+	}
+	fclose(f);
+
+	if (res != 0) {
+		rnd_message(RND_MSG_ERROR, "Failed to get data from the clipboard\n");
+		rnd_tempfile_unlink(tmp_fn);
+		return;
+	}
+	if (res2 != 1) {
+		rnd_message(RND_MSG_ERROR, "Failed to write data in temporary file %s\n", tmp_fn);
+		rnd_tempfile_unlink(tmp_fn);
+		return;
+	}
+	if (fmt != RND_HID_CLIPFMT_TEXT) {
+		rnd_message(RND_MSG_ERROR, "Invalid clipboard format\n");
+		rnd_tempfile_unlink(tmp_fn);
+		return;
+	}
+
+	if (pcb_load_padstack(&ctx->pcb->hidlib, proto, tmp_fn, NULL) != 0)
+		rnd_message(RND_MSG_ERROR, "Padstack failed to import from the clipboard.\n");
+
+	proto->parent = data;
+
+	rnd_tempfile_unlink(tmp_fn);
+
+	/* redraw */
+	hv.str = NULL;
+	rnd_gui->attr_dlg_set_value(ctx->dlg_hid_ctx, ctx->wprev, &hv);
+	rnd_gui->invalidate_all(rnd_gui);
+}
+
+
 static void pstklib_proto_switch(void *hid_ctx, void *caller_data, rnd_hid_attribute_t *attr_btn)
 {
 	pstk_lib_ctx_t *ctx = caller_data;
@@ -480,7 +621,7 @@ static void pstklib_proto_switch(void *hid_ctx, void *caller_data, rnd_hid_attri
 		return;
 
 	from_pid = strtol(r->cell[0], NULL, 10);
-	to_pid = pcb_dlg_pstklib(ctx->pcb, ctx->subc_id, rnd_true, "Select a prototype to switch to");
+	to_pid = pcb_dlg_pstklib(ctx->pcb, ctx->subc_id, rnd_true, -1, "Select a prototype to switch to");
 	if ((to_pid == PCB_PADSTACK_INVALID) || (to_pid == from_pid))
 		return;
 
@@ -570,7 +711,7 @@ static void pstklib_del_unused(void *hid_ctx, void *caller_data, rnd_hid_attribu
 	ctx->stat = NULL;
 }
 
-rnd_cardinal_t pcb_dlg_pstklib(pcb_board_t *pcb, long subc_id, rnd_bool modal, const char *hint)
+rnd_cardinal_t pcb_dlg_pstklib(pcb_board_t *pcb, long subc_id, rnd_bool modal, rnd_cardinal_t presel, const char *hint)
 {
 	static const char *hdr[] = {"ID", "name", "used", NULL};
 	pcb_subc_t *sc;
@@ -652,6 +793,12 @@ rnd_cardinal_t pcb_dlg_pstklib(pcb_board_t *pcb, long subc_id, rnd_bool modal, c
 					RND_DAD_BUTTON(ctx->dlg, "Save...");
 						RND_DAD_HELP(ctx->dlg, "Save the selected padstack to a file");
 						RND_DAD_CHANGE_CB(ctx->dlg, pstklib_save);
+					RND_DAD_BUTTON(ctx->dlg, "Paste");
+						RND_DAD_HELP(ctx->dlg, "Replace the selected padstack prototype with the one from the system clipboard");
+						RND_DAD_CHANGE_CB(ctx->dlg, pstklib_cpaste);
+					RND_DAD_BUTTON(ctx->dlg, "Copy");
+						RND_DAD_HELP(ctx->dlg, "Copy selected padstack prototype to the clipboard");
+						RND_DAD_CHANGE_CB(ctx->dlg, pstklib_ccopy);
 				RND_DAD_END(ctx->dlg);
 			RND_DAD_END(ctx->dlg);
 
@@ -711,6 +858,21 @@ rnd_cardinal_t pcb_dlg_pstklib(pcb_board_t *pcb, long subc_id, rnd_bool modal, c
 	pstklib_data2dlg(ctx);
 	free(name);
 
+	/* set cursor if there's a row to preselect */
+	if (presel != PCB_PADSTACK_INVALID) {
+		rnd_hid_attr_val_t hv;
+		pcb_pstk_t ps;
+		char tmp[64];
+
+		ctx->proto_id = presel;
+		sprintf(tmp, "%ld", (long)presel);
+		hv.str = tmp;
+		rnd_gui->attr_dlg_set_value(ctx->dlg_hid_ctx, ctx->wlist, &hv);
+
+		pstklib_setps(&ps, data, ctx->proto_id);
+		pstklib_force_redraw(ctx, &ps);
+	}
+
 	if (modal) {
 		if (RND_DAD_RUN(ctx->dlg) != 0)
 			return PCB_PADSTACK_INVALID;
@@ -720,13 +882,21 @@ rnd_cardinal_t pcb_dlg_pstklib(pcb_board_t *pcb, long subc_id, rnd_bool modal, c
 	return 0;
 }
 
-const char pcb_acts_pstklib[] = "pstklib([board|subcid|object])\n";
+const char pcb_acts_pstklib[] = "pstklib([board|subcid|object], [retpid, [preselect]])\n";
 const char pcb_acth_pstklib[] = "Present the padstack library dialog on board padstacks or the padstacks of a subcircuit";
 fgw_error_t pcb_act_pstklib(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	long id = -1;
-	const char *cmd = NULL;
+	long id = -1, pid, presel = PCB_PADSTACK_INVALID;
+	const char *cmd = NULL, *opt = NULL;
+	int modal = rnd_false;
+
 	RND_ACT_MAY_CONVARG(1, FGW_STR, pstklib, cmd = argv[1].val.str);
+	RND_ACT_MAY_CONVARG(2, FGW_STR, pstklib, opt = argv[2].val.str);
+	RND_ACT_MAY_CONVARG(3, FGW_LONG, pstklib, presel = argv[3].val.nat_long);
+
+	if ((opt != NULL) && (strcmp(opt, "retpid") == 0))
+		modal = 1;
+
 	if ((cmd != NULL) && (strcmp(cmd, "object") == 0)) {
 		rnd_coord_t x, y;
 		void *r1, *r2, *r3;
@@ -741,12 +911,24 @@ fgw_error_t pcb_act_pstklib(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		sc = r2;
 		id = sc->ID;
 	}
+	else if ((cmd != NULL) && (strcmp(cmd, "board") == 0)) {
+		/* defaults are fine */
+	}
 	else
 		RND_ACT_MAY_CONVARG(1, FGW_LONG, pstklib, id = argv[1].val.nat_long);
-	if (pcb_dlg_pstklib(PCB, id, rnd_false, NULL) == PCB_PADSTACK_INVALID)
-		RND_ACT_IRES(-1);
-	else
+
+	pid = pcb_dlg_pstklib(PCB, id, modal, presel, NULL);
+	if (pid != PCB_PADSTACK_INVALID) {
+		if (modal) {
+			res->type = FGW_LONG;
+			res->val.nat_long = pid;
+			return 0;
+		}
 		RND_ACT_IRES(0);
+	}
+	else
+		RND_ACT_IRES(-1);
+
 	return 0;
 }
 
