@@ -35,6 +35,7 @@
 #include <liblihata/dom.h>
 #include <liblihata/tree.h>
 #include <librnd/core/actions.h>
+#include <librnd/core/anyload.h>
 #include <librnd/core/plugins.h>
 #include <librnd/core/error.h>
 #include <librnd/core/conf.h>
@@ -65,7 +66,6 @@ static rnd_conf_hid_id_t cfgid;
 static const char *drc_query_cookie = "drc_query";
 
 extern conf_drc_query_t conf_drc_query;
-#define DRC_QUERY_CONF_FN "drc_query.conf"
 
 #define DRC_CONF_PATH_PLUGIN "plugins/drc_query/"
 #define DRC_CONF_PATH_DISABLE "design/drc_disable/"
@@ -864,15 +864,31 @@ static rnd_action_t drc_query_action_list[] = {
 	{"DrcQueryImport", pcb_act_DrcQueryImport, pcb_acth_DrcQueryImport, pcb_acts_DrcQueryImport}
 };
 
+static int drc_query_anyload_subtree(const rnd_anyload_t *al, rnd_hidlib_t *hl, lht_node_t *root)
+{
+	return drc_query_lht_load_subtree(hl, root);
+}
+
+static int drc_query_anyload_file(const rnd_anyload_t *al, rnd_hidlib_t *hl, const char *filename, const char *type, lht_node_t *nd)
+{
+	if (strncmp(type, "pcb-rnd-drc-query", 17) == 0)
+		return drc_query_lht_load_rules(hl, filename);
+	return rnd_actionva(hl, "loadtedaxfrom", "drc_query", filename, "", "0", NULL);
+}
+
+
+static rnd_anyload_t drc_query_anyload = {0};
+
 int pplg_check_ver_drc_query(int ver_needed) { return 0; }
 
 void pplg_uninit_drc_query(void)
 {
 	long n;
 
+	rnd_anyload_unreg_by_cookie(drc_query_cookie);
 	pcb_drc_impl_unreg(&drc_query_impl);
 	rnd_event_unbind_allcookie(drc_query_cookie);
-	rnd_conf_unreg_file(DRC_QUERY_CONF_FN, drc_query_conf_internal);
+	rnd_conf_unreg_intern(drc_query_conf_internal);
 	rnd_conf_unreg_fields(DRC_CONF_PATH_PLUGIN);
 	rnd_conf_hid_unreg(drc_query_cookie);
 
@@ -901,7 +917,7 @@ int pplg_init_drc_query(void)
 	cbs.new_hlist_item_post = drc_query_newconf;
 	cfgid = rnd_conf_hid_reg(drc_query_cookie, &cbs);
 
-	rnd_conf_reg_file(DRC_QUERY_CONF_FN, drc_query_conf_internal);
+	rnd_conf_reg_intern(drc_query_conf_internal);
 #define conf_reg(field,isarray,type_name,cpath,cname,desc,flags) \
 	rnd_conf_reg_field(conf_drc_query, field,isarray,type_name,cpath,cname,desc,flags);
 #include "drc_query_conf_fields.h"
@@ -912,6 +928,12 @@ int pplg_init_drc_query(void)
 	RND_REGISTER_ACTIONS(drc_query_action_list, drc_query_cookie)
 	pcb_drc_impl_reg(&drc_query_impl);
 
+	drc_query_anyload.load_subtree = drc_query_anyload_subtree;
+	drc_query_anyload.load_file = drc_query_anyload_file;
+	drc_query_anyload.cookie = drc_query_cookie;
+	rnd_anyload_reg("^pcb-rnd-drc-query-v[0-9]*$", &drc_query_anyload);
+	rnd_anyload_reg("^drc_query_rule$", &drc_query_anyload); /* tEDAx ext file */
+
 	return 0;
 }
 
