@@ -29,7 +29,7 @@
 
 const char *bom_cookie = "bom HID";
 
-static rnd_export_opt_t bom_options[] = {
+static const rnd_export_opt_t bom_options[] = {
 /* %start-doc options "8 BOM Creation"
 @ftable @code
 @item --bomfile <string>
@@ -38,11 +38,11 @@ Name of the BOM output file.
 %end-doc
 */
 	{"bomfile", "Name of the BOM output file",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_bomfile 0
 
 	{"cam", "CAM instruction",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_cam 1
 
 };
@@ -61,11 +61,11 @@ typedef struct pcb_bom_list_s {
 	struct pcb_bom_list_s *next;
 } pcb_bom_list_t;
 
-static rnd_export_opt_t *bom_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *bom_get_export_options(rnd_hid_t *hid, int *n)
 {
-	char **val = bom_options[HA_bomfile].value;
-	if ((PCB != NULL) && ((val == NULL) || (*val == NULL) || (**val == '\0')))
-		pcb_derive_default_filename(PCB->hidlib.filename, &bom_options[HA_bomfile], ".bom");
+	const char *val = bom_values[HA_bomfile].str;
+	if ((PCB != NULL) && ((val == NULL) || (*val == '\0')))
+		pcb_derive_default_filename(PCB->hidlib.filename, &bom_values[HA_bomfile], ".bom");
 
 	if (n)
 		*n = NUM_OPTIONS;
@@ -287,5 +287,7 @@ int pplg_init_export_bom(void)
 	bom_hid.usage = bom_usage;
 
 	rnd_hid_register_hid(&bom_hid);
+	rnd_hid_load_defaults(&bom_hid, bom_options, NUM_OPTIONS);
+
 	return 0;
 }
