@@ -78,26 +78,24 @@ static rnd_coord_t viadrill = 25;
 
 static rnd_hid_t dsn_hid;
 
-static rnd_export_opt_t dsn_options[] = {
+static const rnd_export_opt_t dsn_options[] = {
 	{"dsnfile", "SPECCTRA output file",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_dsnfile 0
 	{"trackwidth", "track width in mils",
-	 RND_HATT_COORD, RND_MIL_TO_COORD(0), RND_MIL_TO_COORD(100), {0, 0, 0, RND_MIL_TO_COORD(8)}, 0, 0},
+	 RND_HATT_COORD, RND_MIL_TO_COORD(0), RND_MIL_TO_COORD(100), {0, 0, 0, RND_MIL_TO_COORD(8)}, 0},
 #define HA_trackwidth 1
 	{"clearance", "clearance in mils",
-	 RND_HATT_COORD, RND_MIL_TO_COORD(0), RND_MIL_TO_COORD(100), {0, 0, 0, RND_MIL_TO_COORD(8)}, 0, 0},
+	 RND_HATT_COORD, RND_MIL_TO_COORD(0), RND_MIL_TO_COORD(100), {0, 0, 0, RND_MIL_TO_COORD(8)}, 0},
 #define HA_clearance 2
 	{"viawidth", "via width in mils",
-	 RND_HATT_COORD, RND_MIL_TO_COORD(0), RND_MIL_TO_COORD(100), {0, 0, 0, RND_MIL_TO_COORD(27)}, 0,
-	 0},
+	 RND_HATT_COORD, RND_MIL_TO_COORD(0), RND_MIL_TO_COORD(100), {0, 0, 0, RND_MIL_TO_COORD(27)}, 0},
 #define HA_viawidth 3
 	{"viadrill", "via drill diameter in mils",
-	 RND_HATT_COORD, RND_MIL_TO_COORD(0), RND_MIL_TO_COORD(100), {0, 0, 0, RND_MIL_TO_COORD(15)}, 0,
-	 0},
+	 RND_HATT_COORD, RND_MIL_TO_COORD(0), RND_MIL_TO_COORD(100), {0, 0, 0, RND_MIL_TO_COORD(15)}, 0},
 #define HA_viadrill 4
 	{"cam", "CAM instruction",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_cam 5
 
 };
@@ -108,11 +106,11 @@ static rnd_hid_attr_val_t dsn_values[NUM_OPTIONS];
 
 static const char *dsn_filename;
 
-static rnd_export_opt_t *dsn_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *dsn_get_export_options(rnd_hid_t *hid, int *n)
 {
-	char **val = dsn_options[HA_dsnfile].value;
-	if ((PCB != NULL) && ((val == NULL) || (*val == NULL) || (**val == '\0')))
-		pcb_derive_default_filename(PCB->hidlib.filename, &dsn_options[HA_dsnfile], ".dsn");
+	const char *val = dsn_values[HA_dsnfile].str;
+	if ((PCB != NULL) && ((val == NULL) || (*val == '\0')))
+		pcb_derive_default_filename(PCB->hidlib.filename, &dsn_values[HA_dsnfile], ".dsn");
 	if (n)
 		*n = NUM_OPTIONS;
 	return dsn_options;
@@ -623,6 +621,7 @@ int pplg_init_export_dsn(void)
 	dsn_hid.argument_array = dsn_values;
 
 	rnd_hid_register_hid(&dsn_hid);
+	rnd_hid_load_defaults(&dsn_hid, dsn_options, NUM_OPTIONS);
 
 	rnd_export_register_opts2(&dsn_hid, dsn_options, sizeof(dsn_options) / sizeof(dsn_options[0]), dsn_cookie, 0);
 	return 0;
