@@ -108,7 +108,7 @@ static struct rnd_hid_gc_s thin = {
 #include "dxf_draw.c"
 
 
-rnd_export_opt_t dxf_attribute_list[] = {
+static const rnd_export_opt_t dxf_attribute_list[] = {
 	/* other HIDs expect this to be first.  */
 
 /* %start-doc options "93 DXF Options"
@@ -119,7 +119,7 @@ Name of the file to be exported to. Can contain a path.
 %end-doc
 */
 	{"outfile", "Graphics output file",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_dxffile 0
 
 /* %start-doc options "93 DXF Options"
@@ -130,7 +130,7 @@ Name of the lihata template file to be used instead of the default dxf template.
 %end-doc
 */
 	{"template", "DXF template (lihata file)",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_template 1
 
 /* %start-doc options "93 DXF Options"
@@ -141,7 +141,7 @@ Draw outline and drills with thin lines.
 %end-doc
 */
 	{"thin", "Draw outline and drill with thin lines",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_thin 2
 
 /* %start-doc options "93 DXF Options"
@@ -152,7 +152,7 @@ Fill polygons using hatch
 %end-doc
 */
 	{"poly-fill", "Fill polygons using hatch",
-	 RND_HATT_BOOL, 0, 0, {1, (void *)1, 1}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, (void *)1, 1}, 0},
 #define HA_poly_fill 3
 
 /* %start-doc options "93 DXF Options"
@@ -163,7 +163,7 @@ Draw polygons contour with thin line
 %end-doc
 */
 	{"poly-contour", "Draw polygons contour with thin line",
-	 RND_HATT_BOOL, 0, 0, {1, (void *)1, 1}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, (void *)1, 1}, 0},
 #define HA_poly_contour 4
 
 /* %start-doc options "93 DXF Options"
@@ -174,7 +174,7 @@ Fill drill (hole) circles using hatch
 %end-doc
 */
 	{"drill-fill", "Fill drill (hole) circles using hatch",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_drill_fill 5
 
 /* %start-doc options "93 DXF Options"
@@ -185,11 +185,11 @@ Draw drill contour with thin line
 %end-doc
 */
 	{"drill-contour", "Draw drill contour with thin line",
-	 RND_HATT_BOOL, 0, 0, {1, (void *)1, 1}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, (void *)1, 1}, 0},
 #define HA_drill_contour 6
 
 	{"cam", "CAM instruction",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_cam 7
 
 };
@@ -198,12 +198,12 @@ Draw drill contour with thin line
 
 static rnd_hid_attr_val_t dxf_values[NUM_OPTIONS];
 
-static rnd_export_opt_t *dxf_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *dxf_get_export_options(rnd_hid_t *hid, int *n)
 {
 	const char *suffix = ".dxf";
-	char **val = dxf_attribute_list[HA_dxffile].value;
-	if ((PCB != NULL) && ((val == NULL) || (*val == NULL) || (**val == '\0')))
-		pcb_derive_default_filename(PCB->hidlib.filename, &dxf_attribute_list[HA_dxffile], suffix);
+	const char *val = dxf_values[HA_dxffile].str;
+	if ((PCB != NULL) && ((val == NULL) || (*val == '\0')))
+		pcb_derive_default_filename(PCB->hidlib.filename, &dxf_values[HA_dxffile], suffix);
 
 	if (n)
 		*n = NUM_OPTIONS;
@@ -503,12 +503,6 @@ static void dxf_fill_rect(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1, rnd_c
 	fix_rect_coords();
 }
 
-static void dxf_calibrate(rnd_hid_t *hid, double xval, double yval)
-{
-	rnd_message(RND_MSG_ERROR, "dxf_calibrate() not implemented");
-	return;
-}
-
 static void dxf_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y, int a)
 {
 }
@@ -560,13 +554,13 @@ int pplg_init_export_dxf(void)
 	dxf_hid.fill_polygon = dxf_fill_polygon;
 	dxf_hid.fill_polygon_offs = dxf_fill_polygon_offs;
 	dxf_hid.fill_rect = dxf_fill_rect;
-	dxf_hid.calibrate = dxf_calibrate;
 	dxf_hid.set_crosshair = dxf_set_crosshair;
 	dxf_hid.argument_array = dxf_values;
 
 	dxf_hid.usage = dxf_usage;
 
 	rnd_hid_register_hid(&dxf_hid);
+	rnd_hid_load_defaults(&dxf_hid, dxf_attribute_list, NUM_OPTIONS);
 
 	return 0;
 }
