@@ -164,7 +164,7 @@ typedef struct rnd_hid_gc_s {
 
 static long exc_drawn_objs;
 
-static rnd_export_opt_t excellon_options[] = {
+static const rnd_export_opt_t excellon_options[] = {
 
 /* %start-doc options "90 excellon Export"
 @ftable @code
@@ -174,27 +174,27 @@ excellon output file prefix. Can include a path.
 %end-doc
 */
 	{"filename", "excellon output file base - used to generate default plated and/or unplated file name in case they are not specified",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_excellonfile 0
 
 	{"filename-plated", "excellon output file name for plated features",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_excellonfile_plated 1
 
 	{"filename-unplated", "excellon output file name for unplated features",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_excellonfile_unplated 2
 
 	{"coord-format", "Coordinate format (resolution)",
-	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, coord_format_names, 0},
+	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, coord_format_names},
 #define HA_excellonfile_coordfmt 3
 
 	{"aperture-per-file", "Restart aperture numbering in each new file",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_apeture_per_file 4
 
 	{"cam", "CAM instruction",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_cam 5
 };
 
@@ -202,11 +202,11 @@ excellon output file prefix. Can include a path.
 
 static rnd_hid_attr_val_t excellon_values[NUM_OPTIONS];
 
-static rnd_export_opt_t *excellon_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *excellon_get_export_options(rnd_hid_t *hid, int *n)
 {
-	char **val = excellon_options[HA_excellonfile].value;
-	if ((PCB != NULL) && ((val == NULL) || (*val == NULL) || (**val == '\0')))
-		pcb_derive_default_filename(PCB->hidlib.filename, &excellon_options[HA_excellonfile], "");
+	const char *val = excellon_values[HA_excellonfile].str;
+	if ((PCB != NULL) && ((val == NULL) || (*val == '\0')))
+		pcb_derive_default_filename(PCB->hidlib.filename, &excellon_values[HA_excellonfile], "");
 
 	if (n)
 		*n = NUM_OPTIONS;
@@ -457,11 +457,6 @@ static void excellon_fill_rect(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1,
 	excellon_fill_polygon(gc, 0, NULL, NULL);
 }
 
-static void excellon_calibrate(rnd_hid_t *hid, double xval, double yval)
-{
-	rnd_message(RND_MSG_ERROR, "Excellon internal error: can not calibrate()\n");
-}
-
 static int excellon_usage(rnd_hid_t *hid, const char *topic)
 {
 	fprintf(stderr, "\nexcellon exporter command line arguments:\n\n");
@@ -525,12 +520,12 @@ int pplg_init_export_excellon(void)
 	excellon_hid.fill_polygon = excellon_fill_polygon;
 	excellon_hid.fill_polygon_offs = excellon_fill_polygon_offs;
 	excellon_hid.fill_rect = excellon_fill_rect;
-	excellon_hid.calibrate = excellon_calibrate;
 	excellon_hid.set_crosshair = excellon_set_crosshair;
 	excellon_hid.usage = excellon_usage;
 	excellon_hid.argument_array = excellon_values;
 
 	rnd_hid_register_hid(&excellon_hid);
+	rnd_hid_load_defaults(&excellon_hid, excellon_options, NUM_OPTIONS);
 
 	rnd_event_bind(RND_EVENT_EXPORT_SESSION_BEGIN, exc_session_begin, NULL, excellon_cookie);
 	return 0;
