@@ -195,11 +195,11 @@ static const char *coord_format_names[NUM_COORD_FORMATS+1] = {
 	NULL
 };
 
-static void gerber_warning(rnd_hid_export_opt_func_action_t act, void *call_ctx, rnd_export_opt_t *opt);
+static void gerber_warning(rnd_hid_export_opt_func_action_t act, void *call_ctx, const rnd_export_opt_t *opt, rnd_hid_attr_val_t *val);
 
-static rnd_export_opt_t gerber_options[] = {
+static const rnd_export_opt_t gerber_options[] = {
 	{"", "WARNING",
-	 RND_HATT_BEGIN_VBOX, 0, 0, {0, 0, 0, 0, {0}, gerber_warning}, 0, 0},
+	 RND_HATT_BEGIN_VBOX, 0, 0, {0, 0, 0, 0, {0}, gerber_warning}, 0},
 #define HA_warning 0
 
 /* %start-doc options "90 Gerber Export"
@@ -210,7 +210,7 @@ Gerber output file prefix. Can include a path.
 %end-doc
 */
 	{"gerberfile", "Gerber output file base",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_gerberfile 1
 
 /* %start-doc options "90 Gerber Export"
@@ -221,7 +221,7 @@ Output contains all layers, even empty ones.
 %end-doc
 */
 	{"all-layers", "Output all layers, even empty ones",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_all_layers 2
 
 /* %start-doc options "90 Gerber Export"
@@ -232,25 +232,25 @@ Print file names and aperture counts on stdout.
 %end-doc
 */
 	{"verbose", "Print file names and aperture counts on stdout",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_verbose 3
 	{"copy-outline", "Copy outline onto other layers",
-	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, copy_outline_names, 0},
+	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, copy_outline_names},
 #define HA_copy_outline 4
 	{"cross-sect", "Export the cross section layer",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_cross_sect 5
 
 	{"coord-format", "Coordinate format (resolution)",
-	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, coord_format_names, 0},
+	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, coord_format_names},
 #define HA_coord_format 6
 
 	{"aperture-per-file", "Restart aperture numbering in each new file",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_apeture_per_file 7
 
 	{"cam", "CAM instruction",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_cam 8
 };
 
@@ -258,11 +258,11 @@ Print file names and aperture counts on stdout.
 
 static rnd_hid_attr_val_t gerber_values[NUM_OPTIONS];
 
-static rnd_export_opt_t *gerber_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *gerber_get_export_options(rnd_hid_t *hid, int *n)
 {
-	char **val = gerber_options[HA_gerberfile].value;
-	if ((PCB != NULL) && ((val == NULL) || (*val == NULL) || (**val == '\0')))
-		pcb_derive_default_filename(PCB->hidlib.filename, &gerber_options[HA_gerberfile], "");
+	const char *val = gerber_values[HA_gerberfile].str;
+	if ((PCB != NULL) && ((val == NULL) || (*val == '\0')))
+		pcb_derive_default_filename(PCB->hidlib.filename, &gerber_values[HA_gerberfile], "");
 
 	if (n)
 		*n = NUM_OPTIONS;
@@ -993,11 +993,6 @@ static void gerber_fill_rect(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1, rn
 	gerber_fill_polygon(gc, 5, x, y);
 }
 
-static void gerber_calibrate(rnd_hid_t *hid, double xval, double yval)
-{
-	CRASH("gerber_calibrate");
-}
-
 static int gerber_usage(rnd_hid_t *hid, const char *topic)
 {
 	fprintf(stderr, "\ngerber exporter command line arguments:\n\n");
@@ -1014,7 +1009,7 @@ static void gerber_go_to_cam_cb(void *hid_ctx, void *caller_data, rnd_hid_attrib
 }
 
 
-static void gerber_warning(rnd_hid_export_opt_func_action_t act, void *call_ctx, rnd_export_opt_t *opt)
+static void gerber_warning(rnd_hid_export_opt_func_action_t act, void *call_ctx, const rnd_export_opt_t *opt, rnd_hid_attr_val_t *val)
 {
 	const char warn_txt[] =
 		"WARNING: direct gerber export is most probably NOT what\n"
@@ -1106,12 +1101,12 @@ int pplg_init_export_gerber(void)
 	gerber_hid.fill_polygon = gerber_fill_polygon;
 	gerber_hid.fill_polygon_offs = gerber_fill_polygon_offs;
 	gerber_hid.fill_rect = gerber_fill_rect;
-	gerber_hid.calibrate = gerber_calibrate;
 	gerber_hid.set_crosshair = gerber_set_crosshair;
 	gerber_hid.usage = gerber_usage;
 	gerber_hid.argument_array = gerber_values;
 
 	rnd_hid_register_hid(&gerber_hid);
+	rnd_hid_load_defaults(&gerber_hid, gerber_options, NUM_OPTIONS);
 
 	rnd_event_bind(RND_EVENT_EXPORT_SESSION_BEGIN, gerber_session_begin, NULL, gerber_cookie);
 
