@@ -18,9 +18,14 @@
 #include <librnd/core/hid_attrib.h>
 #include <librnd/core/actions.h>
 
+#include "export_lpr_conf.h"
 const char *lpr_cookie = "lpr HID";
+static rnd_hid_t lpr_hid;
 
-static rnd_export_opt_t base_lpr_options[] = {
+conf_export_lpr_t conf_export_lpr;
+
+
+static const rnd_export_opt_t base_lpr_options[] = {
 
 /* %start-doc options "98 lpr Printing Options"
 @ftable @code
@@ -33,7 +38,7 @@ PDF output with a virtual PDF printer. Example: @*
 %end-doc
 */
 	{"lprcommand", "Command to use for printing",
-	 RND_HATT_STRING, 0, 0, {0, "lpr", 0}, 0, 0}
+	 RND_HATT_STRING, 0, 0, {0, "lpr", 0}, 0}
 #define HA_lprcommand 0
 };
 
@@ -43,16 +48,36 @@ static rnd_export_opt_t *lpr_options = 0;
 static int num_lpr_options = 0;
 static rnd_hid_attr_val_t lpr_values[NUM_OPTIONS];
 
-static rnd_export_opt_t *lpr_get_export_options(rnd_hid_t *hid, int *n)
+static void lpr_maybe_set_value(const char *name, double new_val)
 {
-	char **val;
+	int n;
+
+	if (new_val <= 0)
+		return;
+
+	for(n = 0; n < num_lpr_options; n++) {
+		if (strcmp(name, lpr_options[n].name) == 0) {
+			lpr_values[n].dbl = new_val;
+			break;
+		}
+	}
+}
+
+static const rnd_export_opt_t *lpr_get_export_options(rnd_hid_t *hid, int *n)
+{
+	const char *val;
 
 
 	if (lpr_options == 0) {
-		rnd_export_opt_t *ps_opts = ps_hid.get_export_options(&ps_hid, &num_lpr_options);
+		const rnd_export_opt_t *ps_opts = ps_hid.get_export_options(&ps_hid, &num_lpr_options);
 		lpr_options = calloc(num_lpr_options, sizeof(rnd_hid_attribute_t));
 		memcpy(lpr_options, ps_opts, num_lpr_options * sizeof(rnd_hid_attribute_t));
 		memcpy(lpr_options, base_lpr_options, sizeof(base_lpr_options));
+
+		rnd_hid_load_defaults(&lpr_hid, lpr_options, num_lpr_options);
+
+		lpr_maybe_set_value("xcalib", conf_export_lpr.plugins.export_lpr.default_xcalib);
+		lpr_maybe_set_value("ycalib", conf_export_lpr.plugins.export_lpr.default_ycalib);
 	}
 
 	/*
@@ -60,12 +85,10 @@ static rnd_export_opt_t *lpr_get_export_options(rnd_hid_t *hid, int *n)
 	 * HID's may want to free() this string value and replace it with a
 	 * new one based on how a user fills out a print dialog.
 	 */
-	val = lpr_options[HA_lprcommand].value;
-	if (val == NULL)
-		val = lpr_options[HA_lprcommand].value = &lpr_values[HA_lprcommand].str;
-	if ((*val == NULL) || (**val == '\0')) {
-		free(*val);
-		*val = rnd_strdup("lpr");
+	val = lpr_values[HA_lprcommand].str;
+	if ((val == NULL) || (*val == '\0')) {
+		free((char *)lpr_values[HA_lprcommand].str);
+		lpr_values[HA_lprcommand].str = rnd_strdup("lpr");
 	}
 
 	if (n)
@@ -104,13 +127,6 @@ static int lpr_parse_arguments(rnd_hid_t *hid, int *argc, char ***argv)
 	return rnd_hid_parse_command_line(argc, argv);
 }
 
-static void lpr_calibrate(rnd_hid_t *hid, double xval, double yval)
-{
-	ps_calibrate_1(hid, xval, yval, 1);
-}
-
-static rnd_hid_t lpr_hid;
-
 static int lpr_usage(rnd_hid_t *hid, const char *topic)
 {
 	fprintf(stderr, "\nlpr exporter command line arguments:\n\n");
@@ -125,6 +141,7 @@ void pplg_uninit_export_lpr(void)
 {
 	rnd_remove_actions_by_cookie(lpr_cookie);
 	rnd_hid_remove_hid(&lpr_hid);
+	rnd_conf_unreg_fields("plugins/export_lpr/");
 }
 
 int pplg_init_export_lpr(void)
@@ -143,12 +160,16 @@ int pplg_init_export_lpr(void)
 	lpr_hid.get_export_options = lpr_get_export_options;
 	lpr_hid.do_export = lpr_do_export;
 	lpr_hid.parse_arguments = lpr_parse_arguments;
-	lpr_hid.calibrate = lpr_calibrate;
 	lpr_hid.argument_array = lpr_values;
 
 	lpr_hid.usage = lpr_usage;
 
 	rnd_hid_register_hid(&lpr_hid);
+	rnd_hid_load_defaults(&lpr_hid, base_lpr_options, NUM_OPTIONS);
+
+#define conf_reg(field,isarray,type_name,cpath,cname,desc,flags) \
+	rnd_conf_reg_field(conf_export_lpr, field,isarray,type_name,cpath,cname,desc,flags);
+#include "export_lpr_conf_fields.h"
 
 	return 0;
 }
