@@ -103,61 +103,61 @@ static int openems_ovr;
 
 #define THMAX RND_MM_TO_COORD(100)
 
-rnd_export_opt_t openems_attribute_list[] = {
+static const rnd_export_opt_t openems_attribute_list[] = {
 	{"outfile", "Graphics output file",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_openemsfile 0
 
 	{"def-copper-thick", "Default copper thickness",
-	 RND_HATT_COORD, 0, THMAX, {0, 0, 0, RND_MM_TO_COORD(0.035)}, 0, 0},
+	 RND_HATT_COORD, 0, THMAX, {0, 0, 0, RND_MM_TO_COORD(0.035)}, 0},
 #define HA_def_copper_thick 1
 
 	{"def-substrate-thick", "Default substrate thickness",
-	 RND_HATT_COORD, 0, THMAX, {0, 0, 0, RND_MM_TO_COORD(0.8)}, 0, 0},
+	 RND_HATT_COORD, 0, THMAX, {0, 0, 0, RND_MM_TO_COORD(0.8)}, 0},
 #define HA_def_substrate_thick 2
 
 	{"def-copper-cond", "Default copper conductivity",
-	 RND_HATT_STRING, 0, 0, {0, "56e6", 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, "56e6", 0}, 0},
 #define HA_def_copper_cond 3
 
 	{"def-subst-epsilon", "Default substrate epsilon",
-	 RND_HATT_STRING, 0, 0, {0, "3.66", 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, "3.66", 0}, 0},
 #define HA_def_subst_epsilon 4
 
 	{"def-subst-mue", "Default substrate mue",
-	 RND_HATT_STRING, 0, 0, {0, "0", 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, "0", 0}, 0},
 #define HA_def_subst_mue 5
 
 	{"def-subst-kappa", "Default substrate kappa",
-	 RND_HATT_STRING, 0, 0, {0, "0", 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, "0", 0}, 0},
 #define HA_def_subst_kappa 6
 
 	{"def-subst-sigma", "Default substrate sigma",
-	 RND_HATT_STRING, 0, 0, {0, "0", 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, "0", 0}, 0},
 #define HA_def_subst_sigma 7
 
 	{"void-name", "Name of the void (sorrunding material)",
-	 RND_HATT_STRING, 0, 0, {0, "AIR", 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, "AIR", 0}, 0},
 #define HA_void_name 8
 
 	{"void-epsilon", "epsilon value for the void (sorrunding material)",
-	 RND_HATT_REAL, 0, 1000, {0, 0, 1}, 0, 0},
+	 RND_HATT_REAL, 0, 1000, {0, 0, 1}, 0},
 #define HA_void_epsilon 9
 
 	{"void-mue", "mue value for the void (sorrunding material)",
-	 RND_HATT_REAL, 0, 1000, {0, 0, 1}, 0, 0},
+	 RND_HATT_REAL, 0, 1000, {0, 0, 1}, 0},
 #define HA_void_mue 10
 
 	{"segments", "kludge: number of segments used to approximate round cap trace ends",
-	 RND_HATT_INTEGER, 0, 100, {10, 0, 0}, 0, 0},
+	 RND_HATT_INTEGER, 0, 100, {10, 0, 0}, 0},
 #define HA_segments 11
 
 	{"base-prio", "base priority: if the board displaces the chassis",
-	 RND_HATT_INTEGER, 0, 10, {0, 0, 0}, 0, 0},
+	 RND_HATT_INTEGER, 0, 10, {0, 0, 0}, 0},
 #define HA_base_prio 12
 
 	{"port-resistance", "default port resistance",
-	 RND_HATT_REAL, 0, 1000, {0, 0, 50}, 0, 0}
+	 RND_HATT_REAL, 0, 1000, {0, 0, 50}, 0}
 #define HA_def_port_res 13
 
 };
@@ -166,27 +166,27 @@ rnd_export_opt_t openems_attribute_list[] = {
 
 static rnd_hid_attr_val_t openems_values[NUM_OPTIONS];
 
-static rnd_export_opt_t *openems_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *openems_get_export_options(rnd_hid_t *hid, int *n)
 {
 	const char *suffix = ".m";
 	pcb_mesh_t *mesh = pcb_mesh_get(MESH_NAME);
-	char **val = openems_attribute_list[HA_openemsfile].value;
+	const char *val = openems_values[HA_openemsfile].str;
 
-	if ((PCB != NULL) && ((val == NULL) || (*val == NULL) || (**val == '\0')))
-		pcb_derive_default_filename(PCB->hidlib.filename, &openems_attribute_list[HA_openemsfile], suffix);
+	if ((PCB != NULL) && ((val == NULL) || (*val == '\0')))
+		pcb_derive_default_filename(PCB->hidlib.filename, &openems_values[HA_openemsfile], suffix);
 
 	if (mesh != NULL) {
-		openems_attribute_list[HA_def_substrate_thick].default_val.crd = mesh->def_subs_thick;
-		openems_attribute_list[HA_def_copper_thick].default_val.crd = mesh->def_copper_thick;
+		openems_values[HA_def_substrate_thick].crd = mesh->def_subs_thick;
+		openems_values[HA_def_copper_thick].crd = mesh->def_copper_thick;
 	}
 
 TODO(": when export dialogs change into DAD, this hack to convert the strings to allocated ones will not be needed anymore")
-	openems_attribute_list[HA_def_copper_cond].default_val.str = rnd_strdup(openems_attribute_list[HA_def_copper_cond].default_val.str);
-	openems_attribute_list[HA_def_subst_epsilon].default_val.str = rnd_strdup(openems_attribute_list[HA_def_subst_epsilon].default_val.str);
-	openems_attribute_list[HA_def_subst_mue].default_val.str = rnd_strdup(openems_attribute_list[HA_def_subst_mue].default_val.str);
-	openems_attribute_list[HA_def_subst_kappa].default_val.str = rnd_strdup(openems_attribute_list[HA_def_subst_kappa].default_val.str);
-	openems_attribute_list[HA_def_subst_sigma].default_val.str = rnd_strdup(openems_attribute_list[HA_def_subst_sigma].default_val.str);
-	openems_attribute_list[HA_void_name].default_val.str = rnd_strdup(openems_attribute_list[HA_void_name].default_val.str);
+	openems_values[HA_def_copper_cond].str = rnd_strdup(openems_attribute_list[HA_def_copper_cond].default_val.str);
+	openems_values[HA_def_subst_epsilon].str = rnd_strdup(openems_attribute_list[HA_def_subst_epsilon].default_val.str);
+	openems_values[HA_def_subst_mue].str = rnd_strdup(openems_attribute_list[HA_def_subst_mue].default_val.str);
+	openems_values[HA_def_subst_kappa].str = rnd_strdup(openems_attribute_list[HA_def_subst_kappa].default_val.str);
+	openems_values[HA_def_subst_sigma].str = rnd_strdup(openems_attribute_list[HA_def_subst_sigma].default_val.str);
+	openems_values[HA_void_name].str = rnd_strdup(openems_attribute_list[HA_void_name].default_val.str);
 
 	if (n)
 		*n = NUM_OPTIONS;
@@ -888,13 +888,6 @@ static void openems_draw_line(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1, r
 	}
 }
 
-
-static void openems_calibrate(rnd_hid_t *hid, double xval, double yval)
-{
-	rnd_message(RND_MSG_ERROR, "openems_calibrate() not implemented");
-	return;
-}
-
 static void openems_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y, int a)
 {
 }
@@ -953,7 +946,6 @@ int pplg_init_export_openems(void)
 	openems_hid.fill_polygon = openems_fill_polygon;
 	openems_hid.fill_polygon_offs = openems_fill_polygon_offs;
 	openems_hid.fill_rect = openems_fill_rect;
-	openems_hid.calibrate = openems_calibrate;
 	openems_hid.set_crosshair = openems_set_crosshair;
 	openems_hid.argument_array = openems_values;
 
@@ -964,6 +956,7 @@ int pplg_init_export_openems(void)
 	RND_REGISTER_ACTIONS(openems_action_list, openems_cookie);
 
 	pcb_openems_excitation_init();
+	rnd_hid_load_defaults(&openems_hid, openems_attribute_list, NUM_OPTIONS);
 
 	return 0;
 }
