@@ -84,7 +84,7 @@ static vti0_t scad_comp;
 
 static rnd_hid_attr_val_t *openscad_options;
 
-rnd_export_opt_t openscad_attribute_list[] = {
+static const rnd_export_opt_t openscad_attribute_list[] = {
 	/* other HIDs expect this to be first.  */
 
 /* %start-doc options "93 DXF Options"
@@ -95,31 +95,31 @@ Name of the file to be exported to. Can contain a path.
 %end-doc
 */
 	{"outfile", "Graphics output file",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_openscadfile 0
 
 	{"copper", "enable exporting outer copper layers",
-	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0},
 #define HA_copper 1
 
 	{"silk", "enable exporting silk layers",
-	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0},
 #define HA_silk 2
 
 	{"mask", "enable exporting mask layers",
-	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0},
 #define HA_mask 3
 
 	{"models", "enable searching and inserting model files",
-	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0},
 #define HA_models 4
 
 	{"drill", "enable drilling holes",
-	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0},
 #define HA_drill 5
 
 	{"cam", "CAM instruction",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_cam 6
 
 };
@@ -131,13 +131,13 @@ Name of the file to be exported to. Can contain a path.
 
 static rnd_hid_attr_val_t openscad_values[NUM_OPTIONS];
 
-static rnd_export_opt_t *openscad_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *openscad_get_export_options(rnd_hid_t *hid, int *n)
 {
 	const char *suffix = ".scad";
-	char **val = openscad_attribute_list[HA_openscadfile].value;
+	const char *val = openscad_values[HA_openscadfile].str;
 
-	if ((PCB != NULL) && ((val == NULL) || (*val == NULL) || (**val == '\0')))
-		pcb_derive_default_filename(PCB->hidlib.filename, &openscad_attribute_list[HA_openscadfile], suffix);
+	if ((PCB != NULL) && ((val == NULL) || (*val == '\0')))
+		pcb_derive_default_filename(PCB->hidlib.filename, &openscad_values[HA_openscadfile], suffix);
 
 	if (n)
 		*n = NUM_OPTIONS;
@@ -583,12 +583,6 @@ static void openscad_fill_polygon(rnd_hid_gc_t gc, int n_coords, rnd_coord_t *x,
 	openscad_fill_polygon_offs(gc, n_coords, x, y, 0, 0);
 }
 
-static void openscad_calibrate(rnd_hid_t *hid, double xval, double yval)
-{
-	rnd_message(RND_MSG_ERROR, "openscad_calibrate() not implemented");
-	return;
-}
-
 static void openscad_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y, int a)
 {
 }
@@ -697,13 +691,13 @@ int pplg_init_export_openscad(void)
 	openscad_hid.fill_polygon = openscad_fill_polygon;
 	openscad_hid.fill_polygon_offs = openscad_fill_polygon_offs;
 	openscad_hid.fill_rect = openscad_fill_rect;
-	openscad_hid.calibrate = openscad_calibrate;
 	openscad_hid.set_crosshair = openscad_set_crosshair;
 	openscad_hid.argument_array = openscad_values;
 
 	openscad_hid.usage = openscad_usage;
 
 	rnd_hid_register_hid(&openscad_hid);
+	rnd_hid_load_defaults(&openscad_hid, openscad_attribute_list, NUM_OPTIONS);
 
 	RND_REGISTER_ACTIONS(scad_action_list, openscad_cookie)
 
