@@ -165,7 +165,7 @@ static const char *filetypes[] = {
 
 #include "png_photo1.c"
 
-rnd_export_opt_t png_attribute_list[] = {
+static const rnd_export_opt_t png_attribute_list[] = {
 /* other HIDs expect this to be first.  */
 
 /* %start-doc options "93 PNG Options"
@@ -176,7 +176,7 @@ Name of the file to be exported to. Can contain a path.
 %end-doc
 */
 	{"outfile", "Graphics output file",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_pngfile 0
 
 /* %start-doc options "93 PNG Options"
@@ -187,7 +187,7 @@ Scale factor in pixels/inch. Set to 0 to scale to size specified in the layout.
 %end-doc
 */
 	{"dpi", "Scale factor (pixels/inch). 0 to scale to specified size",
-	 RND_HATT_INTEGER, 0, 10000, {100, 0, 0}, 0, 0},
+	 RND_HATT_INTEGER, 0, 10000, {100, 0, 0}, 0},
 #define HA_dpi 1
 
 /* %start-doc options "93 PNG Options"
@@ -198,7 +198,7 @@ Width of the png image in pixels. No constraint, when set to 0.
 %end-doc
 */
 	{"x-max", "Maximum width (pixels).  0 to not constrain",
-	 RND_HATT_INTEGER, 0, 10000, {0, 0, 0}, 0, 0},
+	 RND_HATT_INTEGER, 0, 10000, {0, 0, 0}, 0},
 #define HA_xmax 2
 
 /* %start-doc options "93 PNG Options"
@@ -209,7 +209,7 @@ Height of the png output in pixels. No constraint, when set to 0.
 %end-doc
 */
 	{"y-max", "Maximum height (pixels).  0 to not constrain",
-	 RND_HATT_INTEGER, 0, 10000, {0, 0, 0}, 0, 0},
+	 RND_HATT_INTEGER, 0, 10000, {0, 0, 0}, 0},
 #define HA_ymax 3
 
 /* %start-doc options "93 PNG Options"
@@ -220,7 +220,7 @@ Maximum width and height of the PNG output in pixels. No constraint, when set to
 %end-doc
 */
 	{"xy-max", "Maximum width and height (pixels).  0 to not constrain",
-	 RND_HATT_INTEGER, 0, 10000, {0, 0, 0}, 0, 0},
+	 RND_HATT_INTEGER, 0, 10000, {0, 0, 0}, 0},
 #define HA_xymax 4
 
 /* %start-doc options "93 PNG Options"
@@ -231,7 +231,7 @@ Export layers as shown on screen.
 %end-doc
 */
 	{"as-shown", "Export layers as shown on screen",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_as_shown 5
 
 /* %start-doc options "93 PNG Options"
@@ -242,7 +242,7 @@ Convert output to monochrome.
 %end-doc
 */
 	{"monochrome", "Convert to monochrome",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_mono 6
 
 /* %start-doc options "93 PNG Options"
@@ -253,7 +253,7 @@ Limit the bounds of the exported PNG image to the visible items.
 %end-doc
 */
 	{"only-visible", "Limit the bounds of the PNG image to the visible items",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_only_visible 7
 
 /* %start-doc options "93 PNG Options"
@@ -264,7 +264,7 @@ Make the background and any holes transparent.
 %end-doc
 */
 	{"use-alpha", "Make the background and any holes transparent",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_use_alpha 8
 
 /* %start-doc options "93 PNG Options"
@@ -276,7 +276,7 @@ File format to be exported. Parameter @code{<string>} can be @samp{PNG},
 %end-doc
 */
 	{"format", "Export file format",
-	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, filetypes, 0},
+	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, filetypes},
 #define HA_filetype 9
 
 /* %start-doc options "93 PNG Options"
@@ -290,7 +290,7 @@ OBSOLETE - do not use this feature!
 %end-doc
 */
 	{"png-bloat", "Amount (in/mm/mil/pix) to add to trace/pad/pin edges (1 = 1/100 mil) (OBSOLETE - do not use this feature!)",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_bloat 10
 
 /* %start-doc options "93 PNG Options"
@@ -302,7 +302,7 @@ Export a photo realistic image of the layout.
 %end-doc
 */
 	{"photo-mode", "Photo-realistic export mode",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_photo_mode 11
 
 /* %start-doc options "93 PNG Options"
@@ -313,7 +313,7 @@ In photo-realistic mode, export the reverse side of the layout. Left-right flip.
 %end-doc
 */
 	{"photo-flip-x", "Show reverse side of the board, left-right flip",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_photo_flip_x 12
 
 /* %start-doc options "93 PNG Options"
@@ -324,7 +324,7 @@ In photo-realistic mode, export the reverse side of the layout. Up-down flip.
 %end-doc
 */
 	{"photo-flip-y", "Show reverse side of the board, up-down flip",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_photo_flip_y 13
 
 /* %start-doc options "93 PNG Options"
@@ -337,7 +337,7 @@ In photo-realistic mode, export the solder mask as this colour. Parameter
 %end-doc
 */
 	{"photo-mask-colour", "Colour for the exported colour mask (green, red, blue, purple)",
-	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, mask_colour_names, 0},
+	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, mask_colour_names},
 #define HA_photo_mask_colour 14
 
 /* %start-doc options "93 PNG Options"
@@ -351,7 +351,7 @@ of plating. Parameter @code{<colour>} can be @samp{tinned}, @samp{gold},
 %end-doc
 */
 	{"photo-plating", "Type of plating applied to exposed copper in photo-mode (tinned, gold, silver, copper)",
-	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, plating_type_names, 0},
+	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, plating_type_names},
 #define HA_photo_plating 15
 
 /* %start-doc options "93 PNG Options"
@@ -364,11 +364,11 @@ In photo-realistic mode, export the silk screen as this colour. Parameter
 %end-doc
 */
 	{"photo-silk-colour", "Colour for the exported colour silk (white, black, yellow)",
-	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, silk_colour_names, 0},
+	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, silk_colour_names},
 #define HA_photo_silk_colour 16
 
 	{"cam", "CAM instruction",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_cam 17
 };
 
@@ -381,7 +381,7 @@ static const char *get_file_suffix(void)
 	const char *result = NULL;
 	const char *fmt;
 
-	fmt = filetypes[png_attribute_list[HA_filetype].default_val.lng];
+	fmt = filetypes[png_values[HA_filetype].lng];
 
 	if (fmt == NULL) { /* Do nothing */ }
 	else if (strcmp(fmt, FMT_gif) == 0)
@@ -398,13 +398,13 @@ static const char *get_file_suffix(void)
 	return result;
 }
 
-static rnd_export_opt_t *png_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *png_get_export_options(rnd_hid_t *hid, int *n)
 {
 	const char *suffix = get_file_suffix();
-	char **val = png_attribute_list[HA_pngfile].value;
+	const char *val = png_values[HA_pngfile].str;
 
-	if ((PCB != NULL) && ((val == NULL) || (*val == NULL) || (**val == '\0')))
-		pcb_derive_default_filename(PCB->hidlib.filename, &png_attribute_list[HA_pngfile], suffix);
+	if ((PCB != NULL) && ((val == NULL) || (*val == '\0')))
+		pcb_derive_default_filename(PCB->hidlib.filename, &png_values[HA_pngfile], suffix);
 
 	if (n)
 		*n = NUM_OPTIONS;
@@ -569,9 +569,13 @@ void png_hid_export_to_file(FILE *the_file, rnd_hid_attr_val_t *options, rnd_xfo
 		}
 	}
 
+	if (!png_cam.active || as_shown)
+		xform->enable_silk_invis_clr = 1;
+
 	if (as_shown) {
 		/* disable (exporter default) hiding overlay in as_shown */
 		xform->omit_overlay = 0;
+		xform->add_gui_xform = 1;
 	}
 
 	ctx.view = *bounds;
@@ -830,6 +834,9 @@ static int png_set_layer_group(rnd_hid_t *hid, rnd_layergrp_id_t group, const ch
 	if (photo_mode)
 		return png_set_layer_group_photo(group, purpose, purpi, layer, flags, is_empty, xform);
 
+	if (png_cam.active) /* CAM decided already */
+		return 1;
+
 	if (as_shown) {
 		if ((flags & PCB_LYT_ANYTHING) == PCB_LYT_SILK) {
 			if (PCB_LAYERFLG_ON_VISIBLE_SIDE(flags))
@@ -1379,12 +1386,6 @@ static void png_fill_polygon(rnd_hid_gc_t gc, int n_coords, rnd_coord_t *x, rnd_
 	png_fill_polygon_offs(gc, n_coords, x, y, 0, 0);
 }
 
-
-static void png_calibrate(rnd_hid_t *hid, double xval, double yval)
-{
-	CRASH("png_calibrate");
-}
-
 static void png_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y, int a)
 {
 }
@@ -1438,7 +1439,6 @@ int pplg_init_export_png(void)
 	png_hid.fill_polygon = png_fill_polygon;
 	png_hid.fill_polygon_offs = png_fill_polygon_offs;
 	png_hid.fill_rect = png_fill_rect;
-	png_hid.calibrate = png_calibrate;
 	png_hid.set_crosshair = png_set_crosshair;
 	png_hid.argument_array = png_values;
 
@@ -1446,6 +1446,8 @@ int pplg_init_export_png(void)
 
 #ifdef HAVE_SOME_FORMAT
 	rnd_hid_register_hid(&png_hid);
+	rnd_hid_load_defaults(&png_hid, png_attribute_list, NUM_OPTIONS);
 #endif
+
 	return 0;
 }
