@@ -48,7 +48,7 @@ static int fast_erase = -1;
 static rnd_composite_op_t drawing_mode;
 static long eps_drawn_objs;
 
-static rnd_export_opt_t eps_attribute_list[] = {
+static const rnd_export_opt_t eps_attribute_list[] = {
 	/* other HIDs expect this to be first.  */
 
 /* %start-doc options "92 Encapsulated Postscript Export"
@@ -59,7 +59,7 @@ Name of the encapsulated postscript output file. Can contain a path.
 %end-doc
 */
 	{"eps-file", "Encapsulated Postscript output file",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_psfile 0
 
 /* %start-doc options "92 Encapsulated Postscript Export"
@@ -70,7 +70,7 @@ Scale EPS output by the parameter @samp{num}.
 %end-doc
 */
 	{"eps-scale", "EPS scale",
-	 RND_HATT_REAL, 0, 100, {0, 0, 1.0}, 0, 0},
+	 RND_HATT_REAL, 0, 100, {0, 0, 1.0}, 0},
 #define HA_scale 1
 
 /* %start-doc options "92 Encapsulated Postscript Export"
@@ -82,7 +82,7 @@ Export layers as shown on screen.
 %end-doc
 */
 	{"as-shown", "Export layers as shown on screen",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_as_shown 2
 
 /* %start-doc options "92 Encapsulated Postscript Export"
@@ -93,7 +93,7 @@ Convert output to monochrome.
 %end-doc
 */
 	{"monochrome", "Convert to monochrome",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_mono 3
 
 /* %start-doc options "92 Encapsulated Postscript Export"
@@ -105,11 +105,11 @@ Limit the bounds of the EPS file to the visible items.
 %end-doc
 */
 	{"only-visible", "Limit the bounds of the EPS file to the visible items",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_only_visible 4
 
 	{"cam", "CAM instruction",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_cam 5
 
 };
@@ -118,12 +118,12 @@ Limit the bounds of the EPS file to the visible items.
 
 static rnd_hid_attr_val_t eps_values[NUM_OPTIONS];
 
-static rnd_export_opt_t *eps_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *eps_get_export_options(rnd_hid_t *hid, int *n)
 {
-	char **val = eps_attribute_list[HA_psfile].value;
+	const char *val = eps_values[HA_psfile].str;
 
-	if ((PCB != NULL) && ((val == NULL) || (*val == NULL) || (**val == '\0')))
-		pcb_derive_default_filename(PCB->hidlib.filename, &eps_attribute_list[HA_psfile], ".eps");
+	if ((PCB != NULL) && ((val == NULL) || (*val == '\0')))
+		pcb_derive_default_filename(PCB->hidlib.filename, &eps_values[HA_psfile], ".eps");
 
 	if (n)
 		*n = NUM_OPTIONS;
@@ -298,6 +298,8 @@ void eps_hid_export_to_file(FILE * the_file, rnd_hid_attr_val_t *options, rnd_xf
 	if (as_shown) {
 		/* disable (exporter default) hiding overlay in as_shown */
 		xform->omit_overlay = 0;
+		xform->add_gui_xform = 1;
+		xform->enable_silk_invis_clr = 1;
 	}
 
 	ctx.view = *bounds;
@@ -412,6 +414,9 @@ static int eps_set_layer_group(rnd_hid_t *hid, rnd_layergrp_id_t group, const ch
 	fprintf(f, "%% Layer %s group %ld drill %d mask %d\n", name, group, is_drill, is_mask);
 	gds_uninit(&tmp_ln);
 
+	if (eps_cam.active) /* CAM has decided already */
+		return 1;
+
 	if (as_shown) {
 		if (PCB_LAYERFLG_ON_VISIBLE_SIDE(flags))
 			return pcb_silk_on(PCB);
@@ -630,11 +635,6 @@ static void eps_fill_rect(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1, rnd_c
 	rnd_fprintf(f, "%mi %mi %mi %mi r\n", x1, y1, x2, y2);
 }
 
-static void eps_calibrate(rnd_hid_t *hid, double xval, double yval)
-{
-	CRASH("eps_calibrate");
-}
-
 static void eps_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y, int action)
 {
 }
@@ -681,11 +681,11 @@ void hid_eps_init()
 	eps_hid.fill_polygon = eps_fill_polygon;
 	eps_hid.fill_polygon_offs = eps_fill_polygon_offs;
 	eps_hid.fill_rect = eps_fill_rect;
-	eps_hid.calibrate = eps_calibrate;
 	eps_hid.set_crosshair = eps_set_crosshair;
 	eps_hid.argument_array = eps_values;
 
 	eps_hid.usage = eps_usage;
 
 	rnd_hid_register_hid(&eps_hid);
+	rnd_hid_load_defaults(&eps_hid, eps_attribute_list, NUM_OPTIONS);
 }
