@@ -47,7 +47,7 @@ typedef struct rnd_hid_gc_s {
 
 static pcb_cam_t ps_cam;
 
-rnd_export_opt_t ps_attribute_list[] = {
+static const rnd_export_opt_t ps_attribute_list[] = {
 	/* other HIDs expect this to be first.  */
 
 /* %start-doc options "91 Postscript Export"
@@ -58,7 +58,7 @@ Name of the postscript output file. Can contain a path.
 %end-doc
 */
 	{"psfile", "Postscript output file",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_psfile 0
 
 /* %start-doc options "91 Postscript Export"
@@ -70,7 +70,7 @@ Draw small holes to center drill but in copper
 %end-doc
 */
 	{"drill-helper", "Draw small holes to center drill but in copper",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_drillhelper 1
 
 /* %start-doc options "91 Postscript Export"
@@ -82,7 +82,7 @@ Print alignment marks on each sheet. This is meant to ease alignment during expo
 %end-doc
 */
 	{"align-marks", "Print alignment marks on each sheet",
-	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0},
 #define HA_alignmarks 2
 
 /* %start-doc options "91 Postscript Export"
@@ -93,7 +93,7 @@ Print the contents of the outline layer on each sheet.
 %end-doc
 */
 	{"outline", "Print outline on each sheet",
-	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0},
 #define HA_outline 3
 /* %start-doc options "91 Postscript Export"
 @ftable @code
@@ -103,7 +103,7 @@ Print mirror image.
 %end-doc
 */
 	{"mirror", "Print mirror image of every page",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_mirror 4
 
 /* %start-doc options "91 Postscript Export"
@@ -114,7 +114,7 @@ Scale output to make the board fit the page.
 %end-doc
 */
 	{"fill-page", "Scale board to fill page",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_fillpage 5
 
 /* %start-doc options "91 Postscript Export"
@@ -125,7 +125,7 @@ Print mirror image of appropriate layers.
 %end-doc
 */
 	{"auto-mirror", "Print mirror image of appropriate layers",
-	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0},
 #define HA_automirror 6
 
 /* %start-doc options "91 Postscript Export"
@@ -136,7 +136,7 @@ Postscript output in color.
 %end-doc
 */
 	{"ps-color", "Prints in color",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_color 7
 
 /* %start-doc options "91 Postscript Export"
@@ -148,7 +148,7 @@ Draw objects as white-on-black.
 %end-doc
 */
 	{"ps-invert", "Draw objects as white-on-black",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_psinvert 8
 
 /* %start-doc options "91 Postscript Export"
@@ -164,7 +164,7 @@ to @samp{A10}, @samp{B0} to @samp{B10}, @samp{Letter}, @samp{11x17},
 %end-doc
 */
 	{"media", "media type",
-	 RND_HATT_ENUM, 0, 0, {22, 0, 0}, pcb_medias, 0},
+	 RND_HATT_ENUM, 0, 0, {22, 0, 0}, pcb_medias},
 #define HA_media 9
 
 /* %start-doc options "91 Postscript Export"
@@ -176,7 +176,7 @@ Fade amount for assembly drawings (0.0=missing, 1.0=solid).
 %end-doc
 */
 	{"psfade", "Fade amount for assembly drawings (0.0=missing, 1.0=solid)",
-	 RND_HATT_REAL, 0, 1, {0, 0, 0.40}, 0, 0},
+	 RND_HATT_REAL, 0, 1, {0, 0, 0.40}, 0},
 #define HA_psfade 10
 
 /* %start-doc options "91 Postscript Export"
@@ -187,7 +187,7 @@ Scale value to compensate for printer sizing errors (1.0 = full scale).
 %end-doc
 */
 	{"scale", "Scale value to compensate for printer sizing errors (1.0 = full scale)",
-	 RND_HATT_REAL, 0.01, 4, {0, 0, 1.00}, 0, 0},
+	 RND_HATT_REAL, 0.01, 4, {0, 0, 1.00}, 0},
 #define HA_scale 11
 
 /* %start-doc options "91 Postscript Export"
@@ -199,7 +199,7 @@ Produce multiple files, one per page, instead of a single multi page file.
 %end-doc
 */
 	{"multi-file", "Produce multiple files, one per page, instead of a single file",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0.40}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0.40}, 0},
 #define HA_multifile 12
 
 /* %start-doc options "91 Postscript Export"
@@ -210,7 +210,7 @@ Paper width. Used for x-Axis calibration.
 %end-doc
 */
 	{"xcalib", "Paper width. Used for x-Axis calibration",
-	 RND_HATT_REAL, 0, 2, {0, 0, 1.0}, 0, 0},
+	 RND_HATT_REAL, 0, 2, {0, 0, 1.0}, 0},
 #define HA_xcalib 13
 
 /* %start-doc options "91 Postscript Export"
@@ -221,7 +221,7 @@ Paper height. Used for y-Axis calibration.
 %end-doc
 */
 	{"ycalib", "Paper height. Used for y-Axis calibration",
-	 RND_HATT_REAL, 0, 2, {0, 0, 1.0}, 0, 0},
+	 RND_HATT_REAL, 0, 2, {0, 0, 1.0}, 0},
 #define HA_ycalib 14
 
 /* %start-doc options "91 Postscript Export"
@@ -232,7 +232,7 @@ Draw drill holes in pins / vias, instead of leaving solid copper.
 %end-doc
 */
 	{"drill-copper", "Draw drill holes in pins / vias, instead of leaving solid copper",
-	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0},
 #define HA_drillcopper 15
 
 /* %start-doc options "91 Postscript Export"
@@ -244,7 +244,7 @@ Print file name and scale on printout.
 %end-doc
 */
 	{"show-legend", "Print file name and scale on printout",
-	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0},
 #define HA_legend 16
 
 /* %start-doc options "91 Postscript Export"
@@ -256,7 +256,7 @@ Generate Table of Contents
 %end-doc
 */
 	{"show-toc", "Print Table of Content",
-	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0},
 #define HA_toc 17
 
 
@@ -269,7 +269,7 @@ Merge all drawings on a single page
 %end-doc
 */
 	{"single-page", "Merge all drawings on a single page",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_single_page 18
 
 /* %start-doc options "91 Postscript Export"
@@ -281,12 +281,12 @@ Diameter of the small hole when drill-helper is on
 %end-doc
 */
 	{"drill-helper-size", "Diameter of the small hole when drill-helper is on",
-	 RND_HATT_COORD, 0, RND_MM_TO_COORD(10), {0, 0, 0, PCB_MIN_PINORVIAHOLE}, 0, 0},
+	 RND_HATT_COORD, 0, RND_MM_TO_COORD(10), {0, 0, 0, RND_MIL_TO_COORD(4)}, 0},
 #define HA_drillhelpersize 19
 
 
 	{"cam", "CAM instruction",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_cam 20
 
 };
@@ -339,12 +339,12 @@ static struct {
 	long drawn_objs;
 } global;
 
-static rnd_export_opt_t *ps_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *ps_get_export_options(rnd_hid_t *hid, int *n)
 {
-	char **val = ps_attribute_list[HA_psfile].value;
+	const char *val = global.ps_values[HA_psfile].str;
 
-	if ((PCB != NULL) && ((val == NULL) || (*val == NULL) || (**val == '\0')))
-		pcb_derive_default_filename(PCB->hidlib.filename, &ps_attribute_list[HA_psfile], ".ps");
+	if ((PCB != NULL) && ((val == NULL) || (*val == '\0')))
+		pcb_derive_default_filename(PCB->hidlib.filename, &global.ps_values[HA_psfile], ".ps");
 
 	if (n)
 		*n = NUM_OPTIONS;
@@ -1248,182 +1248,12 @@ static void ps_fill_rect(rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord_t y1, rnd_co
 	rnd_fprintf(global.f, "%mi %mi %mi %mi r\n", x1, y1, x2, y2);
 }
 
-rnd_hid_attribute_t ps_calib_attribute_list[] = {
-	{"lprcommand", "Command to print",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
-};
-
-static const char *const calib_lines[] = {
-	"%!PS-Adobe-3.0\n",
-	"%%Title: Calibration Page\n",
-	"%%PageOrder: Ascend\n",
-	"%%Pages: 1\n",
-	"%%EndComments\n",
-	"\n",
-	"%%Page: Calibrate 1\n",
-	"72 72 scale\n",
-	"\n",
-	"0 setlinewidth\n",
-	"0.375 0.375 moveto\n",
-	"8.125 0.375 lineto\n",
-	"8.125 10.625 lineto\n",
-	"0.375 10.625 lineto\n",
-	"closepath stroke\n",
-	"\n",
-	"0.5 0.5 translate\n",
-	"0.001 setlinewidth\n",
-	"\n",
-	"/Times-Roman findfont 0.2 scalefont setfont\n",
-	"\n",
-	"/sign {\n",
-	"    0 lt { -1 } { 1 } ifelse\n",
-	"} def\n",
-	"\n",
-	"/cbar {\n",
-	"    /units exch def\n",
-	"    /x exch def\n",
-	"    /y exch def  \n",
-	"\n",
-	"    /x x sign 0.5 mul def\n",
-	"\n",
-	"    0 setlinewidth\n",
-	"    newpath x y 0.25 0 180 arc gsave 0.85 setgray fill grestore closepath stroke\n",
-	"    newpath x 0 0.25 180 360 arc gsave 0.85 setgray fill grestore closepath stroke\n",
-	"    0.001 setlinewidth\n",
-	"\n",
-	"    x 0 moveto\n",
-	"    x y lineto\n",
-	"%    -0.07 -0.2 rlineto 0.14 0 rmoveto -0.07 0.2 rlineto\n",
-	"    x y lineto\n",
-	"    -0.1 0 rlineto 0.2 0 rlineto\n",
-	"    stroke\n",
-	"    x 0 moveto\n",
-	"%    -0.07 0.2 rlineto 0.14 0 rmoveto -0.07 -0.2 rlineto\n",
-	"    x 0 moveto\n",
-	"    -0.1 0 rlineto 0.2 0 rlineto\n",
-	"     stroke\n",
-	"\n",
-	"    x 0.1 add\n",
-	"    y 0.2 sub moveto\n",
-	"    units show\n",
-	"} bind def\n",
-	"\n",
-	"/y 9 def\n",
-	"/t {\n",
-	"    /str exch def\n",
-	"    1.5 y moveto str show\n",
-	"    /y y 0.25 sub def\n",
-	"} bind def\n",
-	"\n",
-	"(Please measure ONE of the horizontal lines, in the units indicated for)t\n",
-	"(that line, and enter that value as X.  Similarly, measure ONE of the)t\n",
-	"(vertical lines and enter that value as Y.  Measurements should be)t\n",
-	"(between the flat faces of the semicircles.)t\n",
-	"()t\n",
-	"(The large box is 10.25 by 7.75 inches)t\n",
-	"\n",
-	"/in { } bind def\n",
-	"/cm { 2.54 div } bind def\n",
-	"/mm { 25.4 div } bind def\n",
-	"\n",
-	0
-};
-
-static int guess(double val, double close_to, double *calib)
-{
-	if (val >= close_to * 0.9 && val <= close_to * 1.1) {
-		*calib = close_to / val;
-		return 0;
-	}
-	return 1;
-}
-
-void ps_calibrate_1(rnd_hid_t *hid, double xval, double yval, int use_command)
-{
-	FILE *ps_cal_file;
-	int used_popen = 0, c;
-
-	if (xval > 0 && yval > 0) {
-		if (guess(xval, 4, &global.calibration_x))
-			if (guess(xval, 15, &global.calibration_x))
-				if (guess(xval, 7.5, &global.calibration_x)) {
-					if (xval < 2)
-						ps_attribute_list[HA_xcalib].default_val.dbl = global.calibration_x = xval;
-					else
-						rnd_message(RND_MSG_ERROR, "X value of %g is too far off.\n" "Expecting it near: 1.0, 4.0, 15.0, 7.5\n", xval);
-				}
-		if (guess(yval, 4, &global.calibration_y))
-			if (guess(yval, 20, &global.calibration_y))
-				if (guess(yval, 10, &global.calibration_y)) {
-					if (yval < 2)
-						ps_attribute_list[HA_ycalib].default_val.dbl = global.calibration_y = yval;
-					else
-						rnd_message(RND_MSG_ERROR, "Y value of %g is too far off.\n" "Expecting it near: 1.0, 4.0, 20.0, 10.0\n", yval);
-				}
-		return;
-	}
-
-	if (ps_calib_attribute_list[0].val.str == NULL) {
-		ps_calib_attribute_list[0].val.str = rnd_strdup("lpr");
-	}
-
-	if (rnd_attribute_dialog("ps_calibrate", ps_calib_attribute_list, 1, "Print Calibration Page", NULL))
-		return;
-
-	if (ps_calib_attribute_list[0].val.str == NULL)
-		return;
-
-	if (use_command || strchr(ps_calib_attribute_list[0].val.str, '|')) {
-		const char *cmd = ps_calib_attribute_list[0].val.str;
-		while (*cmd == ' ' || *cmd == '|')
-			cmd++;
-		ps_cal_file = rnd_popen(&PCB->hidlib, cmd, "w");
-		used_popen = 1;
-	}
-	else
-		ps_cal_file = rnd_fopen(&PCB->hidlib, ps_calib_attribute_list[0].val.str, "w");
-
-	for (c = 0; calib_lines[c]; c++)
-		fputs(calib_lines[c], ps_cal_file);
-
-	fprintf(ps_cal_file, "4 in 0.5 (Y in) cbar\n");
-	fprintf(ps_cal_file, "20 cm 1.5 (Y cm) cbar\n");
-	fprintf(ps_cal_file, "10 in 2.5 (Y in) cbar\n");
-	fprintf(ps_cal_file, "-90 rotate\n");
-	fprintf(ps_cal_file, "4 in -0.5 (X in) cbar\n");
-	fprintf(ps_cal_file, "15 cm -1.5 (X cm) cbar\n");
-	fprintf(ps_cal_file, "7.5 in -2.5 (X in) cbar\n");
-
-	fprintf(ps_cal_file, "showpage\n");
-
-	fprintf(ps_cal_file, "%%%%EOF\n");
-
-	if (used_popen)
-		rnd_pclose(ps_cal_file);
-	else
-		fclose(ps_cal_file);
-}
-
-static void ps_calibrate(rnd_hid_t *hid, double xval, double yval)
-{
-	ps_calibrate_1(hid, xval, yval, 0);
-}
-
 static void ps_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y, int action)
 {
 }
 
 rnd_hid_t ps_hid;
 
-static fgw_error_t pcb_act_PSCalib(fgw_arg_t *res, int argc, fgw_arg_t *argv)
-{
-	ps_calibrate(&ps_hid, 0.0, 0.0);
-	return 0;
-}
-
-rnd_action_t hidps_action_list[] = {
-	{"pscalib", pcb_act_PSCalib, NULL, NULL}
-};
 
 static int ps_inited = 0;
 void ps_ps_init(rnd_hid_t * hid)
@@ -1450,11 +1280,8 @@ void ps_ps_init(rnd_hid_t * hid)
 	hid->fill_polygon_offs = ps_fill_polygon_offs;
 	hid->fill_polygon = ps_fill_polygon;
 	hid->fill_rect = ps_fill_rect;
-	hid->calibrate = ps_calibrate;
 	hid->set_crosshair = ps_set_crosshair;
 
-	RND_REGISTER_ACTIONS(hidps_action_list, ps_cookie)
-
 	ps_inited = 1;
 }
 
@@ -1501,6 +1328,7 @@ int pplg_init_export_ps(void)
 	ps_hid.usage = ps_usage;
 
 	rnd_hid_register_hid(&ps_hid);
+	rnd_hid_load_defaults(&ps_hid, ps_attribute_list, NUM_OPTIONS);
 
 	hid_eps_init();
 	return 0;
