@@ -50,41 +50,41 @@
 static rnd_hid_t stl_hid;
 const char *stl_cookie = "export_stl HID";
 
-rnd_export_opt_t stl_attribute_list[] = {
+static const rnd_export_opt_t stl_attribute_list[] = {
 	{"outfile", "STL output file",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_stlfile 0
 
 	{"models", "enable searching and inserting model files",
-	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0},
 #define HA_models 1
 
 	{"min-drill", "minimum circular hole diameter to render (smaller ones are not drawn)",
-	 RND_HATT_COORD, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_COORD, 0, 0, {0, 0, 0}, 0},
 #define HA_mindrill 2
 
 	{"min-slot-line", "minimum thickness of padstakc slots specified as lines (smaller ones are not drawn)",
-	 RND_HATT_COORD, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_COORD, 0, 0, {0, 0, 0}, 0},
 #define HA_minline 3
 
 	{"slot-poly", "draw cutouts for slots specified as padstack polygons",
-	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0},
 #define HA_slotpoly 4
 
 	{"cutouts", "draw cutouts drawn on 'route' layer groups (outline/slot mech layer groups)",
-	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {1, 0, 0}, 0},
 #define HA_cutouts 5
 
 	{"override-thickness", "override calculated board thickness (when non-zero)",
-	 RND_HATT_COORD, 0, 0, {1, 0, 0}, 0, 0},
+	 RND_HATT_COORD, 0, 0, {1, 0, 0}, 0},
 #define HA_ovrthick 6
 
 	{"z-center", "when true: z=0 is the center of board cross section, instead of being at the bottom side",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_zcent 7
 
 	{"cam", "CAM instruction",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_cam 8
 };
 
@@ -92,13 +92,13 @@ rnd_export_opt_t stl_attribute_list[] = {
 
 static rnd_hid_attr_val_t stl_values[NUM_OPTIONS];
 
-static rnd_export_opt_t *stl_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *stl_get_export_options(rnd_hid_t *hid, int *n)
 {
 	const char *suffix = ".stl";
-	char **val = stl_attribute_list[HA_stlfile].value;
+	const char *val = stl_values[HA_stlfile].str;
 
-	if ((PCB != NULL) && ((val == NULL) || (*val == NULL) || (**val == '\0')))
-		pcb_derive_default_filename(PCB->hidlib.filename, &stl_attribute_list[HA_stlfile], suffix);
+	if ((PCB != NULL) && ((val == NULL) || (*val == '\0')))
+		pcb_derive_default_filename(PCB->hidlib.filename, &stl_values[HA_stlfile], suffix);
 
 	if (n)
 		*n = NUM_OPTIONS;
@@ -315,8 +315,12 @@ static long estimate_cutout_pts(pcb_board_t *pcb, vtp0_t *cutouts, pcb_dynf_t df
 		PCB_LINE_LOOP(layer) {
 			if (PCB_DFLAG_TEST(&line->Flags, df)) continue; /* object already found - either as outline or as a cutout */
 			poly = pcb_topoly_conn_with(pcb, (pcb_any_obj_t *)line, PCB_TOPOLY_FLOATING, df);
-			vtp0_append(cutouts, poly);
-			cnt += poly_len(poly);
+			if (poly != NULL) {
+				vtp0_append(cutouts, poly);
+				cnt += poly_len(poly);
+			}
+			else
+				rnd_message(RND_MSG_ERROR, "Cutout error: need closed loops; cutout omitted\n(Hint: use the wireframe draw mode to see broken connections; use a coarse grid and snap to fix them up!)\n");
 /*			rnd_trace(" line: %ld %d -> %p\n", line->ID, PCB_DFLAG_TEST(&line->Flags, df), poly);*/
 		} PCB_END_LOOP;
 		PCB_ARC_LOOP(layer) {
@@ -384,6 +388,12 @@ int stl_hid_export_to_file(FILE *f, rnd_hid_attr_val_t *options, rnd_coord_t max
 	pcb_data_dynflag_clear(PCB->Data, df);
 	brdpoly = pcb_topoly_1st_outline_with(PCB, PCB_TOPOLY_FLOATING, df);
 
+	if (brdpoly == NULL) {
+		rnd_message(RND_MSG_ERROR, "Contour/outline error: need closed loops\n(Hint: use the wireframe draw mode to see broken connections; use a coarse grid and snap to fix them up!)\n");
+		pcb_dynflag_free(df);
+		return -1;
+	}
+
 	pstk_points = estimate_hole_pts_pstk(PCB, toply, options);
 	cutout_points = estimate_cutout_pts(PCB, &cutouts, df, options);
 
@@ -552,6 +562,7 @@ int pplg_init_export_stl(void)
 	stl_hid.usage = stl_usage;
 
 	rnd_hid_register_hid(&stl_hid);
+	rnd_hid_load_defaults(&stl_hid, stl_attribute_list, NUM_OPTIONS);
 
 	return 0;
 }
