@@ -125,7 +125,7 @@ static struct {
 	/* INNER */  { "#222222", "#111111", "#000000", RND_MM_TO_COORD(0) }
 };
 
-rnd_export_opt_t svg_attribute_list[] = {
+static const rnd_export_opt_t svg_attribute_list[] = {
 	/* other HIDs expect this to be first.  */
 
 /* %start-doc options "93 SVG Options"
@@ -136,7 +136,7 @@ Name of the file to be exported to. Can contain a path.
 %end-doc
 */
 	{"outfile", "Graphics output file",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_svgfile 0
 
 /* %start-doc options "93 SVG Options"
@@ -148,7 +148,7 @@ Export a photo realistic image of the layout.
 %end-doc
 */
 	{"photo-mode", "Photo-realistic export mode",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_photo_mode 1
 
 /* %start-doc options "93 SVG Options"
@@ -160,7 +160,7 @@ Layer opacity
 %end-doc
 */
 	{"opacity", "Layer opacity",
-	 RND_HATT_INTEGER, 0, 100, {100, 0, 0}, 0, 0},
+	 RND_HATT_INTEGER, 0, 100, {100, 0, 0}, 0},
 #define HA_opacity 2
 
 /* %start-doc options "93 SVG Options"
@@ -172,19 +172,19 @@ Flip board, look at it from the bottom side
 %end-doc
 */
 	{"flip", "Flip board",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_flip 3
 
 	{"as-shown", "Render similar to as shown on screen (display overlays)",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_as_shown 4
 
 	{"true-size", "Attempt to preserve true size for printing",
-	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_BOOL, 0, 0, {0, 0, 0}, 0},
 #define HA_true_size 5
 
 	{"cam", "CAM instruction",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0}
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0}
 #define HA_cam 6
 };
 
@@ -199,13 +199,13 @@ do { \
 
 static rnd_hid_attr_val_t svg_values[NUM_OPTIONS];
 
-static rnd_export_opt_t *svg_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *svg_get_export_options(rnd_hid_t *hid, int *n)
 {
 	const char *suffix = ".svg";
-	char **val = svg_attribute_list[HA_svgfile].value;
+	const char *val = svg_values[HA_svgfile].str;
 
-	if ((PCB != NULL) && ((val == NULL) || (*val == NULL) || (**val == '\0')))
-		pcb_derive_default_filename(PCB->hidlib.filename, &svg_attribute_list[HA_svgfile], suffix);
+	if ((PCB != NULL) && ((val == NULL) || (*val == '\0')))
+		pcb_derive_default_filename(PCB->hidlib.filename, &svg_values[HA_svgfile], suffix);
 
 	if (n)
 		*n = NUM_OPTIONS;
@@ -259,6 +259,8 @@ void svg_hid_export_to_file(FILE * the_file, rnd_hid_attr_val_t * options, rnd_x
 
 		/* disable (exporter default) hiding overlay in as_shown */
 		xform->omit_overlay = 0;
+		xform->add_gui_xform = 1;
+		xform->enable_silk_invis_clr = 1;
 	}
 
 	rnd_expose_main(&svg_hid, &ctx, xform);
@@ -872,12 +874,6 @@ static void svg_fill_polygon(rnd_hid_gc_t gc, int n_coords, rnd_coord_t *x, rnd_
 }
 
 
-static void svg_calibrate(rnd_hid_t *hid, double xval, double yval)
-{
-	rnd_message(RND_MSG_ERROR, "svg_calibrate() not implemented");
-	return;
-}
-
 static void svg_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y, int a)
 {
 }
@@ -929,13 +925,13 @@ int pplg_init_export_svg(void)
 	svg_hid.fill_polygon = svg_fill_polygon;
 	svg_hid.fill_polygon_offs = svg_fill_polygon_offs;
 	svg_hid.fill_rect = svg_fill_rect;
-	svg_hid.calibrate = svg_calibrate;
 	svg_hid.set_crosshair = svg_set_crosshair;
 	svg_hid.argument_array = svg_values;
 
 	svg_hid.usage = svg_usage;
 
 	rnd_hid_register_hid(&svg_hid);
+	rnd_hid_load_defaults(&svg_hid, svg_attribute_list, NUM_OPTIONS);
 
 	return 0;
 }
