@@ -47,6 +47,7 @@ const char *xy_cookie = "XY HID";
 #define MAX_TEMP_NAME_LEN 128
 
 
+/* This one can not be const because format enumeration is loaded run-time */
 static rnd_export_opt_t xy_options[] = {
 /* %start-doc options "8 XY Creation"
 @ftable @code
@@ -56,7 +57,7 @@ Name of the XY output file.
 %end-doc
 */
 	{"xyfile", "Name of the XY output file",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_xyfile 0
 
 /* %start-doc options "8 XY Creation"
@@ -67,14 +68,14 @@ Unit of XY dimensions. Defaults to mil.
 %end-doc
 */
 	{"xy-unit", "XY units",
-	 RND_HATT_UNIT, 0, 0, {-1, 0, 0}, NULL, 0},
+	 RND_HATT_UNIT, 0, 0, {-1, 0, 0}, NULL},
 #define HA_unit 1
 	{"format", "file format (template)",
-	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, NULL, 0},
+	 RND_HATT_ENUM, 0, 0, {0, 0, 0}, NULL},
 #define HA_format 2
 
 	{"cam", "CAM instruction",
-	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0, 0},
+	 RND_HATT_STRING, 0, 0, {0, 0, 0}, 0},
 #define HA_cam 3
 
 };
@@ -97,11 +98,11 @@ static void free_fmts(void)
 	}
 }
 
-static rnd_export_opt_t *xy_get_export_options(rnd_hid_t *hid, int *n)
+static const rnd_export_opt_t *xy_get_export_options(rnd_hid_t *hid, int *n)
 {
 	static int last_unit_value = -1;
 	rnd_conf_listitem_t *li;
-	char **val = xy_options[HA_xyfile].value;
+	const char *val = xy_values[HA_xyfile].str;
 	int idx;
 
 	/* load all formats from the config */
@@ -139,16 +140,16 @@ static rnd_export_opt_t *xy_get_export_options(rnd_hid_t *hid, int *n)
 	xy_options[HA_format].enumerations = (const char **)fmt_names.array;
 
 	/* set default unit and filename */
-	if (xy_options[HA_unit].default_val.lng == last_unit_value) {
+	if (xy_values[HA_unit].lng == last_unit_value) {
 		if (rnd_conf.editor.grid_unit)
-			xy_options[HA_unit].default_val.lng = rnd_conf.editor.grid_unit->index;
+			xy_values[HA_unit].lng = rnd_conf.editor.grid_unit->index;
 		else
-			xy_options[HA_unit].default_val.lng = rnd_get_unit_struct("mil")->index;
-		last_unit_value = xy_options[HA_unit].default_val.lng;
+			xy_values[HA_unit].lng = rnd_get_unit_struct("mil")->index;
+		last_unit_value = xy_values[HA_unit].lng;
 	}
 
-	if ((PCB != NULL) && ((val == NULL) || (*val == NULL) || (**val == '\0')))
-		pcb_derive_default_filename(PCB->hidlib.filename, &xy_options[HA_xyfile], ".xy");
+	if ((PCB != NULL) && ((val == NULL) || (*val == '\0')))
+		pcb_derive_default_filename(PCB->hidlib.filename, &xy_values[HA_xyfile], ".xy");
 
 	if (n)
 		*n = NUM_OPTIONS;
@@ -813,7 +814,7 @@ static void xy_do_export(rnd_hid_t *hid, rnd_hid_attr_val_t *options)
 
 	pcb_cam_begin_nolayer(PCB, &cam, NULL, options[HA_cam].str, &xy_filename);
 
-	if (options[HA_unit].lng == -1)
+	if ((options[HA_unit].lng < 0) || (options[HA_unit].lng >= rnd_get_n_units(1)))
 		xy_unit = rnd_get_unit_struct("mil");
 	else
 		xy_unit = &rnd_units[options[HA_unit].lng];
@@ -888,6 +889,7 @@ int pplg_init_export_xy(void)
 	xy_hid.usage = xy_usage;
 
 	rnd_hid_register_hid(&xy_hid);
+	rnd_hid_load_defaults(&xy_hid, xy_options, NUM_OPTIONS);
 
 	vts0_init(&fmt_names);
 	vts0_init(&fmt_ids);
