@@ -57,7 +57,6 @@
 
 #include "fp_fs_conf.h"
 
-#define FP_FS_CONF_FN "ar_extern.conf"
 static conf_fp_fs_t conf_fp_fs;
 static const char fp_fs_cookie[] = "fp_fs plugin";
 
@@ -186,9 +185,10 @@ static int fp_fs_ignore_fn(const char *fn, int len)
 
 	rnd_conf_loop_list_str(&conf_fp_fs.plugins.fp_fs.ignore_prefix, ci, p, n) {
 		const char *s1, *s2;
-		for(s1 = fn, s2 = p; *s1 == *s2; s1++, s2++) {
+		for(s1 = fn, s2 = p; ; s1++, s2++) {
 			if (*s2 == '\0') return 1;
 			if (*s1 == '\0') break;
+			if (*s1 != *s2) break;
 		}
 	}
 
@@ -569,7 +569,7 @@ void pplg_uninit_fp_fs(void)
 
 	rnd_conf_hid_unreg(fp_fs_cookie);
 
-	rnd_conf_unreg_file(FP_FS_CONF_FN, fp_fs_conf_internal);
+	rnd_conf_unreg_intern(fp_fs_conf_internal);
 
 	fp_fs_cache_uninit(&fp_fs_cache);
 	rnd_conf_unreg_fields("plugins/fp_fs/");
@@ -588,7 +588,7 @@ int pplg_init_fp_fs(void)
 	RND_HOOK_REGISTER(pcb_plug_fp_t, pcb_plug_fp_chain, &fp_fs);
 	htsp_init(&fp_fs_cache, strhash, strkeyeq);
 
-	rnd_conf_reg_file(FP_FS_CONF_FN, fp_fs_conf_internal);
+	rnd_conf_reg_intern(fp_fs_conf_internal);
 
 #define conf_reg(field,isarray,type_name,cpath,cname,desc,flags) \
 	rnd_conf_reg_field(conf_fp_fs, field,isarray,type_name,cpath,cname,desc,flags);
