@@ -2,7 +2,7 @@
  *                            COPYRIGHT
  *
  *  pcb-rnd, interactive printed circuit board design
- *  Copyright (C) 2016..2020 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2016..2021 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -64,6 +64,7 @@
 #include "../src_plugins/lib_compat_help/subc_help.h"
 #include "../src_plugins/lib_compat_help/pstk_compat.h"
 #include "../src_plugins/lib_compat_help/elem_rot.h"
+#include "../src_plugins/lib_compat_help/route_style.h"
 
 typedef struct {
 	vtp0_t post_ids, post_thermal_old, post_thermal_heavy;
@@ -119,7 +120,7 @@ static void iolht_warn(lht_read_t *rctx, lht_node_t *nd, int wbit, char *fmt, ..
 	gds_init(&str);
 	gds_append_str(&str, "io_lihata parse warning");
 	if (nd != NULL)
-		rnd_append_printf(&str, "at %s:%d.%d: ", nd->file_name, nd->line, nd->col);
+		rnd_append_printf(&str, " at %s:%d.%d: ", nd->file_name, nd->line, nd->col);
 	else
 		gds_append_str(&str, ": ");
 
@@ -1028,7 +1029,7 @@ static int parse_polygon(lht_read_t *rctx, pcb_layer_t *ly, lht_node_t *obj)
 static int parse_pcb_text(lht_read_t *rctx, pcb_layer_t *ly, lht_node_t *obj)
 {
 	pcb_text_t *text;
-	lht_node_t *role, *nthickness, *nrot, *ndir;
+	lht_node_t *role, *nthickness, *nclearance, *nrot, *ndir;
 	int tmp, err = 0, dir;
 	unsigned char intconn = 0;
 	long int id;
@@ -1066,6 +1067,7 @@ static int parse_pcb_text(lht_read_t *rctx, pcb_layer_t *ly, lht_node_t *obj)
 	err |= parse_text(&text->TextString, hash_get(obj, "string", 0));
 
 	nthickness = lht_dom_hash_get(obj, "thickness");
+	nclearance = lht_dom_hash_get(obj, "clearance");
 	nrot = lht_dom_hash_get(obj, "rot");
 	ndir = lht_dom_hash_get(obj, "direction");
 
@@ -1078,6 +1080,14 @@ static int parse_pcb_text(lht_read_t *rctx, pcb_layer_t *ly, lht_node_t *obj)
 	else
 		text->thickness = 0;
 
+	if (nclearance != NULL) {
+		if (rctx->rdver < 8)
+			iolht_warn(rctx, nthickness, -1, "Text clearance should not be present in a file with version lower than v8\n");
+		err |= parse_coord(&text->clearance, nclearance);
+	}
+	else
+		text->clearance = 0;
+
 	if ((ndir != NULL) && (rctx->rdver >= 6))
 		iolht_warn(rctx, nthickness, -1, "Text direction should not be present in a file with version higher than v5 - use text rot instead\n");
 
@@ -1541,7 +1551,6 @@ static int parse_element(lht_read_t *rctx, pcb_board_t *pcb, pcb_data_t *dt, lht
 	else
 		return iolht_error(obj, "invalid element: objects is not a list\n");
 
-TODO("subc: TextFlags")
 	txt = pcb_subc_add_refdes_text(subc, tx, ty, tdir, tscale, onsld);
 
 	pcb_subc_xy_rot_pnp(subc, ox, oy, onsld);
@@ -2285,6 +2294,8 @@ static int parse_styles(lht_read_t *rctx, pcb_data_t *dt, vtroutestyle_t *styles
 		pcb_route_style_t *s = vtroutestyle_alloc_append(styles, 1);
 		int name_len = strlen(stn->name);
 
+		s->fid = -1;
+
 		if (stn->type != LHT_HASH)
 			return iolht_error(stn, "route style entry must be a hash\n");
 		
@@ -2299,12 +2310,9 @@ static int parse_styles(lht_read_t *rctx, pcb_data_t *dt, vtroutestyle_t *styles
 
 		s->via_proto = 0;
 		s->via_proto_set = 0;
-		s->Hole = 0;
-		s->Diameter = 0;
 		err |= parse_coord(&s->Thick,     hash_get(stn, "thickness", 0));
-		err |= parse_coord(&s->Diameter,  hash_get(stn, "diameter", 1));
-		err |= parse_coord(&s->Hole,      hash_get(stn, "hole", 1));
 		err |= parse_coord(&s->Clearance, hash_get(stn, "clearance", 0));
+
 		parse_attributes(&s->attr, lht_dom_hash_get(stn, "attributes"));
 
 		/* read text thickness */
@@ -2340,7 +2348,49 @@ static int parse_styles(lht_read_t *rctx, pcb_data_t *dt, vtroutestyle_t *styles
 				s->via_proto = pid;
 				s->via_proto_set = 1;
 				if (pcb_pstk_get_proto_(dt, pid) == NULL)
-					iolht_warn(rctx, vp, -1, "Route style %s references to non-existent prototype %ld\n", s->via_proto);
+					iolht_warn(rctx, vp, -1, "Route style %s references to non-existent prototype %ld\n", s->name, s->via_proto);
+
+				if ((hash_get(stn, "diameter", 1) != &missing_ok) || (hash_get(stn, "hole", 1) != &missing_ok))
+					iolht_warn(rctx, stn, -1, "v8+ route style with via padstack proto shall not have diameters in %s: diametres are ignored, proto is used\n", s->name);
+			}
+		}
+
+		if (!s->via_proto_set) {
+			rnd_coord_t drill_dia, pad_dia, mask = 0;
+			int err = 0;
+			lht_node_t *dn, *hn;
+
+			dn = hash_get(stn, "diameter", 1);
+			hn = hash_get(stn, "hole", 1);
+
+			if ((dn != NULL) || (hn != NULL)) {
+				if (rctx->rdver >= 8)
+					iolht_warn(rctx, stn, -1, "v8+ route style sould not have via diameters in %s: it should use via proto\n(Maybe it is a manually written file? Converting diameters to prototypes...)\n", s->name);
+
+				/* import old, diameter based via geometry from old files */
+				err |= parse_coord(&pad_dia,   dn);
+				err |= parse_coord(&drill_dia, hn);
+
+				if ((err == 0) && ((drill_dia <= 0) || (pad_dia <= 0) || (pad_dia <= drill_dia))) {
+					iolht_error(stn, "Invalid route style prototype via diameters: copper or hole dia too small\n(Via ignored for routing style '%s')\n", s->name);
+					s->via_proto = -1;
+				}
+				else if ((err == 0) && (drill_dia > 0) && (pad_dia > 0) && (pad_dia > drill_dia)) {
+					if (pcb_compat_route_style_via_load(dt, s, drill_dia, pad_dia, mask) != 0)
+						iolht_warn(rctx, stn, -1, "Invalid route style via diameters in %s: can not create padstack proto\n", s->name);
+				}
+				else
+					return iolht_error(stn, "Invalid route style prototype via diameters: syntax error\n");
+			}
+		}
+
+		if (rctx->rdver >= 8) {
+			lht_node_t *fidn = lht_dom_hash_get(stn, "fid");
+			if (fidn != NULL) {
+				unsigned long fid;
+				if (parse_ulong(&fid, fidn) != 0)
+					return iolht_error(stn, "Invalid route style font ID\n");
+				s->fid = fid;
 			}
 		}
 	}
@@ -2482,6 +2532,7 @@ static int parse_board(lht_read_t *rctx, pcb_board_t *pcb, lht_node_t *nd)
 		case 5: loader = &plug_io_lihata_v5; break;
 		case 6: loader = &plug_io_lihata_v6; break;
 		case 7: loader = &plug_io_lihata_v7; break;
+		case 8: loader = &plug_io_lihata_v8; break;
 		default:
 			return iolht_error(nd, "Lihata board version %d not supported;\n"
 				"must be 1, 2, 3, 4, 5, 6 or 7.\n", rctx->rdver);
