@@ -2,7 +2,7 @@
  *                            COPYRIGHT
  *
  *  pcb-rnd, interactive printed circuit board design
- *  Copyright (C) 2016..2019 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2016..2021 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -63,6 +63,7 @@
 #include <librnd/core/hid_dad.h>
 
 #include "src_plugins/lib_compat_help/pstk_compat.h"
+#include "src_plugins/lib_compat_help/route_style.h"
 
 /*#define CFMT "%[9]"*/
 #define CFMT "%.08$$mH"
@@ -229,6 +230,12 @@ static lht_node_t *build_flags(pcb_flag_t *f, int object_type, int intconn)
 
 	fh.Flags = *f;
 
+	if (conf_io_lihata.plugins.io_lihata.denoise) {
+		PCB_FLAG_CLEAR(PCB_FLAG_SELECTED, &fh);
+		PCB_FLAG_CLEAR(PCB_FLAG_WARN, &fh);
+		PCB_FLAG_CLEAR(PCB_FLAG_FOUND, &fh);
+	}
+
 	hsh = lht_dom_node_alloc(LHT_HASH, "flags");
 
 	/* create normal flag nodes */
@@ -751,19 +758,28 @@ static lht_node_t *build_pcb_text(const char *role, pcb_text_t *text)
 	lht_dom_hash_put(obj, build_textf("x", CFMT, text->X));
 	lht_dom_hash_put(obj, build_textf("y", CFMT, text->Y));
 
+	if (wrver >= 8) {
+		if (text->clearance != 0)
+			lht_dom_hash_put(obj, build_textf("clearance", CFMT, text->clearance));
+		else
+			lht_dom_hash_put(obj, dummy_node("clearance"));
+	}
+	else if (text->clearance != 0)
+		pcb_io_incompat_save(NULL, (pcb_any_obj_t *)text, "text-clearance", "file format does not support custom text clearance value", "do not use text clearance or save in lihata >=v8");
+
 	if (wrver >= 7) {
 		if (text->Scale != 0)
 			lht_dom_hash_put(obj, build_textf("scale", "%d", text->Scale));
 		else
-			lht_dom_hash_put(obj, dummy_text_node("scale"));
+			lht_dom_hash_put(obj, dummy_node("scale"));
 		if (text->scale_x != 0)
 			lht_dom_hash_put(obj, build_textf("scale_x", "%f", text->scale_x));
 		else
-			lht_dom_hash_put(obj, dummy_text_node("scale"));
+			lht_dom_hash_put(obj, dummy_node("scale_x"));
 		if (text->scale_y != 0)
 			lht_dom_hash_put(obj, build_textf("scale_y", "%f", text->scale_y));
 		else
-			lht_dom_hash_put(obj, dummy_text_node("scale"));
+			lht_dom_hash_put(obj, dummy_node("scale_y"));
 	}
 	else {
 		int scale;
@@ -1371,8 +1387,10 @@ static lht_node_t *build_data(pcb_data_t *data)
 	for(sc = pcb_subclist_first(&data->subc); sc != NULL; sc = pcb_subclist_next(sc))
 		lht_dom_list_append_safe(grp, build_subc(sc));
 
-	ratlist_foreach(&data->Rat, &it, line)
-		lht_dom_list_append(grp, build_rat(line));
+	if (!conf_io_lihata.plugins.io_lihata.denoise) {
+		ratlist_foreach(&data->Rat, &it, line)
+			lht_dom_list_append(grp, build_rat(line));
+	}
 
 	return ndt;
 }
@@ -1472,7 +1490,7 @@ static lht_node_t *build_fontkit(pcb_fontkit_t *fk)
 }
 
 
-static lht_node_t *build_styles(vtroutestyle_t *styles)
+static lht_node_t *build_styles(pcb_data_t *data, vtroutestyle_t *styles)
 {
 	lht_node_t *stl, *sn;
 	int n;
@@ -1486,18 +1504,20 @@ static lht_node_t *build_styles(vtroutestyle_t *styles)
 		sn = lht_dom_node_alloc(LHT_HASH, s->name);
 		lht_dom_list_append(stl, sn);
 		lht_dom_hash_put(sn, build_textf("thickness", CFMT, s->Thick));
-		lht_dom_hash_put(sn, build_textf("diameter", CFMT, s->Diameter));
-		lht_dom_hash_put(sn, build_textf("hole", CFMT, s->Hole));
 		lht_dom_hash_put(sn, build_textf("clearance", CFMT, s->Clearance));
 
-		if (wrver >= 5) {
+		if (wrver >= 8) {
 			if (s->via_proto_set)
 				lht_dom_hash_put(sn, build_textf("via_proto", "%ld", (long int)s->via_proto));
 			else
 			lht_dom_hash_put(sn, dummy_text_node("via_proto"));
 		}
-		else
-			pcb_io_incompat_save(NULL, NULL, "route-style", "lihata boards before version v5 did not support padstack prototype in route style\n", "Either save in lihata v5+ or be aware of losing this information");
+		else {
+			rnd_coord_t drill_dia, pad_dia, mask;
+			pcb_compat_route_style_via_save(data, s, &drill_dia, &pad_dia, &mask);
+			lht_dom_hash_put(sn, build_textf("diameter", CFMT, pad_dia));
+			lht_dom_hash_put(sn, build_textf("hole", CFMT, drill_dia));
+		}
 
 		if (wrver >= 6)
 			lht_dom_hash_put(sn, build_textf("text_thick", CFMT, s->textt));
@@ -1509,6 +1529,11 @@ static lht_node_t *build_styles(vtroutestyle_t *styles)
 		else if (s->texts > 0)
 			pcb_io_incompat_save(NULL, NULL, "route-style", "lihata boards before version v6 did not support text scale in route style\n", "Either save in lihata v6+ or be aware of losing this information");
 
+		if ((wrver >= 8) && (s->fid != -1))
+			lht_dom_hash_put(sn, build_textf("fid", "%ld", s->fid));
+		else if (s->fid != -1)
+			pcb_io_incompat_save(NULL, NULL, "route-style", "lihata boards before version v8 did not support setting font from style\n", "Either save in lihata v8+ or be aware of losing this information");
+
 		lht_dom_hash_put(sn, build_attributes(&s->attr));
 	}
 	return stl;
@@ -1544,8 +1569,11 @@ static lht_node_t *build_netlist(pcb_netlist_t *netlist, const char *name, int *
 
 			pl = lht_dom_node_alloc(LHT_LIST, "conn");
 			lht_dom_hash_put(nnet, pl);
-			if ((style != NULL) && (*style == '\0')) style = NULL;
-			lht_dom_hash_put(nnet, build_text("style", style));
+
+			if (wrver < 8) { /* there is no dedicated field for this since v8: the code relies on the attribute */
+				if ((style != NULL) && (*style == '\0')) style = NULL;
+				lht_dom_hash_put(nnet, build_text("style", style));
+			}
 
 			/* grow the connection list */
 			for(t = pcb_termlist_first(&net->conns); t != NULL; t = pcb_termlist_next(t)) {
@@ -1701,7 +1729,7 @@ static lht_doc_t *build_board(pcb_board_t *pcb)
 	lht_dom_hash_put(brd->root, build_pxms());
 	lht_dom_hash_put(brd->root, build_attributes(&pcb->Attributes));
 	lht_dom_hash_put(brd->root, build_fontkit(&pcb->fontkit));
-	lht_dom_hash_put(brd->root, build_styles(&pcb->RouteStyle));
+	lht_dom_hash_put(brd->root, build_styles(pcb->Data, &pcb->RouteStyle));
 	lht_dom_hash_put(brd->root, build_netlists(pcb, pcb->netlist, pcb->NetlistPatches, PCB_NUM_NETLISTS));
 	lht_dom_hash_put(brd->root, build_conf());
 	pxm_uninit();
@@ -1882,6 +1910,11 @@ int io_lihata_write_pcb_v7(pcb_plug_io_t *ctx, FILE * FP, const char *old_filena
 	return io_lihata_write_pcb(ctx, FP, old_filename, new_filename, emergency, 7);
 }
 
+int io_lihata_write_pcb_v8(pcb_plug_io_t *ctx, FILE * FP, const char *old_filename, const char *new_filename, rnd_bool emergency)
+{
+	return io_lihata_write_pcb(ctx, FP, old_filename, new_filename, emergency, 8);
+}
+
 int io_lihata_write_font(pcb_plug_io_t *ctx, pcb_font_t *font, const char *Filename)
 {
 	FILE *f;
@@ -1919,6 +1952,7 @@ static int plug2ver(const pcb_plug_io_t *ctx)
 	if (ctx == &plug_io_lihata_v5) return 5;
 	if (ctx == &plug_io_lihata_v6) return 6;
 	if (ctx == &plug_io_lihata_v7) return 7;
+	if (ctx == &plug_io_lihata_v8) return 8;
 	return 0;
 }
 
@@ -1952,8 +1986,10 @@ TODO("subc: for subc-in-subc this should be recursive")
 		doc->root = lht_dom_node_alloc(LHT_LIST, "pcb-rnd-subcircuit-v4");
 	else if (wrver == 6)
 		doc->root = lht_dom_node_alloc(LHT_LIST, "pcb-rnd-subcircuit-v6");
-	else if (wrver >= 7)
+	else if (wrver == 7)
 		doc->root = lht_dom_node_alloc(LHT_LIST, "pcb-rnd-subcircuit-v7");
+	else if (wrver >= 8)
+		doc->root = lht_dom_node_alloc(LHT_LIST, "pcb-rnd-subcircuit-v8");
 	else {
 		rnd_message(RND_MSG_ERROR, "Invalid lihata subc version to write: %d\n", wrver);
 		pxm_uninit();
@@ -2043,7 +2079,7 @@ int io_lihata_write_padstack(pcb_plug_io_t *ctx, FILE *f, pcb_pstk_proto_t *prot
 }
 
 typedef struct {
-	int womit_font, womit_config, womit_styles;
+	int womit_font, womit_config, womit_styles, wdenoise;
 	int ver;
 } io_lihata_save_t;
 
@@ -2053,25 +2089,43 @@ void *io_lihata_save_as_subd_init(const pcb_plug_io_t *ctx, rnd_hid_dad_subdialo
 
 	if (type == PCB_IOT_PCB) {
 		RND_DAD_BEGIN_HBOX(sub->dlg);
-			RND_DAD_LABEL(sub->dlg, "Omit font");
-				RND_DAD_HELP(sub->dlg, "Do not save the font subtree\nWARNING: this will make the board depend on\nthe default font available on systems\nwhere it is loaded; if multiple fonts\nare used, all will be displayed using the default");
-			RND_DAD_BOOL(sub->dlg);
-				RND_DAD_DEFAULT_NUM(sub->dlg, !!conf_io_lihata.plugins.io_lihata.omit_font);
-				save->womit_font = RND_DAD_CURRENT(sub->dlg);
-		RND_DAD_END(sub->dlg);
-		RND_DAD_BEGIN_HBOX(sub->dlg);
-			RND_DAD_LABEL(sub->dlg, "Omit config");
-				RND_DAD_HELP(sub->dlg, "Do not save the config subtree\nWARNING: this will lose all DESIGN role\nconfig setting in the resulting file");
-			RND_DAD_BOOL(sub->dlg);
-				RND_DAD_DEFAULT_NUM(sub->dlg, !!conf_io_lihata.plugins.io_lihata.omit_config);
-				save->womit_config = RND_DAD_CURRENT(sub->dlg);
-		RND_DAD_END(sub->dlg);
-		RND_DAD_BEGIN_HBOX(sub->dlg);
-			RND_DAD_LABEL(sub->dlg, "Omit styles");
-				RND_DAD_HELP(sub->dlg, "Do not save the routing style subtree\nThe resulting file will have no\nrouting styles");
-			RND_DAD_BOOL(sub->dlg);
-				RND_DAD_DEFAULT_NUM(sub->dlg, !!conf_io_lihata.plugins.io_lihata.omit_styles);
-				save->womit_styles = RND_DAD_CURRENT(sub->dlg);
+			RND_DAD_BEGIN_VBOX(sub->dlg); /* left column */
+				RND_DAD_BEGIN_HBOX(sub->dlg);
+					RND_DAD_LABEL(sub->dlg, "Omit font");
+						RND_DAD_HELP(sub->dlg, "Do not save the font subtree\nWARNING: this will make the board depend on\nthe default font available on systems\nwhere it is loaded; if multiple fonts\nare used, all will be displayed using the default");
+					RND_DAD_BOOL(sub->dlg);
+						RND_DAD_DEFAULT_NUM(sub->dlg, !!conf_io_lihata.plugins.io_lihata.omit_font);
+						save->womit_font = RND_DAD_CURRENT(sub->dlg);
+				RND_DAD_END(sub->dlg);
+				RND_DAD_BEGIN_HBOX(sub->dlg);
+					RND_DAD_LABEL(sub->dlg, "Omit config");
+						RND_DAD_HELP(sub->dlg, "Do not save the config subtree\nWARNING: this will lose all DESIGN role\nconfig setting in the resulting file");
+					RND_DAD_BOOL(sub->dlg);
+						RND_DAD_DEFAULT_NUM(sub->dlg, !!conf_io_lihata.plugins.io_lihata.omit_config);
+						save->womit_config = RND_DAD_CURRENT(sub->dlg);
+				RND_DAD_END(sub->dlg);
+			RND_DAD_END(sub->dlg);
+
+			RND_DAD_BEGIN_HBOX(sub->dlg);
+				RND_DAD_COMPFLAG(sub->dlg, RND_HATF_TIGHT | RND_HATF_FRAME);
+			RND_DAD_END(sub->dlg);
+
+			RND_DAD_BEGIN_VBOX(sub->dlg); /* right column */
+				RND_DAD_BEGIN_HBOX(sub->dlg);
+					RND_DAD_LABEL(sub->dlg, "Omit styles");
+						RND_DAD_HELP(sub->dlg, "Do not save the routing style subtree\nThe resulting file will have no\nrouting styles");
+					RND_DAD_BOOL(sub->dlg);
+						RND_DAD_DEFAULT_NUM(sub->dlg, !!conf_io_lihata.plugins.io_lihata.omit_styles);
+						save->womit_styles = RND_DAD_CURRENT(sub->dlg);
+				RND_DAD_END(sub->dlg);
+				RND_DAD_BEGIN_HBOX(sub->dlg);
+					RND_DAD_LABEL(sub->dlg, "Denoise");
+						RND_DAD_HELP(sub->dlg, "Do not save volatile data:\n - rats\n - select/warn/found flags");
+					RND_DAD_BOOL(sub->dlg);
+						RND_DAD_DEFAULT_NUM(sub->dlg, !!conf_io_lihata.plugins.io_lihata.denoise);
+						save->wdenoise = RND_DAD_CURRENT(sub->dlg);
+				RND_DAD_END(sub->dlg);
+			RND_DAD_END(sub->dlg);
 		RND_DAD_END(sub->dlg);
 	}
 	return save;
@@ -2085,6 +2139,7 @@ void io_lihata_save_as_subd_uninit(const pcb_plug_io_t *ctx, void *plg_ctx, rnd_
 		int omit_font = !!sub->dlg[save->womit_font].val.lng;
 		int omit_config = !!sub->dlg[save->womit_config].val.lng;
 		int omit_styles = !!sub->dlg[save->womit_styles].val.lng;
+		int denoise = !!sub->dlg[save->wdenoise].val.lng;
 
 		if (omit_font != !!conf_io_lihata.plugins.io_lihata.omit_font)
 			rnd_conf_setf(RND_CFR_CLI, "plugins/io_lihata/omit_font", 0, "%d", omit_font);
@@ -2094,6 +2149,9 @@ void io_lihata_save_as_subd_uninit(const pcb_plug_io_t *ctx, void *plg_ctx, rnd_
 
 		if (omit_styles != !!conf_io_lihata.plugins.io_lihata.omit_styles)
 			rnd_conf_setf(RND_CFR_CLI, "plugins/io_lihata/omit_styles", 0, "%d", omit_styles);
+
+		if (denoise != !!conf_io_lihata.plugins.io_lihata.denoise)
+			rnd_conf_setf(RND_CFR_CLI, "plugins/io_lihata/denoise", 0, "%d", denoise);
 	}
 
 	free(save);
