@@ -214,13 +214,17 @@ static lhtpers_rule_t r_line[] = {
 
 static const char *pat_lgrp1_line[] = {"te:lgrp1", "*", NULL};
 static const char *pat_lgrp2_line[] = {"te:lgrp2", "*", NULL};
+static const char *pat_anch1_line[] = {"te:anchor1", "*", NULL};
+static const char *pat_anch2_line[] = {"te:anchor2", "*", NULL};
 static lhtpers_rule_t r_rat[] = {
 	{pat_x1_line,    &style_inline, NULL},
 	{pat_y1_line,    &style_inline, NULL},
 	{pat_lgrp1_line, &style_inline, NULL},
+	{pat_anch1_line, &style_inline, NULL},
 	{pat_x2_line,    &style_inline, NULL},
 	{pat_y2_line,    &style_inline, NULL},
 	{pat_lgrp2_line, &style_inline, NULL},
+	{pat_anch2_line, &style_inline, NULL},
 	{lhtpers_early_end, &early_nl, NULL},
 	{pat_flags,      &style_nlstruct, r_thermal},
 	{pat_attributes, &style_nlstruct, NULL},
@@ -248,6 +252,29 @@ static lhtpers_rule_t r_arc[] = {
 	{NULL, NULL, NULL}
 };
 
+static const char *pat_gfx_ref[]       = {"te:pixmap_ref", "*", NULL};
+static const char *pat_gfx_sx[]        = {"te:sx", "*", NULL};
+static const char *pat_gfx_sy[]        = {"te:sy", "*", NULL};
+static const char *pat_gfx_cx[]        = {"te:cx", "*", NULL};
+static const char *pat_gfx_cy[]        = {"te:cy", "*", NULL};
+static const char *pat_gfx_rot[]       = {"te:rot", "*", NULL};
+static const char *pat_gfx_xmirror[]   = {"te:xmirror", "*", NULL};
+static const char *pat_gfx_ymirror[]   = {"te:ymirror", "*", NULL};
+static lhtpers_rule_t r_gfx[] = {
+	{pat_gfx_ref,         &style_inline, NULL},
+	{pat_gfx_cx,          &style_inline, NULL},
+	{pat_gfx_cy,          &style_inline, NULL},
+	{pat_gfx_sx,          &style_inline, NULL},
+	{pat_gfx_sy,          &style_inline, NULL},
+	{pat_gfx_rot,         &style_inline, NULL},
+	{pat_gfx_xmirror,     &style_inline, NULL},
+	{pat_gfx_ymirror,     &style_inline, NULL},
+	{lhtpers_early_end, &early_nl, NULL},
+	{pat_flags,      &style_nlstruct, r_thermal},
+	{pat_attributes, &style_nlstruct, NULL},
+	{NULL, NULL, NULL}
+};
+
 static const char *pat_name[] = {"te:name", "*", NULL};
 static const char *pat_numb[] = {"te:number", "*", NULL};
 static const char *pat_hole[] = {"te:hole", "*", NULL};
@@ -484,6 +511,11 @@ static const char *pat_ls_lyr1[] = {"te:*", "li:layers", "ha:*", "li:groups", "h
 static const char *pat_ls_lyr2[] = {"li:layers", "ha:*", "li:groups", "ha:layer_stack", "*", NULL};
 static const char *pat_ls_name[] = {"te:name", "ha:*", "li:groups", "ha:layer_stack", "*", NULL};
 static const char *pat_ls_grps[] = {"li:groups", "ha:layer_stack", "*", NULL};
+static const char *pat_pxm_ulzw[] = {"ha:ulzw.*", "ha:pixmaps", "*", NULL};
+static const char *pat_ulzw_sx[] = {"te:sx", "ha:ulzw.*", "*", NULL};
+static const char *pat_ulzw_sy[] = {"te:sy", "ha:ulzw.*", "*", NULL};
+static const char *pat_ulzw_transp[] = {"te:transparent", "ha:ulzw.*", "*", NULL};
+static const char *pat_ulzw_pixmap[] = {"te:pixmap", "ha:ulzw.*", "*", NULL};
 
 static lhtpers_rule_t r_layergrp[] = {
 	{pat_ls_name,  NULL, NULL},
@@ -492,11 +524,20 @@ static lhtpers_rule_t r_layergrp[] = {
 	{NULL, NULL, NULL}
 };
 
+static lhtpers_rule_t r_ulzw[] = {
+	{pat_ulzw_sx,  &style_inlinesp, NULL},
+	{pat_ulzw_sy,  &style_inline, NULL},
+	{pat_ulzw_transp,  &style_inline, NULL},
+	{pat_ulzw_pixmap,  &style_inline, NULL},
+	{NULL, NULL, NULL}
+};
+
 
 static const char *pat_line[] = {"ha:line.*", "*", NULL};
 static const char *pat_spoly[]= {"li:simplepoly.*", "*", NULL};
 static const char *pat_rat[]  = {"ha:rat.*", "*", NULL};
 static const char *pat_arc[]  = {"ha:arc.*", "*", NULL};
+static const char *pat_gfx[]  = {"ha:gfx.*", "*", NULL};
 static const char *pat_via[]  = {"ha:via.*", "*", NULL};
 static const char *pat_psref[]= {"ha:padstack_ref.*", "*", NULL};
 static const char *pat_pin[]  = {"ha:pin.*", "*", NULL};
@@ -532,6 +573,7 @@ static lhtpers_rule_t r_istructs[] = {
 	{pat_spoly,   &style_structi,  NULL},
 	{pat_rat,     &style_structi,  r_rat},
 	{pat_arc,     &style_structi,  r_arc},
+	{pat_gfx,     &style_structi,  r_gfx},
 	{pat_via,     &style_structi,  r_pinvia},
 	{pat_psref,   &style_structi,  r_psref},
 	{pat_pin,     &style_structi,  r_pinvia},
@@ -551,6 +593,7 @@ static lhtpers_rule_t r_istructs[] = {
 	{pat_layergrp,&style_structnp, r_layergrp},
 	{pat_ls_grps, &style_structnp, NULL},
 
+	{pat_pxm_ulzw, &style_structsp, r_ulzw},
 
 	{pat_cell,    &style_inline,   NULL},
 	{pat_thermt,  &style_inline,   NULL},
