@@ -122,6 +122,7 @@ typedef struct {
 	rnd_coord_t x, y;
 	rnd_coord_t width;
 	pads_line_type_t ltype;
+	pcb_layer_type_t lyt;
 	long level;
 	rnd_coord_t xo, yo;
 } pads_line_piece_t;
@@ -140,6 +141,7 @@ static int pads_parse_piece_crd(pads_read_ctx_t *rctx, pads_line_piece_t *lpc, l
 	if ((idx != 0) && ((lpc->x != x) || (lpc->y != y)))  {
 		pcb_dlcr_draw_t *line = pcb_dlcr_line_new(&rctx->dlcr, lpc->x, lpc->y, x, y, lpc->width, 0);
 		line->val.obj.layer_id = lpc->level;
+		line->val.obj.lyt = lpc->lyt;
 		line->loc_line = rctx->line;
 	}
 
@@ -165,6 +167,7 @@ static int pads_parse_piece_crd(pads_read_ctx_t *rctx, pads_line_piece_t *lpc, l
 		{
 			pcb_dlcr_draw_t *arc = pcb_dlcr_arc_new(&rctx->dlcr, cx, cy, r, starta, deltaa, lpc->width, 0);
 			arc->val.obj.layer_id = lpc->level;
+			arc->val.obj.lyt = lpc->lyt;
 			arc->loc_line = rctx->line;
 
 			/* need to determine the endpoint because of the incremental drawing;
@@ -195,11 +198,17 @@ static int pads_parse_piece_crd(pads_read_ctx_t *rctx, pads_line_piece_t *lpc, l
 	return 1;
 }
 
-static int pads_parse_piece_circle(pads_read_ctx_t *rctx, pads_line_piece_t *lpc, int poly)
+static int pads_parse_piece_circle(pads_read_ctx_t *rctx, pads_line_piece_t *lpc, long num_crds, int filled)
 {
-	rnd_coord_t x1, y1, x2, y2, cx, cy, r;
+	rnd_coord_t x1, y1, x2, y2, cx, cy, dummy;
 	pcb_dlcr_draw_t *arc;
-	int res;
+	int n, res;
+
+	if (num_crds < 2) {
+		PADS_ERROR((RND_MSG_ERROR, "A circle needs to have at least two coords\n"));
+		return -1;
+	}
+
 
 	if ((res = pads_read_coord(rctx, &x1)) <= 0) return res;
 	if ((res = pads_read_coord(rctx, &y1)) <= 0) return res;
@@ -208,24 +217,40 @@ static int pads_parse_piece_circle(pads_read_ctx_t *rctx, pads_line_piece_t *lpc
 	if ((res = pads_read_coord(rctx, &y2)) <= 0) return res;
 	pads_eatup_till_nl(rctx);
 
+	num_crds -= 2;
+	if (num_crds > 0) {
+		PADS_ERROR((RND_MSG_ERROR, "Circle with more than two coords - ignored\n(please report this bug with the file!)\n"));
+		for(n = 0; n < num_crds; n++) {
+			if ((res = pads_read_coord(rctx, &dummy)) <= 0) return res;
+			if ((res = pads_read_coord(rctx, &dummy)) <= 0) return res;
+			pads_eatup_till_nl(rctx);
+		}
+		exit(1);
+	}
+
+
 	cx = rnd_round((x1+x2)/2.0 + lpc->xo);
 	cy = rnd_round((y1+y2)/2.0 + lpc->yo);
-	r = rnd_round(rnd_distance(x1, y1, x2, y2)/2.0);
 
-	TODO("draw polygon circle");
-/*	if (poly) {
-	
+	if (filled) {
+		double d = rnd_round(rnd_distance(x1, y1, x2, y2));
+		pcb_dlcr_draw_t *line = pcb_dlcr_line_new(&rctx->dlcr, cx, cy, cx, cy, d, 0);
+		line->val.obj.layer_id = lpc->level;
+		line->val.obj.lyt = lpc->lyt;
+		line->loc_line = rctx->line;
 	}
-	else*/ {
+	else {
+		double r = rnd_round(rnd_distance(x1, y1, x2, y2)/2.0);
 		arc = pcb_dlcr_arc_new(&rctx->dlcr, cx, cy, r, 0, 360, lpc->width, 0);
 		arc->val.obj.layer_id = lpc->level;
+		arc->val.obj.lyt = lpc->lyt;
 		arc->loc_line = rctx->line;
 	}
 	return 1;
 }
 
 
-static int pads_parse_piece(pads_read_ctx_t *rctx, pads_line_type_t ltype, rnd_coord_t xo, rnd_coord_t yo)
+static int pads_parse_piece(pads_read_ctx_t *rctx, pads_line_type_t ltype, rnd_coord_t xo, rnd_coord_t yo, int in_partdecal)
 {
 	char ptype[32], rest[32];
 	long n, num_crds, layer = -100, lstyle;
@@ -236,7 +261,7 @@ static int pads_parse_piece(pads_read_ctx_t *rctx, pads_line_type_t ltype, rnd_c
 	if ((res = pads_read_word(rctx, ptype, sizeof(ptype), 0)) <= 0) return res;
 	if ((res = pads_read_long(rctx, &num_crds)) <= 0) return res;
 	if ((res = pads_read_coord(rctx, &width)) <= 0) return res;
-	if ((rctx->ver > 9.4) && (floor(rctx->ver) > 2007)) {
+	if (((rctx->ver >= 9.4) && (rctx->ver < 1000)) || (floor(rctx->ver) > 2007)) {
 		if ((res = pads_read_long(rctx, &lstyle)) <= 0) return res;
 		if ((res = pads_read_long(rctx, &layer)) <= 0) return res;
 		if (pads_has_field(rctx))
@@ -254,15 +279,24 @@ static int pads_parse_piece(pads_read_ctx_t *rctx, pads_line_type_t ltype, rnd_c
 	lpc.width = width;
 	lpc.ltype = ltype;
 	lpc.level = ltype == PLTY_BOARD ? PADS_LID_BOARD : layer;
+	lpc.lyt = 0;
 	lpc.xo = xo;
 	lpc.yo = yo;
 
-TODO("KEEPOUT objects are not handled");
+	if (in_partdecal) {
+		/* in partdecals layer 1 and 2 are special: top and bottom silk */
+		switch(lpc.level) {
+			case 1: lpc.lyt = PCB_LYT_TOP | PCB_LYT_SILK; lpc.level = PCB_DLCR_INVALID_LAYER_ID; break;
+			case 2: lpc.lyt = PCB_LYT_BOTTOM | PCB_LYT_SILK; lpc.level = PCB_DLCR_INVALID_LAYER_ID; break;
+		}
+	}
+
+TODO("''KEEPOUT'' objects are not handled: CIRCUT and COPCUT and COPCIR are polygon holes; requires powerpcb to check");
 	if ((strcmp(ptype, "CIRCLE") == 0) || (strcmp(ptype, "BRDCIR") == 0)) {
-		if ((res = pads_parse_piece_circle(rctx, &lpc, 0)) <= 0) return res;
+		if ((res = pads_parse_piece_circle(rctx, &lpc, num_crds, 0)) <= 0) return res;
 	}
 	else if ((strcmp(ptype, "COPCCO") == 0) || (strcmp(ptype, "COPCIR") == 0) || (strcmp(ptype, "CIRCUR") == 0)) {
-		if ((res = pads_parse_piece_circle(rctx, &lpc, 1)) <= 0) return res;
+		if ((res = pads_parse_piece_circle(rctx, &lpc, num_crds, 1)) <= 0) return res;
 	}
 	else {
 		for(n = 0; n < num_crds; n++)
@@ -272,16 +306,27 @@ TODO("KEEPOUT objects are not handled");
 	return 1;
 }
 
-static int pads_parse_text_(pads_read_ctx_t *rctx, int is_label, rnd_coord_t xo, rnd_coord_t yo)
+typedef enum { /* bitfield */
+	TEXT_IS_LABEL = 1,     /* subcircuit label which may be a dyntext */
+	TEXT_IN_LAST_SUBC = 2, /* in_subc means create text in the last subc placed */
+	TEXT_SKIP_REFDES = 4,
+	TEXT_SUBC_RELATIVE = 8
+} pads_text_flags_t;
+
+static int pads_parse_text(pads_read_ctx_t *rctx, rnd_coord_t xo, rnd_coord_t yo, pads_text_flags_t how)
 {
 	pcb_dlcr_draw_t *text;
 	char name[16], hjust[16], vjust[16], font[128], str[1024];
-	rnd_coord_t x, y, w, h;
-	double rot, scale;
-	long level;
+	rnd_coord_t x, y, thick, h, str_width;
+	double rot, anx = 0, any = 0;
+	long level, flg = 0;
 	int res, mirr = 0;
+	rnd_bool is_label = !!(how & TEXT_IS_LABEL);
+	rnd_bool in_subc  = !!(how & TEXT_IN_LAST_SUBC);
+	rnd_coord_t anchx, anchy;
+	pcb_text_mirror_t tmir = 0;
 
-	*font = *str = '\0';
+	*font = *str = *hjust = *vjust = '\0';
 
 	if (is_label)
 		if ((res = pads_read_word(rctx, name, sizeof(name), 0)) <= 0) return res;
@@ -291,9 +336,13 @@ static int pads_parse_text_(pads_read_ctx_t *rctx, int is_label, rnd_coord_t xo,
 	if ((res = pads_read_double(rctx, &rot)) <= 0) return res;
 	if ((res = pads_read_long(rctx, &level)) <= 0) return res;
 	if ((res = pads_read_coord(rctx, &h)) <= 0) return res;
-	if ((res = pads_read_coord(rctx, &w)) <= 0) return res;
+	if ((res = pads_read_coord(rctx, &thick)) <= 0) return res;
 
-TODO("w is really thickness");
+	TODO("make this configurable:");
+	if ((thick <= 0) || (thick >= h/4)) {
+		PADS_ERROR((RND_MSG_ERROR, "invalid text thickness: %mm (for height %mm) - adjusted\n", thick, h));
+		thick = h/16;
+	}
 
 	/* next field is either mirror (M or N) or hjust */
 	if ((res = pads_read_word(rctx, hjust, sizeof(hjust), 0)) <= 0) return res;
@@ -306,7 +355,16 @@ TODO("w is really thickness");
 		if ((res = pads_read_word(rctx, hjust, sizeof(hjust), 0)) <= 0) return res;
 	}
 
+	if (strcmp(hjust, "CENTER") == 0)     anx = 0.5;
+	else if (strcmp(hjust, "LEFT") == 0)  anx = 0.0;
+	else if (strcmp(hjust, "RIGHT") == 0) anx = 1.0;
+	else PADS_ERROR((RND_MSG_ERROR, "invalid text horizontal adjustment: %s\n", hjust));
+
 	if ((res = pads_read_word(rctx, vjust, sizeof(vjust), 0)) <= 0) return res;
+	if (strcmp(vjust, "CENTER") == 0)     any = 0.5;
+	else if (strcmp(vjust, "UP") == 0)    any = 0.0;
+	else if (strcmp(vjust, "DOWN") == 0)  any = 1.0;
+	else PADS_ERROR((RND_MSG_ERROR, "invalid text vertical adjustment: %s\n", vjust));
 
 	/* fields ignored: ndim and .REUSE.*/
 	pads_eatup_till_nl(rctx);
@@ -316,56 +374,69 @@ TODO("w is really thickness");
 		pads_eatup_till_nl(rctx);
 	}
 
-	if ((res = pads_read_word(rctx, str, sizeof(str), 1)) <= 0) return res;
+	if ((res = pads_read_word_all(rctx, str, sizeof(str), 1)) <= 0) return res;
 
-	rnd_trace(" text: [%s] at %mm;%mm rot=%f size=%mm;%mm mirr=%d: '%s'",
-		font, x, y, rot, w, h, mirr, str);
+	rnd_trace(" text: [%s] at %mm;%mm rot=%f height=%mm thick=%mm mirr=%d: '%s'",
+		font, x, y, rot, h, thick, mirr, str);
 	if (is_label)
 		rnd_trace(" (%s)\n", name);
 	else
 		rnd_trace("\n");
 
-	TODO("This should rather use font height");
-	scale = (double)w/RND_MM_TO_COORD(1.0) * 100.0 * 8;
+	if (is_label) {
+		if (strcmp(str, "Part Type") == 0)
+			goto skip;
+		if (strcmp(str, "Ref.Des.") == 0) {
+			if (how & TEXT_SKIP_REFDES)
+				goto skip;
+			strcpy(str, "%a.parent.refdes%");
+			flg |= PCB_FLAG_DYNTEXT;
+		}
+	}
+
+	str_width = rnd_round(h / 1.9 * strlen(str));
+	anchx = rnd_round(str_width * anx);
+	anchy = rnd_round(h * any);
+
+	text = pcb_dlcr_text_by_bbox_new(&rctx->dlcr, pcb_font(rctx->pcb, 0, 1),
+		x+xo, y+yo+h, str_width, h, anchx, anchy, 1, tmir, rot, thick, str,
+		(is_label ? PCB_FLAG_FLOATER : 0) | flg);
 
-	TODO("do not ignore text alignment");
-	text = pcb_dlcr_text_new(&rctx->dlcr, x+xo, y+yo+w*9, rot, scale, 0, str);
 	text->loc_line = rctx->line;
+	if (in_subc) {
+		text->in_last_subc = 1;
+		if (how & TEXT_SUBC_RELATIVE)
+			text->subc_relative = 1;
+	}
 
 	if (is_label) {
 		text->val.obj.layer_id = PCB_DLCR_INVALID_LAYER_ID;
+#if 0
 		switch(level) {
 			case 1: text->val.obj.lyt = PCB_LYT_TOP | PCB_LYT_SILK; break;
-			case 2: text->val.obj.lyt = PCB_LYT_BOTTOM | PCB_LYT_SILK; break; TODO("need to check what happens on the bottom side, is it really LID 2?");
+			case 2: text->val.obj.lyt = PCB_LYT_BOTTOM | PCB_LYT_SILK; break;
 			default:
-				TODO("Figure what does this mean");
 				PADS_ERROR((RND_MSG_ERROR, "invalid label level %ld\n", level));
 				text->val.obj.layer_id = level;
 				break;
 		}
+#endif
+		/* batch3/llevel2 and batch3/llevel3 shows that a label on any level
+		   will end up on primary side silk. */
+		text->val.obj.lyt = PCB_LYT_TOP | PCB_LYT_SILK;
 	}
 	else
 		text->val.obj.layer_id = level;
 
+	skip:;
 	pads_eatup_till_nl(rctx);
 
 	return 1;
 }
 
-static int pads_parse_text(pads_read_ctx_t *rctx, rnd_coord_t xo, rnd_coord_t yo)
-{
-	return pads_parse_text_(rctx, 0, xo, yo);
-}
-
 static int pads_parse_text0(pads_read_ctx_t *rctx)
 {
-	return pads_parse_text_(rctx, 0, 0, 0);
-}
-
-
-static int pads_parse_label(pads_read_ctx_t *rctx, rnd_coord_t xo, rnd_coord_t yo)
-{
-	return pads_parse_text_(rctx, 1, xo, yo);
+	return pads_parse_text(rctx, 0, 0, 0);
 }
 
 static int pads_parse_texts(pads_read_ctx_t *rctx)
@@ -425,9 +496,9 @@ static int pads_parse_line(pads_read_ctx_t *rctx)
 	}
 
 	for(n = 0; n < num_pcs; n++)
-		if ((res = pads_parse_piece(rctx, ltype, xo, yo)) <= 0) return res;
+		if ((res = pads_parse_piece(rctx, ltype, xo, yo, 0)) <= 0) return res;
 	for(n = 0; n < num_texts; n++)
-		if ((res = pads_parse_text(rctx, xo, yo)) <= 0) return res;
+		if ((res = pads_parse_text(rctx, xo, yo, 0)) <= 0) return res;
 
 	return 1;
 }
@@ -437,12 +508,14 @@ static int pads_parse_lines(pads_read_ctx_t *rctx)
 	return pads_parse_list_sect(rctx, pads_parse_line);
 }
 
+#define PADS_TERM_NAME_LEN 32
 typedef struct pads_term_s pads_term_t;
 
 struct pads_term_s {
 	rnd_coord_t x, y;
 	long loc_line;
 	pads_term_t *next;
+	char name[PADS_TERM_NAME_LEN];
 };
 
 static void pads_create_pins(pads_read_ctx_t *rctx, pads_term_t *first, long pinidx, long pid)
@@ -450,7 +523,7 @@ static void pads_create_pins(pads_read_ctx_t *rctx, pads_term_t *first, long pin
 	pads_term_t *t, *tnext;
 rnd_trace("  pin create: %ld pid=%ld first=%p\n", pinidx, pid, first);
 	for(t = first; t != NULL; t = tnext) {
-		pcb_dlcr_draw_t *pin = pcb_dlcr_via_new(&rctx->dlcr, t->x, t->y, 0, pid, NULL);
+		pcb_dlcr_draw_t *pin = pcb_dlcr_via_new(&rctx->dlcr, t->x, t->y, 0, pid, NULL, t->name);
 		if (pin != NULL)
 			pin->loc_line = t->loc_line;
 rnd_trace("    %mm;%mm (%p) %p pid=%ld\n", t->x, t->y, t, pin, pid);
@@ -468,14 +541,23 @@ static void free_terms(pads_read_ctx_t *rctx, vtp0_t *terms, long defpid)
 	vtp0_uninit(terms);
 }
 
+static int pads_parse_pstk_optf(pads_read_ctx_t *rctx, int line, int has_arg)
+{
+	if (line == 0) return 1; /* first line must have all optional args */
+	return has_arg;
+}
+
+#define OPTF \
+	pads_parse_pstk_optf(rctx, n, pads_has_field(rctx))
+
 static int pads_parse_pstk_proto(pads_read_ctx_t *rctx, vtp0_t *terms, long *defpid)
 {
 	pcb_pstk_proto_t *proto;
 	pcb_pstk_tshape_t *ts;
 	char name[256], shape[8], pinno[10];
 	rnd_coord_t drill = 0, size;
-	long n, num_lines, level, start = 0, end = 0, pid;
-	int res;
+	long sn, n, num_lines, level, start = 0, end = 0, pid;
+	int res, has_slot = 0;
 
 	if ((res = pads_read_word(rctx, name, sizeof(name), 0)) <= 0) return res;
 	if (terms != NULL) {
@@ -506,27 +588,27 @@ static int pads_parse_pstk_proto(pads_read_ctx_t *rctx, vtp0_t *terms, long *def
 
 	rnd_trace(" pstk_proto: %s drill=%mm [%ld..%ld] pr=%p ts=%p\n", name, drill, start, end, proto, ts);
 
-	for(n = 0; n < num_lines; n++) {
+	for(n = sn = 0; n < num_lines; n++) {
 		double rot = 0, slotrot = 0, spokerot = 0;
 		char plated[8];
 		int c, is_thermal;
 		long spoke_num = 0;
 		rnd_coord_t finlen = 0, finoffs = 0, inner = 0, corner = 0, drill = 0, slotlen = 0, slotoffs = 0, spoke_outsize = 0, spoke_width = 0;
-		pcb_pstk_shape_t *shp = ts->shape + n;
+		pcb_pstk_shape_t *shp = ts->shape + sn;
 
 		if ((res = pads_read_long(rctx, &level)) <= 0) return res;
 		if ((res = pads_read_coord(rctx, &size)) <= 0) return res;
 		if ((res = pads_read_word(rctx, shape, sizeof(shape), 0)) <= 0) return res;
 
 		if (shape[0] == 'A') /* only annular pad has inner dia */
-			if ((res = pads_read_coord(rctx, &inner)) <= 0) return res;
+			if (OPTF) if ((res = (pads_read_coord(rctx, &inner))) <= 0) return res;
 
 		if (shape[1] != 'T') { /* normal pad (T is for thermal) */
 			is_thermal = 0;
 			if (shape[1] == 'F') { /* finger-type pads have rotation */
-				if ((res = pads_read_double(rctx, &rot)) <= 0) return res;
-				if ((res = pads_read_coord(rctx, &finlen)) <= 0) return res;
-				if ((res = pads_read_coord(rctx, &finoffs)) <= 0) return res;
+				if (OPTF) if ((res = pads_read_double(rctx, &rot)) <= 0) return res;
+				if (OPTF) if ((res = pads_read_coord(rctx, &finlen)) <= 0) return res;
+				if (OPTF) if ((res = pads_read_coord(rctx, &finoffs)) <= 0) return res;
 				if (pads_has_field(rctx) && (shape[0] == 'R')) /* RF=rectangular finger */
 					if ((res = pads_read_coord(rctx, &corner)) <= 0) return res;
 			}
@@ -540,25 +622,34 @@ static int pads_parse_pstk_proto(pads_read_ctx_t *rctx, vtp0_t *terms, long *def
 			ungetc(c, rctx->f);
 
 			/* optional drill */
-			if (pads_has_field(rctx) && (!isalpha(c)))
+			if (pads_has_field(rctx) && (!isalpha(c))) {
 				if ((res = pads_read_coord(rctx, &drill)) <= 0) return res;
+				proto->hdia = drill;
+			}
 			plated[0] = '\0';
 			if (pads_has_field(rctx))
 				if ((res = pads_read_word(rctx, plated, sizeof(plated), 0)) <= 0) return res;
 
 			/* optional slot */
 			if (pads_has_field(rctx)) {
-				if ((res = pads_read_double(rctx, &slotrot)) <= 0) return res;
-				if ((res = pads_read_coord(rctx, &slotlen)) <= 0) return res;
-				if ((res = pads_read_coord(rctx, &slotoffs)) <= 0) return res;
+				if (OPTF) if ((res = pads_read_double(rctx, &slotrot)) <= 0) return res;
+				if (OPTF) if ((res = pads_read_coord(rctx, &slotlen)) <= 0) return res;
+				if (OPTF) if ((res = pads_read_coord(rctx, &slotoffs)) <= 0) return res;
+
+				if (slotlen > 0) {
+					if (!has_slot) {
+						TODO("Create slot; requires powerpcb for testing; pstk_slot.asc; need to redo that in batch4 with warnings fixed");
+					}
+					has_slot = 1;
+				}
 			}
 		}
 		else { /* thermal */
 			is_thermal = 1;
-			if ((res = pads_read_double(rctx, &spokerot)) <= 0) return res;
-			if ((res = pads_read_coord(rctx, &spoke_outsize)) <= 0) return res;
-			if ((res = pads_read_coord(rctx, &spoke_width)) <= 0) return res;
-			if ((res = pads_read_long(rctx, &spoke_num)) <= 0) return res;
+			if (OPTF) if ((res = pads_read_double(rctx, &spokerot)) <= 0) return res;
+			if (OPTF) if ((res = pads_read_coord(rctx, &spoke_outsize)) <= 0) return res;
+			if (OPTF) if ((res = pads_read_coord(rctx, &spoke_width)) <= 0) return res;
+			if (OPTF) if ((res = pads_read_long(rctx, &spoke_num)) <= 0) return res;
 		}
 
 		if ((plated[0] == 'P') || (plated[0] == '\0'))
@@ -567,22 +658,83 @@ static int pads_parse_pstk_proto(pads_read_ctx_t *rctx, vtp0_t *terms, long *def
 		pads_eatup_till_nl(rctx);
 		rnd_trace("  lev=%ld size=%mm/%mm shape=%s is_thermal=%d\n", level, size, inner, shape, is_thermal);
 
+		shp->shape = -1;
+
 		/* create the shape in shp */
-		if ((shape[0] == 'R') && (shape[1] == '\0')) {
-			shp->shape = PCB_PSSH_CIRC;
-			shp->data.circ.x = shp->data.circ.y = 0;
-			shp->data.circ.dia = size;
+		if (((shape[0] == 'R') || (shape[0] == 'A')) && (shape[1] == '\0')) {
+			if (size != 0) {
+				shp->shape = PCB_PSSH_CIRC;
+				shp->data.circ.x = shp->data.circ.y = 0;
+				shp->data.circ.dia = size;
+			}
+		}
+		else if ((shape[0] == 'S') && (shape[1] == '\0')) {
+			rnd_coord_t r2 = rnd_round(size / 2.0);
+			if (r2 != 0) {
+				shp->shape = PCB_PSSH_POLY;
+				pcb_pstk_shape_alloc_poly(&shp->data.poly, 4);
+				shp->data.poly.x[0] = -r2; shp->data.poly.y[0] = -r2;
+				shp->data.poly.x[1] = +r2; shp->data.poly.y[1] = -r2;
+				shp->data.poly.x[2] = +r2; shp->data.poly.y[2] = +r2;
+				shp->data.poly.x[3] = -r2; shp->data.poly.y[3] = +r2;
+			}
+		}
+		else if ((shape[0] == 'R') && (shape[1] == 'F') && (shape[2] == '\0')) {
+			rnd_coord_t r1 = rnd_round(finlen / 2.0), r2 = rnd_round(size / 2.0);
+			if (r2 != 0) {
+
+				shp->shape = PCB_PSSH_POLY;
+				pcb_pstk_shape_alloc_poly(&shp->data.poly, 4);
+				shp->data.poly.x[0] = -r1 + finoffs; shp->data.poly.y[0] = -r2;
+				shp->data.poly.x[1] = +r1 + finoffs; shp->data.poly.y[1] = -r2;
+				shp->data.poly.x[2] = +r1 + finoffs; shp->data.poly.y[2] = +r2;
+				shp->data.poly.x[3] = -r1 + finoffs; shp->data.poly.y[3] = +r2;
+
+				if (rot != 0) {
+					double cs = cos(rot / RND_RAD_TO_DEG), sn = sin(rot / RND_RAD_TO_DEG);
+					int n;
+					for(n = 0; n < shp->data.poly.len; n++)
+						rnd_rotate(&shp->data.poly.x[n], &shp->data.poly.y[n], 0, 0, cs, sn);
+				}
+			}
+		}
+		else if ((shape[0] == 'O') && (shape[1] == 'F') && (shape[2] == '\0')) {
+			rnd_coord_t r1 = rnd_round(finlen / 2.0), r2 = rnd_round(size / 2.0);
+			if ((r1 != 0) && (r2 != 0)) {
+				shp->shape = PCB_PSSH_LINE;
+				shp->data.line.x1 = -r1 + r2 + finoffs; shp->data.line.y1 = 0;
+				shp->data.line.x2 = +r1 - r2 + finoffs; shp->data.line.y1 = 0;
+				shp->data.line.thickness = size;
+				if (rot != 0) {
+					double cs = cos(rot / RND_RAD_TO_DEG), sn = sin(rot / RND_RAD_TO_DEG);
+					rnd_rotate(&shp->data.line.x1, &shp->data.line.y1, 0, 0, cs, sn);
+					rnd_rotate(&shp->data.line.x2, &shp->data.line.y2, 0, 0, cs, sn);
+				}
+			}
 		}
 		else { /* final fallback so that we have a prototype to draw */
 			shp->shape = PCB_PSSH_CIRC;
 			shp->data.circ.x = shp->data.circ.y = 0;
 			shp->data.circ.dia = RND_MM_TO_COORD(0.5);
-			PADS_ERROR((RND_MSG_ERROR, "failed to understand padstack shape '%s'; created dummy small circle\n", shape));
+			if ((shape[0] == 'O') && (shape[1] == '\0')) {
+				/* O: ring shape; see batch3/pstk_o.asc; our padstack model could capture
+				   this as an almost closed U shaped poly? But it's most probably never
+				   used outside of normal holed pin/via */
+				PADS_ERROR((RND_MSG_WARNING, "Padstack shape 'O' is not fully supported '%s'; used a circle assuming the hole will make it a ring\n", shape));
+			}
+			else
+				PADS_ERROR((RND_MSG_ERROR, "failed to understand padstack shape '%s'; created dummy small circle\n", shape));
 		}
 
-TODO("pads-specific code in delay draw!!; see also: TODO#71");
-		shp->layer_mask = level; /* ugly hack: save the layer id for now */
+		if (shp->shape != -1) {
+			shp->dlcr_psh_layer_id = level; /* ugly hack: save the layer id for now */
+			sn++;
+		}
 	}
+	ts->len = sn;
+
+	if ((proto->hdia <= 0) && !has_slot)
+		proto->hplated = 0;
 	pcb_pstk_proto_update(proto);
 
 	if (terms != NULL) {
@@ -603,12 +755,13 @@ TODO("pads-specific code in delay draw!!; see also: TODO#71");
 			t = *tp;
 			*tp = NULL;
 		}
-		if (t == NULL)
+		if ((t == NULL) && (pinno[0] != '0') && (pinno[1] != '\0'))
 			PADS_ERROR((RND_MSG_ERROR, "*PARTDECAL* padstack proto referencing non-existing pin number or duplicate pin number reference: '%s'\n", pinno));
 		pads_create_pins(rctx, t, pinidx, pid);
 	}
 	return 1;
 }
+#undef OPTF
 
 static int pads_parse_via_pstk_proto(pads_read_ctx_t *rctx)
 {
@@ -623,7 +776,7 @@ static int pads_parse_vias(pads_read_ctx_t *rctx)
 static int pads_parse_term(pads_read_ctx_t *rctx, long idx, vtp0_t *terms)
 {
 	pads_term_t *t, **tp;
-	char name[10];
+	char name[PADS_TERM_NAME_LEN];
 	int c, res;
 	long loc_line = rctx->line;
 	rnd_coord_t x, y, nmx, nmy;
@@ -641,15 +794,16 @@ static int pads_parse_term(pads_read_ctx_t *rctx, long idx, vtp0_t *terms)
 		if ((res = pads_read_word(rctx, name, sizeof(name), 0)) <= 0) return res;
 	}
 	else
-		*name = '\0';
+		sprintf(name, "%ld", idx+1);
 
 	pads_eatup_till_nl(rctx);
 
-	rnd_trace("  terminal: %s at %mm;%mm IDX=%ld\n", name, x, y, idx+1);
+	rnd_trace("  terminal: '%s' at %mm;%mm IDX=%ld\n", name, x, y, idx+1);
 	tp = (pads_term_t **)vtp0_get(terms, idx+1, 0);
 	t = calloc(sizeof(pads_term_t), 1);
 	t->x = x;
 	t->y = y;
+	strcpy(t->name, name);
 	t->loc_line = loc_line;
 	if (tp != NULL)
 		t->next = *tp;
@@ -661,16 +815,17 @@ static int pads_parse_term(pads_read_ctx_t *rctx, long idx, vtp0_t *terms)
 static int pads_parse_partdecal(pads_read_ctx_t *rctx)
 {
 	char name[256], unit[8];
-	rnd_coord_t xo, yo;
+	rnd_coord_t xo = 0, yo = 0, dummy;
 	long n, num_pieces, num_terms, num_stacks, num_texts = 0, num_labels = 0, defpid = -1;
 	int res;
 	pcb_subc_t *subc;
 	vtp0_t terms = {0};
+	double old_coord_scale;
 
 	if ((res = pads_read_word(rctx, name, sizeof(name), 0)) <= 0) return res;
 	if ((res = pads_read_word(rctx, unit, sizeof(unit), 0)) <= 0) return res;
-	if ((res = pads_read_coord(rctx, &xo)) <= 0) return res;
-	if ((res = pads_read_coord(rctx, &yo)) <= 0) return res;
+	if ((res = pads_read_coord(rctx, &dummy)) <= 0) return res; /* these are for the footprint editor, ignore them for the board */
+	if ((res = pads_read_coord(rctx, &dummy)) <= 0) return res;
 	if ((res = pads_read_long(rctx, &num_pieces)) <= 0) return res;
 	if ((res = pads_read_long(rctx, &num_terms)) <= 0) return res;
 	if ((res = pads_read_long(rctx, &num_stacks)) <= 0) return res;
@@ -684,17 +839,25 @@ static int pads_parse_partdecal(pads_read_ctx_t *rctx)
 
 	rnd_trace("part '%s', original %mm;%mm pcs=%ld texts=%ld labels=%ld terms=%ld stacks=%ld\n",
 		name, xo, yo, num_pieces, num_texts, num_labels, num_terms, num_stacks);
-TODO("set unit and origin");
+
+	old_coord_scale = rctx->coord_scale;
+	if (rctx->coord_scale > 1) { /* it seems if the file is in BASIC mode, this field should be ignored because partdecals are in BASIC too */
+		if ((*unit == 'I') || (*unit == 'i'))
+			rctx->coord_scale = RND_MIL_TO_COORD(1);
+		else if ((*unit == 'M') || (*unit == 'm'))
+			rctx->coord_scale = RND_MM_TO_COORD(1.0);
+	}
+
 
 	subc = pcb_dlcr_subc_new_in_lib(&rctx->dlcr, name);
 	pcb_dlcr_subc_begin(&rctx->dlcr, subc);
 
 	for(n = 0; n < num_pieces; n++)
-		if ((res = pads_parse_piece(rctx, PLTY_LINES, xo, yo)) <= 0) return res;
+		if ((res = pads_parse_piece(rctx, PLTY_LINES, xo, yo, 1)) <= 0) return res;
 	for(n = 0; n < num_texts; n++)
-		if ((res = pads_parse_text(rctx, xo, yo)) <= 0) return res;
+		if ((res = pads_parse_text(rctx, xo, yo, 0)) <= 0) return res;
 	for(n = 0; n < num_labels; n++)
-		if ((res = pads_parse_label(rctx, xo, yo)) <= 0) return res;
+		if ((res = pads_parse_text(rctx, xo, yo, TEXT_IS_LABEL | TEXT_SKIP_REFDES)) <= 0) return res;
 	for(n = 0; n < num_terms; n++)
 		if ((res = pads_parse_term(rctx, n, &terms)) <= 0) { free_terms(rctx, &terms, defpid); return res; }
 	for(n = 0; n < num_stacks; n++)
@@ -704,6 +867,8 @@ TODO("set unit and origin");
 
 	pcb_dlcr_subc_end(&rctx->dlcr);
 
+	rctx->coord_scale = old_coord_scale;
+
 	return 1;
 }
 
@@ -881,16 +1046,16 @@ static int pads_parse_part(pads_read_ctx_t *rctx)
 	if (part != NULL) {
 		pcb_dlcr_draw_t *po = pcb_dlcr_subc_new_from_lib(&rctx->dlcr, xo, yo, rot, (*mirr == 'M'), decalname, dln);
 		po->loc_line = rctx->line;
+		pcb_dlcr_attrib_set_prev(&rctx->dlcr, "refdes", refdes);
 	}
 	else {
 		PADS_ERROR((RND_MSG_ERROR, "*PART* on undefined parttype '%s'\n", partname));
 		return -1;
 	}
 
-TODO("add refdes and labels");
 
 	for(n = 0; n < num_labels; n++)
-		if ((res = pads_parse_label(rctx, xo, yo)) <= 0) return res;
+		if ((res = pads_parse_text(rctx, 0, 0, TEXT_IS_LABEL | TEXT_IN_LAST_SUBC | TEXT_SUBC_RELATIVE)) <= 0) return res;
 	return 1;
 }
 
@@ -908,19 +1073,60 @@ static void get_arc_angles(rnd_coord_t r, rnd_coord_t cx, rnd_coord_t cy, rnd_co
 		return;
 	}
 
-	s = atan2(y0 - cy, x0 - cx);
-	e = atan2(y1 - cy, x1 - cx);
+	/* pcb-rnd's own (unusual) arc representation */
+	s = atan2(y0 - cy, -(x0 - cx));
+	e = atan2(y1 - cy, -(x1 - cx));
 	d = e - s;
 
-	if ((d < 0) && (arcdir > 1))
-		d = 360+d;
-	else if ((d > 0) && (arcdir < 1))
-		d = 360-d;
+	/* flip arc dir upon mismatching CW or CCW */
+	if ((d < 0) && (arcdir < 0))
+		d = 2*M_PI + d;
+	else if ((d > 0) && (arcdir > 0))
+		d = d - 2*M_PI;
 
 	*starta = s;
 	*deltaa = d;
 }
 
+static void add_teardrop(void *rctx_, pcb_any_obj_t *obj, void *call_ctx)
+{
+	pads_read_ctx_t *rctx = rctx_;
+	pcb_layer_t *ly;
+	pcb_data_t *dt;
+
+	if (obj->parent_type != PCB_PARENT_LAYER)
+		return;
+
+	ly = obj->parent.layer;
+	assert(ly->parent_type == PCB_PARENT_DATA);
+	dt = ly->parent.data;
+	if (dt->parent_type != PCB_PARENT_BOARD)
+		return;
+
+	if (!(pcb_layer_flags_(ly) & PCB_LYT_COPPER)) /* teardrop makes sense on copper only */
+		return;
+
+	if ((rctx->teardrop_eo == NULL) && !rctx->teardrop_warned) {
+		pcb_extobj_t **eo = NULL;
+		int idx = pcb_extobj_lookup_idx("teardrop");
+
+		if (idx != 0)
+			eo = (pcb_extobj_t **)vtp0_get(&pcb_extobj_i2o, idx, 0);
+
+		if ((eo == NULL) || (*eo == NULL)) {
+			/* extobj backend got deregistered meanwhile */
+			PADS_ERROR((RND_MSG_ERROR, "*SIGNAL* line teardrop: teardrop extended object not found\nSkipping teardrops, please enable that plugin if you need them.\n"));
+			rctx->teardrop_warned = 1;
+		}
+		rctx->teardrop_eo = *eo;
+	}
+	if (rctx->teardrop_eo != NULL) {
+		pcb_subc_t *eo = pcb_extobj_conv_obj(rctx->pcb, rctx->teardrop_eo, rctx->pcb->Data, obj, rnd_true);
+		if (eo == NULL)
+			PADS_ERROR((RND_MSG_ERROR, "*SIGNAL* line teardrop: failed to create teardrop\n"));
+	}
+}
+
 typedef struct {
 	rnd_coord_t x, y; /* last x;y */
 	rnd_coord_t x0, y0, cx, cy; /* last arc start coords and center, when arcdir != 0 */
@@ -931,10 +1137,11 @@ typedef struct {
 
 static int pads_parse_signal_crd(pads_read_ctx_t *rctx, pads_sig_piece_t *spc, long idx)
 {
-	char vianame[64], teardrop[16];
+	char vianame[64], teardrop[16], tdir[4];
 	long real_level, level, flags, loc_line;
 	rnd_coord_t x, y, width;
 	int res, arcdir = 0, thermal = 0;
+	int teardrop_dir = 0; /* -1 is previous, +1 is next */
 
 	if ((res = pads_read_coord(rctx, &x)) <= 0) return res;
 	if ((res = pads_read_coord(rctx, &y)) <= 0) return res;
@@ -947,7 +1154,6 @@ static int pads_parse_signal_crd(pads_read_ctx_t *rctx, pads_sig_piece_t *spc, l
 	*vianame = '\0';
 	if (pads_has_field(rctx)) {
 		if ((res = pads_read_word(rctx, vianame, sizeof(vianame), 0)) <= 0) return res;
-
 		if (strcmp(vianame, "TEARDROP") == 0) {
 			*vianame = '\0';
 			goto teardrop;
@@ -965,6 +1171,17 @@ static int pads_parse_signal_crd(pads_read_ctx_t *rctx, pads_sig_piece_t *spc, l
 		if ((res = pads_read_word(rctx, teardrop, sizeof(teardrop), 0)) <= 0) return res;
 		if (strcmp(teardrop, "TEARDROP") == 0) {
 			teardrop:;
+			if ((res = pads_read_word(rctx, tdir, sizeof(tdir), 0)) <= 0) return res;
+			if (tdir[1] != '\0') {
+				PADS_ERROR((RND_MSG_ERROR, "*SIGNAL* line teardrop: empty P/N field\n"));
+				return -1;
+			}
+			if (tdir[0] == 'P') teardrop_dir = -1;
+			else if (tdir[0] == 'N') teardrop_dir = +1;
+			else {
+				PADS_ERROR((RND_MSG_ERROR, "*SIGNAL* line teardrop: need a P or N after the teardrop keyword\n"));
+				return -1;
+			}
 			/* for now, ignore teardrop fields */
 		}
 	}
@@ -986,7 +1203,7 @@ static int pads_parse_signal_crd(pads_read_ctx_t *rctx, pads_sig_piece_t *spc, l
 			spc->x0 = spc->x; spc->y0 = spc->y;
 			spc->cx = x; spc->cy = y;
 			spc->omit = 0;
-			return 0;
+			return 1;
 		}
 		else {
 			PADS_ERROR((RND_MSG_ERROR, "*SIGNAL* arc arguments without the arc flag\n"));
@@ -1000,13 +1217,12 @@ static int pads_parse_signal_crd(pads_read_ctx_t *rctx, pads_sig_piece_t *spc, l
 		/* in this case level is the via's target level and our trace segment is
 		   really on the same level as the previous */
 		level = spc->lastlev;
-		via = pcb_dlcr_via_new(&rctx->dlcr, x, y, 0, -1, vianame);
+		via = pcb_dlcr_via_new(&rctx->dlcr, x, y, 0, -1, vianame, rctx->signal_netname);
 		if (via != NULL)
 			via->loc_line = loc_line;
 	}
 
-	TODO("process flags\n");
-	TODO("do not ignore the miter flag: 0x0e000\n");
+	TODO("do not ignore the miter flag: 0x0e000: requires original powerpcb to check\n");
 
 	if (level == 65)
 		level = spc->lastlev;
@@ -1018,7 +1234,7 @@ static int pads_parse_signal_crd(pads_read_ctx_t *rctx, pads_sig_piece_t *spc, l
 				rnd_coord_t r = rnd_distance(spc->cx, spc->cy, spc->x0, spc->y0);
 				double starta, deltaa;
 				get_arc_angles(r, spc->cx, spc->cy, spc->x0, spc->y0, x, y, spc->arcdir, &starta, &deltaa);
-				arc = pcb_dlcr_arc_new(&rctx->dlcr, spc->cx, spc->cy, r, starta, deltaa, width, 0);
+				arc = pcb_dlcr_arc_new(&rctx->dlcr, spc->cx, spc->cy, r, starta * RND_RAD_TO_DEG, deltaa * RND_RAD_TO_DEG, width, 0);
 				if (arc != NULL) {
 					arc->val.obj.layer_id = level;
 					arc->loc_line = loc_line;
@@ -1032,12 +1248,15 @@ static int pads_parse_signal_crd(pads_read_ctx_t *rctx, pads_sig_piece_t *spc, l
 	}
 	else {
 		if (level != 0) {
+/*rnd_printf("*** line: %ld idx=%d omit=%d level=%d %mm %mm to %mm %mm\n", loc_line, idx, spc->omit, level, spc->x, spc->y, x, y);*/
 			if ((idx > 0) && (!spc->omit)) {
 				pcb_dlcr_draw_t *line = pcb_dlcr_line_new(&rctx->dlcr, spc->x, spc->y, x, y, width, 0);
 				if (line != NULL) {
 					line->val.obj.layer_id = level;
 					line->loc_line = loc_line;
 				}
+				if (teardrop_dir != 0)
+					pcb_dlcr_call_on(&rctx->dlcr, add_teardrop, rctx, NULL, (teardrop_dir > 0));
 			}
 			spc->omit = 0;
 		}
@@ -1050,12 +1269,29 @@ static int pads_parse_signal_crd(pads_read_ctx_t *rctx, pads_sig_piece_t *spc, l
 	return 1;
 }
 
+static void net_add_term(pads_read_ctx_t *rctx, pcb_net_t *net, char *pinname)
+{
+	char *refdes = pinname, *term = strchr(pinname, '.');
+
+	if (term == NULL) {
+		PADS_ERROR((RND_MSG_ERROR, "*SIGNAL* terminal name error: '%s' is not refdes.pin\nPlease report this bug!\n"));
+		return;
+	}
+
+	*term = '\0';
+	term++;
+
+	if (pcb_net_term_get(net, refdes, term, 1) == NULL)
+		PADS_ERROR((RND_MSG_ERROR, "*SIGNAL* failed to create terminal : '%s:%s'\nPlease report this bug!\n", refdes, term));
+}
+
 static int pads_parse_net(pads_read_ctx_t *rctx)
 {
 	char term1[128], term2[128];
 	long idx;
 	int c, res;
 	pads_sig_piece_t spc = {0};
+	pcb_net_t *net;
 
 	if ((res = pads_read_word(rctx, term1, sizeof(term1), 0)) <= 0) return res;
 	if ((res = pads_read_word(rctx, term2, sizeof(term2), 0)) <= 0) return res;
@@ -1063,6 +1299,10 @@ static int pads_parse_net(pads_read_ctx_t *rctx)
 
 	rnd_trace(" '%s' -> '%s'\n", term1, term2);
 
+	net = pcb_net_get(rctx->pcb, &rctx->pcb->netlist[PCB_NETLIST_INPUT], rctx->signal_netname, 1);
+	net_add_term(rctx, net, term1);
+	net_add_term(rctx, net, term2);
+
 	for(idx = 0; ;idx++) {
 		double tmp;
 
@@ -1087,10 +1327,19 @@ static int pads_parse_signal(pads_read_ctx_t *rctx)
 	if ((res = pads_read_word(rctx, netname, sizeof(netname), 0)) <= 0) return res;
 
 	rnd_trace("signal: netname=%s\n", netname);
-	return pads_parse_list_sect(rctx, pads_parse_net);
+	rctx->signal_netname = netname;
+	res = pads_parse_list_sect(rctx, pads_parse_net);
+	rctx->signal_netname = NULL;
+
+	return res;
 }
 
-static int pads_parse_pour_piece_crd(pads_read_ctx_t *rctx)
+typedef enum {
+	PADS_POTY_CONTOUR,
+	PADS_POTY_NOOP
+} pads_poly_type_t;
+
+static int pads_parse_pour_piece_crd(pads_read_ctx_t *rctx, pcb_dlcr_draw_t *poly, pads_poly_type_t poty, rnd_coord_t xo, rnd_coord_t yo)
 {
 	rnd_coord_t x, y;
 	double start, end;
@@ -1106,28 +1355,52 @@ static int pads_parse_pour_piece_crd(pads_read_ctx_t *rctx)
 	}
 	pads_eatup_till_nl(rctx);
 
-	if (isarc)
+	if (poty == PADS_POTY_NOOP)
+		return 1;
+
+	if (isarc) {
+		TODO("poly arc: requires powerpcb to check; batch3/polyarc* ; resulted in errors");
 		rnd_trace("  arc %mm;%mm %f..%f\n", x, y, start, end);
-	else
+	}
+	else {
 		rnd_trace("  line %mm;%mm\n", x, y);
+		pcb_dlcr_poly_lineto(&rctx->dlcr, poly, x+xo, y+yo);
+	}
 
 	return 1;
 }
 
-static int pads_parse_pour_piece(pads_read_ctx_t *rctx)
+/* draw polygon contour from pieces */
+static int pads_parse_pour_piece_polycnt(pads_read_ctx_t *rctx, pads_poly_type_t poty, rnd_coord_t xo, rnd_coord_t yo, const char *signame)
 {
 	char type[64];
-	long n, num_corners, num_arcs;
+	long n, num_corners, num_arcs, level, loc = rctx->line;
 	int res;
+	rnd_coord_t width;
+	pcb_dlcr_draw_t *poly = NULL;
 
 	if ((res = pads_read_word(rctx, type, sizeof(type), 0)) <= 0) return res;
 	if ((res = pads_read_long(rctx, &num_corners)) <= 0) return res;
 	if ((res = pads_read_long(rctx, &num_arcs)) <= 0) return res;
+	if ((res = pads_read_coord(rctx, &width)) <= 0) return res;
+	if ((res = pads_read_long(rctx, &level)) <= 0) return res;
 	pads_eatup_till_nl(rctx); /* ignore rest of the arguments */
 
 	rnd_trace(" %s:\n", type);
-	for(n = 0; n < num_corners + num_arcs; n++)
-		if ((res = pads_parse_pour_piece_crd(rctx)) <= 0) return res;
+	if (poty != PADS_POTY_NOOP)
+		poly = pcb_dlcr_poly_new(&rctx->dlcr, 0, num_corners + num_arcs*8);
+	if ((poly != NULL) || (poty == PADS_POTY_NOOP)) {
+		for(n = 0; n < num_corners + num_arcs; n++)
+			if ((res = pads_parse_pour_piece_crd(rctx, poly, poty, xo, yo)) <= 0) return res;
+		if (poly != NULL) {
+			poly->val.obj.layer_id = level;
+			poly->loc_line = loc;
+		}
+	}
+
+TODO("if width != 0: round corners as if poly was drawn with the line as thick as specified, with the line's _outer edge_ being the poly contour; because it probably affects not only as-drawn but clipped version too, this needs arc-in-poly support? batch3/poly_bloat*");
+	if ((*signame != '\0') && (poly != NULL))
+		pcb_dlcr_set_net(poly, signame);
 
 	return 1;
 }
@@ -1135,22 +1408,39 @@ static int pads_parse_pour_piece(pads_read_ctx_t *rctx)
 
 static int pads_parse_pour(pads_read_ctx_t *rctx)
 {
-	char name[64], type[64];
-	long n, num_pieces;
+	char name[64], type[64], owner[64], signame[64];
+	long n, num_pieces, flags = 0;
 	rnd_coord_t xo, yo;
-	double rot;
 	int res;
+	pads_poly_type_t poty = PADS_POTY_NOOP;
+
+	owner[0] = signame[0] = '\0';
 
 	if ((res = pads_read_word(rctx, name, sizeof(name), 0)) <= 0) return res;
 	if ((res = pads_read_word(rctx, type, sizeof(type), 0)) <= 0) return res;
 	if ((res = pads_read_coord(rctx, &xo)) <= 0) return res;
 	if ((res = pads_read_coord(rctx, &yo)) <= 0) return res;
 	if ((res = pads_read_long(rctx, &num_pieces)) <= 0) return res;
+	if ((pads_has_field(rctx)) && ((res = pads_read_long(rctx, &flags)) <= 0)) return res;
+	if ((pads_has_field(rctx)) && ((res = pads_read_word(rctx, owner, sizeof(owner), 0)) <= 0)) return res;
+	if ((pads_has_field(rctx)) && ((res = pads_read_word(rctx, signame, sizeof(signame), 0)) <= 0)) return res;
 	pads_eatup_till_nl(rctx); /* ignore rest of the arguments */
 
-	rnd_trace("pour '%s' type='%s' at %mm;%mm\n", name, type, xo, yo);
+	rnd_trace("pour '%s' type='%s' at %mm;%mm signame='%s'\n", name, type, xo, yo, signame);
+	if ((strcmp(type, "POUROUT") == 0) || (strcmp(type, "HATOUT") == 0)) {
+		poty = PADS_POTY_CONTOUR;
+	}
+	else if (strcmp(type, "VOIDOUT") == 0) {
+		TODO("poly cutout: needs powerpcb to test");
+	}
+	else if (strcmp(type, "VIATHERM") == 0) {
+		TODO("poly via therm: needs powerpcb to test");
+	}
+	else
+		PADS_ERROR((RND_MSG_ERROR, "Unhandled pour type: '%s' - please report this bug\n", type));
+
 	for(n = 0; n < num_pieces; n++)
-		if ((res = pads_parse_pour_piece(rctx)) <= 0) return res;
+		if ((res = pads_parse_pour_piece_polycnt(rctx, poty, xo, yo, signame)) <= 0) return res;
 
 	return 1;
 }
