@@ -2,7 +2,7 @@
  *                            COPYRIGHT
  *
  *  pcb-rnd, interactive printed circuit board design
- *  Copyright (C) 2017 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2017,2021 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include "pstk_compat.h"
 #include "obj_pstk_inlines.h"
 #include <librnd/core/compat_misc.h>
+#include <librnd/core/rotate.h>
 #include <librnd/poly/polygon1_gen.h>
 
 #include "plug_io.h"
@@ -137,7 +138,7 @@ static void compat_shape_free(pcb_pstk_shape_t *shp)
 		free(shp->data.poly.x);
 }
 
-static pcb_pstk_t *pcb_pstk_new_compat_via_(pcb_data_t *data, long int id, rnd_coord_t x, rnd_coord_t y, rnd_coord_t drill_dia, rnd_coord_t pad_dia, rnd_coord_t clearance, rnd_coord_t mask, pcb_pstk_compshape_t cshape, rnd_bool plated, rnd_bool hole_clearance_hack)
+rnd_cardinal_t pcb_pstk_new_compat_via_proto(pcb_data_t *data, rnd_coord_t drill_dia, rnd_coord_t pad_dia, rnd_coord_t mask, pcb_pstk_compshape_t cshape, rnd_bool plated, rnd_bool hole_clearance_hack)
 {
 	pcb_pstk_proto_t proto;
 	pcb_pstk_shape_t shape[5]; /* max number of shapes: 3 coppers, 2 masks */
@@ -146,20 +147,7 @@ static pcb_pstk_t *pcb_pstk_new_compat_via_(pcb_data_t *data, long int id, rnd_c
 	pcb_pstk_tshape_t tshp;
 	int n;
 
-	if (hole_clearance_hack && !plated) {
-		/* in PCB hole means unplated with clearance; emulate this by placing a
-		   zero diameter copper circle on all layers and set clearance large
-		   enough to cover the hole too */
-		clearance = rnd_round((double)clearance + (double)drill_dia/2.0);
-		pad_dia = 0.0;
-	}
-
-	/* for plated vias, positive pad is required */
-	if (plated && !hole_clearance_hack)
-		assert(pad_dia > drill_dia);
-
 	assert(drill_dia > 0);
-	assert(clearance >= 0);
 	assert(mask >= 0);
 
 	memset(&proto, 0, sizeof(proto));
@@ -176,7 +164,7 @@ static pcb_pstk_t *pcb_pstk_new_compat_via_(pcb_data_t *data, long int id, rnd_c
 
 		/* we need to generate the shape only once as it's the same on all */
 		if (compat_via_shape_gen(&copper_master, cshape, pad_dia) != 0)
-			return NULL;
+			return -1;
 
 		for(n = 0; n < 3; n++)
 			memcpy(&shape[n], &copper_master, sizeof(copper_master));
@@ -190,7 +178,7 @@ static pcb_pstk_t *pcb_pstk_new_compat_via_(pcb_data_t *data, long int id, rnd_c
 
 	if (mask > 0) {
 		if (compat_via_shape_gen(&mask_master, cshape, mask) != 0)
-			return NULL;
+			return -1;
 	
 		memcpy(&shape[tshp.len+0], &mask_master, sizeof(mask_master));
 		memcpy(&shape[tshp.len+1], &mask_master, sizeof(mask_master));
@@ -206,7 +194,7 @@ static pcb_pstk_t *pcb_pstk_new_compat_via_(pcb_data_t *data, long int id, rnd_c
 	pid = pcb_pstk_proto_insert_dup(data, &proto, 1, 0);
 	if (pid == PCB_PADSTACK_INVALID) {
 		compat_shape_free(&copper_master);
-		return NULL;
+		return -1;
 	}
 
 	compat_shape_free(&copper_master);
@@ -214,9 +202,34 @@ static pcb_pstk_t *pcb_pstk_new_compat_via_(pcb_data_t *data, long int id, rnd_c
 	if (mask > 0)
 		compat_shape_free(&mask_master);
 
+	return pid;
+}
+
+static pcb_pstk_t *pcb_pstk_new_compat_via_(pcb_data_t *data, long int id, rnd_coord_t x, rnd_coord_t y, rnd_coord_t drill_dia, rnd_coord_t pad_dia, rnd_coord_t clearance, rnd_coord_t mask, pcb_pstk_compshape_t cshape, rnd_bool plated, rnd_bool hole_clearance_hack)
+{
+	rnd_cardinal_t pid;
+
+	assert(clearance >= 0);
+
+	if (hole_clearance_hack && !plated) {
+		/* in PCB hole means unplated with clearance; emulate this by placing a
+		   zero diameter copper circle on all layers and set clearance large
+		   enough to cover the hole too */
+		clearance = rnd_round((double)clearance + (double)drill_dia/2.0);
+		pad_dia = 0.0;
+	}
+
+	/* for plated vias, positive pad is required */
+	if (plated && !hole_clearance_hack)
+		assert(pad_dia > drill_dia);
+
+	pid = pcb_pstk_new_compat_via_proto(data, drill_dia, pad_dia, mask, cshape, plated, hole_clearance_hack);
+	if (pid == -1)
+		return NULL;
 	return pcb_pstk_new(data, -1, pid, x, y, clearance, pcb_flag_make(PCB_FLAG_CLEARLINE));
 }
 
+
 pcb_pstk_t *pcb_pstk_new_compat_via(pcb_data_t *data, long int id, rnd_coord_t x, rnd_coord_t y, rnd_coord_t drill_dia, rnd_coord_t pad_dia, rnd_coord_t clearance, rnd_coord_t mask, pcb_pstk_compshape_t cshape, rnd_bool plated)
 {
 	return pcb_pstk_new_compat_via_(data, id, x, y, drill_dia, pad_dia, clearance, mask, cshape, plated, 0);
@@ -359,18 +372,13 @@ static pcb_pstk_compshape_t get_old_shape(rnd_coord_t *dia, const pcb_pstk_shape
 	return PCB_PSTK_COMPAT_INVALID;
 }
 
-rnd_bool pcb_pstk_export_compat_via(pcb_pstk_t *ps, rnd_coord_t *x, rnd_coord_t *y, rnd_coord_t *drill_dia, rnd_coord_t *pad_dia, rnd_coord_t *clearance, rnd_coord_t *mask, pcb_pstk_compshape_t *cshape, rnd_bool *plated)
+rnd_bool pcb_pstk_export_compat_proto(pcb_pstk_proto_t *proto, rnd_coord_t *drill_dia, rnd_coord_t *pad_dia, rnd_coord_t *mask, pcb_pstk_compshape_t *cshape, rnd_bool *plated)
 {
-	pcb_pstk_proto_t *proto;
 	pcb_pstk_tshape_t *tshp;
 	int n, coppern = -1, maskn = -1;
 	pcb_pstk_compshape_t old_shape[5];
 	rnd_coord_t old_dia[5];
 
-	proto = pcb_pstk_get_proto_(ps->parent.data, ps->proto);
-	if ((proto == NULL) || (proto->tr.used < 1))
-		return rnd_false;
-
 	tshp = &proto->tr.array[0];
 
 	if ((tshp->len != 3) && (tshp->len != 5))
@@ -421,22 +429,37 @@ rnd_bool pcb_pstk_export_compat_via(pcb_pstk_t *ps, rnd_coord_t *x, rnd_coord_t
 	}
 
 	/* all went fine, collect and return data */
-	*x = ps->x;
-	*y = ps->y;
 	*drill_dia = proto->hdia;
 	*pad_dia = old_dia[coppern];
-	*clearance = ps->Clearance;
 	*mask = maskn >= 0 ? old_dia[maskn] : 0;
 	*cshape = old_shape[0];
 	*plated = proto->hplated;
 
-	/* clerance workaround for copper smaller than hole; inverse of r25735 */
-	if (*pad_dia < *drill_dia)
-		*clearance -= (*drill_dia - *pad_dia)/2;
-
 	return rnd_true;
 }
 
+rnd_bool pcb_pstk_export_compat_via(pcb_pstk_t *ps, rnd_coord_t *x, rnd_coord_t *y, rnd_coord_t *drill_dia, rnd_coord_t *pad_dia, rnd_coord_t *clearance, rnd_coord_t *mask, pcb_pstk_compshape_t *cshape, rnd_bool *plated)
+{
+	pcb_pstk_proto_t *proto;
+	rnd_bool res;
+
+	proto = pcb_pstk_get_proto_(ps->parent.data, ps->proto);
+	if ((proto == NULL) || (proto->tr.used < 1))
+		return rnd_false;
+
+	res = pcb_pstk_export_compat_proto(proto, drill_dia, pad_dia, mask, cshape, plated);
+	if (res) {
+		*x = ps->x;
+		*y = ps->y;
+		*clearance = ps->Clearance;
+		/* clerance workaround for copper smaller than hole; inverse of r25735 */
+		if (*pad_dia < *drill_dia)
+			*clearance -= (*drill_dia - *pad_dia)/2;
+	}
+	return res;
+}
+
+
 /* emulate the old 'square flag' pad */
 static void pad_shape(pcb_pstk_poly_t *dst, rnd_coord_t x1, rnd_coord_t y1, rnd_coord_t x2, rnd_coord_t y2, rnd_coord_t thickness)
 {
@@ -556,6 +579,7 @@ rnd_bool pcb_pstk_export_compat_pad(pcb_pstk_t *ps, rnd_coord_t *x1, rnd_coord_t
 	int n, coppern = -1, maskn = -1, pasten = -1;
 	pcb_layer_type_t side;
 	rnd_coord_t lx1[3], ly1[3], lx2[3], ly2[3], lt[3]; /* poly->line conversion cache */
+	double cs, sn, ang;
 
 	proto = pcb_pstk_get_proto_(ps->parent.data, ps->proto);
 	if ((proto == NULL) || (proto->tr.used < 1))
@@ -597,12 +621,16 @@ rnd_bool pcb_pstk_export_compat_pad(pcb_pstk_t *ps, rnd_coord_t *x1, rnd_coord_t
 	if ((coppern < 0) || (maskn < 0))
 		return rnd_false;
 
+	ang = ps->rot;
+	if (ps->xmirror)
+		ang = -ang;
+	cs = cos(ang / RND_RAD_TO_DEG);
+	sn = sin(ang / RND_RAD_TO_DEG);
+
 	/* if the shape is poly, convert to line to make the rest of the code simpler */
 	if (tshp->shape[0].shape == PCB_PSSH_POLY) {
 		for(n = 0; n < tshp->len; n++) {
-			rnd_coord_t w, h, x1, x2, y1, y2;
-			double stepd, step2;
-			int step;
+			double x0, y0, x1, y1, x2, y2, x3, y3, w, h, vx01, vy01, vx03, vy03, cx, cy;
 
 			if (tshp->shape[n].shape != PCB_PSSH_POLY)
 				continue;
@@ -610,37 +638,37 @@ rnd_bool pcb_pstk_export_compat_pad(pcb_pstk_t *ps, rnd_coord_t *x1, rnd_coord_t
 			if (tshp->shape[n].data.poly.len != 4)
 				return rnd_false;
 
-			stepd = ps->rot / 90.0;
-			step = stepd;
-			step2 = (double)step * 90.0;
-			if (fabs(ps->rot - step2) > 0.01)
-				return rnd_false;
+			x0 = tshp->shape[n].data.poly.x[0]; y0 = tshp->shape[n].data.poly.y[0];
+			x1 = tshp->shape[n].data.poly.x[1]; y1 = tshp->shape[n].data.poly.y[1];
+			x2 = tshp->shape[n].data.poly.x[2]; y2 = tshp->shape[n].data.poly.y[2];
+			x3 = tshp->shape[n].data.poly.x[3]; y3 = tshp->shape[n].data.poly.y[3];
+			w = rnd_distance(x0, y0, x1, y1);
+			h = rnd_distance(x0, y0, x3, y3);
+			vx01 = (x1 - x0) / w;
+			vy01 = (y1 - y0) / w;
+			vx03 = (x3 - x0) / h;
+			vy03 = (y3 - y0) / h;
 
-			x1 = tshp->shape[n].data.poly.x[0];
-			x2 = tshp->shape[n].data.poly.x[2];
-			if (x1 < x2) {
-				x2 = tshp->shape[n].data.poly.x[0];
-				x1 = tshp->shape[n].data.poly.x[2];
-			}
-			y1 = tshp->shape[n].data.poly.y[0];
-			y2 = tshp->shape[n].data.poly.y[2];
-			if (y1 < y2) {
-				y2 = tshp->shape[n].data.poly.y[0];
-				y1 = tshp->shape[n].data.poly.y[2];
-			}
-			if ((step % 2) == 1) {
-				h = x1 - x2;
-				w = y1 - y2;
+			cx = (x0+x1+x2+x3)/4.0;
+			cy = (y0+y1+y2+y3)/4.0;
+
+			if (w <= h) {
+				lt[n] = rnd_round(w);
+				lx1[n] = rnd_round(cx + h/2.0 * vx03 - w/2.0 * vx03);
+				ly1[n] = rnd_round(cy + h/2.0 * vy03 - w/2.0 * vy03);
+				lx2[n] = rnd_round(cx - h/2.0 * vx03 + w/2.0 * vx03);
+				ly2[n] = rnd_round(cy - h/2.0 * vy03 + w/2.0 * vy03);
 			}
 			else {
-				w = x1 - x2;
-				h = y1 - y2;
+				lt[n] = rnd_round(h);
+				lx1[n] = rnd_round(cx + w/2.0 * vx01 - h/2.0 * vx01);
+				ly1[n] = rnd_round(cy + w/2.0 * vy01 - h/2.0 * vy01);
+				lx2[n] = rnd_round(cx - w/2.0 * vx01 + h/2.0 * vx01);
+				ly2[n] = rnd_round(cy - w/2.0 * vy01 + h/2.0 * vy01);
 			}
-			lt[n] = (w < h) ? w : h;
-			lx1[n] = x2 + lt[n] / 2;
-			ly1[n] = y2 + lt[n] / 2;
-			lx2[n] = lx1[n] + (w - lt[n]);
-			ly2[n] = ly1[n] + (h - lt[n]);
+
+			rnd_rotate(&lx1[n], &ly1[n], cx, cy, cs, sn);
+			rnd_rotate(&lx2[n], &ly2[n], cx, cy, cs, sn);
 		}
 	}
 
