@@ -2,7 +2,7 @@
  *                            COPYRIGHT
  *
  *  pcb-rnd, interactive printed circuit board design
- *  Copyright (C) 2018,2019 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2018,2019,2021 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -82,9 +82,13 @@ static void rst_install_menu(void)
 	rnd_hid_menu_unload(rnd_gui, props.cookie);
 	/* have to go reverse to keep order because this will insert items */
 	for(idx = vtroutestyle_len(&PCB->RouteStyle)-1; idx >= 0; idx--) {
+		char *sep;
 		sprintf(act, "RouteStyle(%d)", idx+1); /* for historical reasons this action counts from 1 */
 		sprintf(chk, "ChkRst(%d)", idx);
 		strcpy(end, PCB->RouteStyle.array[idx].name);
+		for(sep = end; *sep !='\0'; sep++) /* do not allow '/' in layer name, that would kill the menu system */
+			if (*sep == '/')
+				*sep = '|';
 		rnd_hid_menu_create(path, &props);
 	}
 	rnd_hid_menu_merge_inhibit_dec();
@@ -94,7 +98,7 @@ static void rst_install_menu(void)
 /* Update the edit dialog and all checkboxes, but nothing else on the sub */
 static void rst_force_update_chk_and_dlg()
 {
-	int n, target = pcb_route_style_lookup(&PCB->RouteStyle, conf_core.design.line_thickness, conf_core.design.via_thickness, conf_core.design.via_drilling_hole, conf_core.design.clearance, NULL);
+	int n, target = PCB_LOOKUP_ROUTE_STYLE_PEN(PCB);
 	rnd_hid_attr_val_t hv;
 
 	idx_changed();
@@ -129,7 +133,7 @@ static void rst_update(rnd_hidlib_t *hidlib)
 
 	if (rst.sub_inited) {
 		int n, target;
-		target = pcb_route_style_lookup(&PCB->RouteStyle, conf_core.design.line_thickness, conf_core.design.via_thickness, conf_core.design.via_drilling_hole, conf_core.design.clearance, NULL);
+		target = pcb_lookup_route_style_pen_bestfit(PCB);
 		for(n = 0; n < vtroutestyle_len(&PCB->RouteStyle); n++) {
 			rnd_hid_attr_val_t hv;
 
@@ -168,7 +172,7 @@ static void rst_select_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t
 
 static void rst_edit_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t *attr)
 {
-	int target = pcb_route_style_lookup(&PCB->RouteStyle, conf_core.design.line_thickness, conf_core.design.via_thickness, conf_core.design.via_drilling_hole, conf_core.design.clearance, NULL);
+	int target = PCB_LOOKUP_ROUTE_STYLE_PEN(PCB);
 	if (target >= 0)
 		pcb_dlg_rstdlg(target);
 }
@@ -182,7 +186,7 @@ static void rst_new_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t *at
 
 static void rst_del_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t *attr)
 {
-	int target = pcb_route_style_lookup(&PCB->RouteStyle, conf_core.design.line_thickness, conf_core.design.via_thickness, conf_core.design.via_drilling_hole, conf_core.design.clearance, NULL);
+	int target = PCB_LOOKUP_ROUTE_STYLE_PEN(PCB);
 	if (target >= 0) {
 		pcb_route_style_del(PCB, target, 1);
 		rst_updated(NULL);
