@@ -2,7 +2,7 @@
  *                            COPYRIGHT
  *
  *  pcb-rnd, interactive printed circuit board design
- *  Copyright (C) 2019 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2019,2021 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,12 +27,13 @@
 /* included from routest.c - split for clarity */
 
 #include <librnd/core/hid_dad_tree.h>
+#include "route_style.h"
 
 typedef struct{
 	RND_DAD_DECL_NOINIT(dlg)
 	int active; /* already open - allow only one instance */
 	int curr;
-	int wname, wlineth, wclr, wtxtscale, wtxtth, wviahole, wviaring, wattr;
+	int wname, wlineth, wclr, wtxtscale, wtxtth, wfont, wproto, wattr;
 
 	rnd_hidval_t name_timer;
 
@@ -51,6 +52,7 @@ static void rstdlg_pcb2dlg(int rst_idx)
 	rnd_hid_attribute_t *attr;
 	rnd_hid_tree_t *tree;
 	pcb_attribute_t *a;
+	char tmp[128];
 
 	if (!rstdlg_ctx.active)
 		return;
@@ -79,17 +81,27 @@ static void rstdlg_pcb2dlg(int rst_idx)
 	hv.crd = rst->textt;
 	rnd_gui->attr_dlg_set_value(rstdlg_ctx.dlg_hid_ctx, rstdlg_ctx.wtxtth, &hv);
 
+	if (rst->fid != -1) {
+		sprintf(tmp, "#%ld", (long int)rst->fid);
+		hv.str = tmp;
+	}
+	else
+		hv.str = "<unset>";
+	rnd_gui->attr_dlg_set_value(rstdlg_ctx.dlg_hid_ctx, rstdlg_ctx.wfont, &hv);
+
 	hv.lng = rst->texts;
 	rnd_gui->attr_dlg_set_value(rstdlg_ctx.dlg_hid_ctx, rstdlg_ctx.wtxtscale, &hv);
 
 	hv.crd = rst->Clearance;
 	rnd_gui->attr_dlg_set_value(rstdlg_ctx.dlg_hid_ctx, rstdlg_ctx.wclr, &hv);
 
-	hv.crd = rst->Hole;
-	rnd_gui->attr_dlg_set_value(rstdlg_ctx.dlg_hid_ctx, rstdlg_ctx.wviahole, &hv);
-
-	hv.crd = rst->Diameter;
-	rnd_gui->attr_dlg_set_value(rstdlg_ctx.dlg_hid_ctx, rstdlg_ctx.wviaring, &hv);
+	if (rst->via_proto_set) {
+		sprintf(tmp, "#%ld", (long int)rst->via_proto);
+		hv.str = tmp;
+	}
+	else
+		hv.str = "<unset>";
+	rnd_gui->attr_dlg_set_value(rstdlg_ctx.dlg_hid_ctx, rstdlg_ctx.wproto, &hv);
 
 	rnd_dad_tree_clear(tree);
 
@@ -166,37 +178,51 @@ static void rst_change_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t
 			name_chg_cb(rstdlg_ctx.name_timer);
 	}
 	else if (idx == rstdlg_ctx.wlineth)
-		pcb_route_style_change(PCB, rstdlg_ctx.curr, &attr->val.crd, NULL, NULL, NULL, NULL, 1);
+		pcb_route_style_change(PCB, rstdlg_ctx.curr, &attr->val.crd, NULL, NULL, NULL, NULL, NULL, 1);
 	else if (idx == rstdlg_ctx.wtxtth)
-		pcb_route_style_change(PCB, rstdlg_ctx.curr, NULL, &attr->val.crd, NULL, NULL, NULL, 1);
+		pcb_route_style_change(PCB, rstdlg_ctx.curr, NULL, &attr->val.crd, NULL, NULL, NULL, NULL, 1);
 	else if (idx == rstdlg_ctx.wtxtscale) {
 		int tmp = attr->val.lng;
-		pcb_route_style_change(PCB, rstdlg_ctx.curr, NULL, NULL, &tmp, NULL, NULL, 1);
+		pcb_route_style_change(PCB, rstdlg_ctx.curr, NULL, NULL, &tmp, NULL, NULL, NULL, 1);
+	}
+	else if (idx == rstdlg_ctx.wfont) {
+
+		fgw_error_t err;
+		fgw_arg_t res, args[2];
+		
+		args[1].type = FGW_STR;
+		args[1].val.cstr = "fontid";
+		err = rnd_actionv_bin(&PCB->hidlib, "fontsel", &res, 2, args);
+		if ((err == 0) && (res.type == FGW_LONG)) {
+			pcb_font_id_t tmp = res.val.nat_long;
+			pcb_route_style_change(PCB, rstdlg_ctx.curr, NULL, NULL, NULL, &tmp, NULL, NULL, 1);
+		}
 	}
 	else if (idx == rstdlg_ctx.wclr)
-		pcb_route_style_change(PCB, rstdlg_ctx.curr, NULL, NULL, NULL, &attr->val.crd, NULL, 1);
-	else if (idx == rstdlg_ctx.wviahole) {
-		rst->Hole = attr->val.crd;
-		if (rst->Hole * 1.1 >= rstdlg_ctx.dlg[rstdlg_ctx.wviaring].val.crd) {
-			hv.crd = rst->Hole * 1.1;
-			rnd_gui->attr_dlg_set_value(rstdlg_ctx.dlg_hid_ctx, rstdlg_ctx.wviaring, &hv);
-			rst->Diameter = hv.crd;
+		pcb_route_style_change(PCB, rstdlg_ctx.curr, NULL, NULL, NULL, NULL, &attr->val.crd, NULL, 1);
+	else if (idx == rstdlg_ctx.wproto) {
+		fgw_error_t err;
+		fgw_arg_t res, args[4];
+		
+		args[1].type = FGW_STR;
+		args[1].val.cstr = "board";
+		args[2].type = FGW_STR;
+		args[2].val.cstr = "retpid";
+		args[3].type = FGW_LONG;
+		args[3].val.nat_long = rst->via_proto;
+		err = rnd_actionv_bin(&PCB->hidlib, "pstklib", &res, 4, args);
+		if ((err == 0) && (res.type == FGW_LONG)) {
+			rnd_cardinal_t tmp = res.val.nat_long;
+			pcb_route_style_change(PCB, rstdlg_ctx.curr, NULL, NULL, NULL, NULL, NULL, &tmp, 1);
 		}
-		rst_updated(rst);
-	}
-	else if (idx == rstdlg_ctx.wviaring) {
-		rst->Diameter = attr->val.crd;
-		if (rst->Diameter / 1.1 <= rstdlg_ctx.dlg[rstdlg_ctx.wviahole].val.crd) {
-			hv.crd = rst->Diameter / 1.1;
-			rnd_gui->attr_dlg_set_value(rstdlg_ctx.dlg_hid_ctx, rstdlg_ctx.wviahole, &hv);
-			rst->Hole = hv.crd;
-		}
-		rst_updated(rst);
 	}
 	else {
 		rnd_message(RND_MSG_ERROR, "Internal error: route style field does not exist");
 		return;
 	}
+
+	/* something got modified - re-select the current route style so "pen" is updated */
+	pcb_use_route_style(&(PCB->RouteStyle.array[rstdlg_ctx.curr]));
 }
 
 static int rst_edit_attr(char **key, char **val)
@@ -281,14 +307,6 @@ static void rstdlg_close_cb(void *caller_data, rnd_hid_attr_ev_t ev)
 
 	idx_changed(); /* flush pending timed changes */
 
-	{ /* can be safely removed when route style switches over to padstacks */
-		pcb_route_style_t *rst = vtroutestyle_get(&PCB->RouteStyle, ctx->curr, 0);
-		if (rst->Diameter <= rst->Hole) {
-			rnd_message(RND_MSG_ERROR, "had to increase the via ring diameter - can not be smaller than the hole");
-			rst->Diameter = rst->Hole+RND_MIL_TO_COORD(1);
-		}
-	}
-
 	RND_DAD_FREE(ctx->dlg);
 	memset(ctx, 0, sizeof(rstdlg_ctx_t)); /* reset all states to the initial - includes ctx->active = 0; */
 }
@@ -342,20 +360,20 @@ static int pcb_dlg_rstdlg(int rst_idx)
 				RND_DAD_MINMAX(rstdlg_ctx.dlg, 1, RND_MAX_COORD);
 				RND_DAD_CHANGE_CB(rstdlg_ctx.dlg, rst_change_cb);
 
-			RND_DAD_LABEL(rstdlg_ctx.dlg, "*Via hole:");
-			RND_DAD_COORD(rstdlg_ctx.dlg);
-				rstdlg_ctx.wviahole = RND_DAD_CURRENT(rstdlg_ctx.dlg);
-				RND_DAD_HELP(rstdlg_ctx.dlg, "Via hole diameter\nwarning: will be replaced with the padstack selector");
-				RND_DAD_MINMAX(rstdlg_ctx.dlg, 1, RND_MAX_COORD);
+			RND_DAD_LABEL(rstdlg_ctx.dlg, "Text font:");
+			RND_DAD_BUTTON(rstdlg_ctx.dlg, "<fid>");
+				rstdlg_ctx.wfont = RND_DAD_CURRENT(rstdlg_ctx.dlg);
+				RND_DAD_HELP(rstdlg_ctx.dlg, "Text font ID (button invokes the font selector)");
 				RND_DAD_CHANGE_CB(rstdlg_ctx.dlg, rst_change_cb);
 
-			RND_DAD_LABEL(rstdlg_ctx.dlg, "*Via ring:");
-			RND_DAD_COORD(rstdlg_ctx.dlg);
-				rstdlg_ctx.wviaring = RND_DAD_CURRENT(rstdlg_ctx.dlg);
-				RND_DAD_HELP(rstdlg_ctx.dlg, "Via ring diameter\nwarning: will be replaced with the padstack selector");
-				RND_DAD_MINMAX(rstdlg_ctx.dlg, 1, RND_MAX_COORD);
+
+			RND_DAD_LABEL(rstdlg_ctx.dlg, "Via padstack:");
+			RND_DAD_BUTTON(rstdlg_ctx.dlg, "<pid>");
+				rstdlg_ctx.wproto = RND_DAD_CURRENT(rstdlg_ctx.dlg);
+				RND_DAD_HELP(rstdlg_ctx.dlg, "Padstack prototype ID used for new vias with this style\n(button invokes the board padstack proto list dialog)");
 				RND_DAD_CHANGE_CB(rstdlg_ctx.dlg, rst_change_cb);
 
+
 		RND_DAD_END(rstdlg_ctx.dlg);
 		RND_DAD_TREE(rstdlg_ctx.dlg, 2, 0, attr_hdr);
 			RND_DAD_HELP(rstdlg_ctx.dlg, "These attributes are automatically added to\nany object drawn with this routing style");
@@ -400,7 +418,7 @@ fgw_error_t pcb_act_AdjustStyle(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	}
 
 	if (idx < 0) {
-		idx = pcb_route_style_lookup(&PCB->RouteStyle, conf_core.design.line_thickness, conf_core.design.via_thickness, conf_core.design.via_drilling_hole, conf_core.design.clearance, NULL);
+		idx = PCB_LOOKUP_ROUTE_STYLE_PEN(PCB);
 		if (idx < 0) {
 			rnd_message(RND_MSG_ERROR, "No style selected\n");
 			RND_ACT_IRES(-1);
