@@ -91,12 +91,12 @@ static void build_st_line1(void)
 static void build_st_line2(void)
 {
 	rnd_append_printf(&status.buf,
-		"%svia=%mS (%mS)  "
+		"%svia=#%ld  "
 		"clr=%mS  "
 		"text=%d%% %$mS "
 		"buff=#%d",
 		conf_core.appearance.compact ? "" : " ", /* "line" separator for single-line print */
-		conf_core.design.via_thickness, conf_core.design.via_drilling_hole,
+		(long)conf_core.design.via_proto,
 		conf_core.design.clearance,
 		conf_core.design.text_scale,
 		conf_core.design.text_thickness,
@@ -121,12 +121,12 @@ static void build_st_help(void)
 		"%m+"
 		"grid=%$mS  "
 		"line=%mS "
-		"via=%mS (%mS) "
+		"via=#%ld "
 		"clearance=%mS",
 		unit_inv->allow,
 		PCB->hidlib.grid,
 		conf_core.design.line_thickness,
-		conf_core.design.via_thickness, conf_core.design.via_drilling_hole, 
+		conf_core.design.via_proto,
 		conf_core.design.clearance);
 }
 
@@ -456,7 +456,7 @@ const char pcb_acth_DescribeLocation[] = "Return a string constant (valud until
 fgw_error_t pcb_act_DescribeLocation(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
 	rnd_coord_t x, y;
-	void *ptr1, *ptr2, *ptr3, *rptr1, *rptr2, *rptr3;
+	void *ptr1, *ptr2 = NULL, *ptr3, *rptr1, *rptr2, *rptr3;
 	pcb_any_obj_t *obj;
 	int type, rattype;
 	pcb_subc_t *subc;
@@ -486,7 +486,7 @@ fgw_error_t pcb_act_DescribeLocation(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		goto maybe_rat;
 
 	obj = ptr2;
-	if (obj->term == NULL)
+	if ((obj == NULL) || (obj->term == NULL))
 		goto maybe_rat;
 
 	subc = pcb_obj_parent_subc(ptr2);
