@@ -141,18 +141,33 @@ static void elarc90(pcb_poly_t *p, rnd_coord_t ccx, rnd_coord_t ccy, rnd_coord_t
 #define CORNER(outx, outy, rect_signx, rect_signy, rsignx, rsigny) \
 	outx = rnd_round((double)cx + rect_signx * (double)w/2 + rsignx*rx); \
 	outy = rnd_round((double)cy + rect_signy * (double)h/2 + rsigny*ry);
-pcb_poly_t *pcb_genpoly_roundrect(pcb_layer_t *layer, rnd_coord_t w, rnd_coord_t h, rnd_coord_t rx, rnd_coord_t ry, double rot_deg, rnd_coord_t cx, rnd_coord_t cy, pcb_shape_corner_t corner[4], double roundres)
+#define is_sq(idx) (corner[idx] == PCB_CORN_SQUARE)
+pcb_poly_t *pcb_genpoly_roundrect(pcb_layer_t *layer, rnd_coord_t w, rnd_coord_t h, rnd_coord_t rx, rnd_coord_t ry, double rot_deg, rnd_coord_t cx, rnd_coord_t cy, pcb_shape_corner_t corner[4], double roundres, const char **err)
 {
 	pcb_poly_t *p;
-	rnd_coord_t maxr = (w < h ? w : h) / 2, x, y, ex, ey, acx, acy, ccx, ccy;
+	rnd_coord_t maxr, maxrh, maxrw, x, y, ex, ey, acx, acy, ccx, ccy;
 	int segs, need_rot, flags = PCB_FLAG_CLEARPOLY;
+	int hsq, wsq;
 	double cosra, sinra;
 
-	if ((w <= 10) || (h <= 10))
-		return NULL;
 
-	if ((rx > maxr) || (ry > maxr))
+	/* calculate the minimum number of square corners in horizontal or vertical dir */
+	wsq = RND_MIN(is_sq(0) + is_sq(1), is_sq(2) + is_sq(3));
+	hsq = RND_MIN(is_sq(0) + is_sq(3), is_sq(1) + is_sq(2));
+
+	maxrw = (wsq == 0) ? w/2 : w;
+	maxrh = (hsq == 0) ? h/2 : h;
+	maxr = RND_MIN(maxrh, maxrw);
+
+	if ((w <= 10) || (h <= 10)) {
+		*err = "side too small";
 		return NULL;
+	}
+
+	if ((rx > maxr) || (ry > maxr)) {
+		*err = "rounding radius too large";
+		return NULL;
+	}
 
 	if (rot_deg != 0.0) {
 		cosra = cos(rot_deg / RND_RAD_TO_DEG);
@@ -169,8 +184,10 @@ pcb_poly_t *pcb_genpoly_roundrect(pcb_layer_t *layer, rnd_coord_t w, rnd_coord_t
 	segs = rnd_round((double)segs * roundres);
 
 	p = pcb_poly_new(layer, 2 * conf_core.design.clearance, pcb_flag_make(flags));
-	if (p == NULL)
+	if (p == NULL) {
+		*err = "poly gen error";
 		return NULL;
+	}
 
 	/* top right */
 	CORNER(  x,   y, +1, -1,  0, +1); /* start */
@@ -204,6 +221,7 @@ pcb_poly_t *pcb_genpoly_roundrect(pcb_layer_t *layer, rnd_coord_t w, rnd_coord_t
 
 	return p;
 }
+#undef is_sq
 #undef CORNER
 
 void pcb_shape_roundrect(pcb_pstk_shape_t *shape, rnd_coord_t width, rnd_coord_t height, double roundness)
@@ -215,7 +233,7 @@ void pcb_shape_roundrect(pcb_pstk_shape_t *shape, rnd_coord_t width, rnd_coord_t
 	int inited = 0, n;
 	rnd_coord_t rr, minor = MIN(width, height);
 	pcb_shape_corner_t corner[4] = { PCB_CORN_ROUND, PCB_CORN_ROUND, PCB_CORN_ROUND, PCB_CORN_ROUND};
-
+	const char *err;
 
 	if (!inited) {
 		pcb_data_init(&data);
@@ -230,7 +248,7 @@ void pcb_shape_roundrect(pcb_pstk_shape_t *shape, rnd_coord_t width, rnd_coord_t
 		layer = &data.Layer[0];
 
 	rr = rnd_round(minor * roundness);
-	p = pcb_genpoly_roundrect(layer, width, height, rr, rr, 0, 0, 0, corner, 4.0);
+	p = pcb_genpoly_roundrect(layer, width, height, rr, rr, 0, 0, 0, corner, 4.0, &err);
 	pcb_pstk_shape_alloc_poly(dst, p->PointN);
 	shape->shape = PCB_PSSH_POLY;
 
@@ -273,14 +291,14 @@ static pcb_poly_t *regpoly_place(pcb_data_t *data, pcb_layer_t *layer, int corne
 	return any_poly_place(data, layer, p);
 }
 
-static pcb_poly_t *roundrect_place(pcb_data_t *data, pcb_layer_t *layer, rnd_coord_t w, rnd_coord_t h, rnd_coord_t rx, rnd_coord_t ry, double rot_deg, rnd_coord_t cx, rnd_coord_t cy, pcb_shape_corner_t corner[4], double roundres)
+static pcb_poly_t *roundrect_place(pcb_data_t *data, pcb_layer_t *layer, rnd_coord_t w, rnd_coord_t h, rnd_coord_t rx, rnd_coord_t ry, double rot_deg, rnd_coord_t cx, rnd_coord_t cy, pcb_shape_corner_t corner[4], double roundres, const char **err)
 {
 	pcb_poly_t *p;
 
 	if (layer == pcb_shape_current_layer)
 		layer = PCB_CURRLAYER(PCB);
 
-	p = pcb_genpoly_roundrect(layer, w, h, rx, ry, rot_deg, cx, cy, corner, roundres);
+	p = pcb_genpoly_roundrect(layer, w, h, rx, ry, rot_deg, cx, cy, corner, roundres, err);
 	return any_poly_place(data, layer, p);
 }
 
@@ -459,6 +477,7 @@ fgw_error_t pcb_act_roundrect(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	double rot = 0.0, roundres = 1.0;
 	char *end;
 	pcb_shape_corner_t corner[4] = { PCB_CORN_ROUND, PCB_CORN_ROUND, PCB_CORN_ROUND, PCB_CORN_ROUND};
+	const char *err;
 
 	if (argc < 2) {
 		rnd_message(RND_MSG_ERROR, "roundrect() needs at least one parameters\n");
@@ -555,7 +574,7 @@ fgw_error_t pcb_act_roundrect(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	if ((data == PCB->Data) && (!have_coords))
 		rnd_hid_get_coords("Click on the center of the polygon", &x, &y, 0);
 
-	if (roundrect_place(data, PCB_CURRLAYER(PCB), w, h, rx, ry, rot, x, y, corner, roundres) == NULL)
+	if (roundrect_place(data, PCB_CURRLAYER(PCB), w, h, rx, ry, rot, x, y, corner, roundres, &err) == NULL)
 		rnd_message(RND_MSG_ERROR, "roundrect(): failed to create the polygon\n");
 
 	RND_ACT_IRES(0);
