@@ -4,7 +4,7 @@
  *  pcb-rnd, interactive printed circuit board design
  *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 2004, 2007 Dan McMahill
- *  Copyright (C) 2018 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2018,2021 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,8 +35,11 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <genht/hash.h>
+#include <genvector/vtp0.h>
 #include <genregex/regex_sei.h>
 
+#include <librnd/core/anyload.h>
 #include "change.h"
 #include "board.h"
 #include "data.h"
@@ -60,12 +63,80 @@ conf_vendor_t conf_vendor;
 
 static void add_to_drills(char *);
 static void apply_vendor_map(void);
-static void process_skips(lht_node_t *);
-static rnd_bool rematch(const char *, const char *);
+static long process_skips(lht_node_t *);
 static void vendor_free_all(void);
 rnd_coord_t vendorDrillMap(rnd_coord_t in);
 
 
+/*** skip ***/
+static vtp0_t skip_invalid;
+#define HT_INVALID_VALUE skip_invalid
+#define HT_HAS_CONST_KEY
+typedef char *htsv_key_t;
+typedef const char *htsv_const_key_t;
+typedef vtp0_t htsv_value_t;
+#define HT(x) htsv_ ## x
+#include <genht/ht.h>
+#include <genht/ht.c>
+#undef HT
+
+static int skips_inited = 0;
+static htsv_t skips; /* key=attribute-name, value = vtp0_t pairs of (precompiled regex list):(string) */
+
+/* compile regex_str and remember it for skipping an attrib */
+static void skip_add(const char *attrib, const char *regex_str)
+{
+	vtp0_t *slot;
+	re_sei_t *regex;
+	htsv_entry_t *e;
+
+	if (!skips_inited) {
+		htsv_init(&skips, strhash, strkeyeq);
+		skips_inited = 1;
+	}
+
+	e = htsv_getentry(&skips, attrib);
+	if (e == NULL) {
+		htsv_set(&skips, rnd_strdup(attrib), skip_invalid);
+		e = htsv_getentry(&skips, attrib);
+	}
+	slot = &e->value;
+
+	/* compile the regular expression */
+	regex = re_sei_comp(regex_str);
+	if (re_sei_errno(regex) != 0) {
+		rnd_message(RND_MSG_ERROR, "regexp error: %s\n", re_error_str(re_sei_errno(regex)));
+		re_sei_free(regex);
+		return;
+	}
+	vtp0_append(slot, regex);
+	vtp0_append(slot, rnd_strdup(regex_str));
+}
+
+static void skip_uninit(void)
+{
+	htsv_entry_t *e;
+
+	for(e = htsv_first(&skips); e != NULL; e = htsv_next(&skips, e)) {
+		long n;
+		vtp0_t *slot = &e->value;
+
+		for(n = 0; n < slot->used; n+=2) {
+			void *rx  = slot->array[n];
+			char *str = slot->array[n+1];
+			re_sei_free(rx);
+			free(str);
+			
+		}
+		vtp0_uninit(slot);
+		free(e->key);
+	}
+	htsv_uninit(&skips);
+	skips_inited = 0;
+}
+
+/***/
+
 /* list of vendor drills and a count of them */
 static rnd_coord_t *vendor_drills = NULL;
 static int n_vendor_drills = 0;
@@ -73,36 +144,21 @@ static int n_vendor_drills = 0;
 static int cached_drill = -1;
 static int cached_map = -1;
 
-/* lists of elements to ignore */
-static char **ignore_refdes = NULL;
-static int n_refdes = 0;
-static char **ignore_value = NULL;
-static int n_value = 0;
-static char **ignore_descr = NULL;
-static int n_descr = 0;
-
-/* vendor name */
-static char *vendor_name = NULL;
-
 /* resource file to PCB units scale factor */
 static double sf;
 
 
 /* type of drill mapping */
-#define CLOSEST 1
-#define ROUND_UP 0
-static int rounding_method = ROUND_UP;
+typedef enum {
+	ROUND_UP = 0,
+	ROUND_NEAREST,
+	ROUND_DOWN
+} vendor_round_t;
+
+static vendor_round_t rounding_method = ROUND_UP;
 
 #define FREE(x) if((x) != NULL) { free (x) ; (x) = NULL; }
 
-/* load a board metadata into conf_core */
-static void load_meta_coord(const char *path, rnd_coord_t crd)
-{
-	char tmp[128];
-	rnd_sprintf(tmp, "%$mm", crd);
-	rnd_conf_set(RND_CFR_DESIGN, path, -1, tmp, RND_POL_OVERWRITE);
-}
-
 static rnd_bool vendorIsSubcMappable(pcb_subc_t *subc);
 
 
@@ -129,60 +185,36 @@ fgw_error_t pcb_act_UnloadVendor(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	return 0;
 }
 
-
-static const char pcb_acts_LoadVendorFrom[] = "LoadVendorFrom(filename)";
-static const char pcb_acth_LoadVendorFrom[] = "Loads the specified vendor lihata file.";
-/* DOC: loadvendorfrom.html */
-fgw_error_t pcb_act_LoadVendorFrom(fgw_arg_t *res, int argc, fgw_arg_t *argv)
+const char *lht_get_htext(lht_node_t *h, const char *name)
 {
-	const char *fname = NULL;
-	static char *default_file = NULL;
-	const char *sval;
-	lht_doc_t *doc;
+	lht_node_t *n = lht_dom_hash_get(h, name);
+
+	if ((n == NULL) || (n->type != LHT_TEXT))
+		return NULL;
+
+	return n->data.text.value;
+}
+
+static int vendor_load_root(const char *fname, lht_node_t *root, rnd_bool pure)
+{
+	long num_skips;
 	lht_node_t *drlres;
-	rnd_bool free_fname = rnd_false;
+	const char *sval;
+	int res = 0;
 
-	cached_drill = -1;
-
-	RND_ACT_MAY_CONVARG(1, FGW_STR, LoadVendorFrom, fname = argv[1].val.str);
-
-	if (!fname || !*fname) {
-		fname = rnd_gui->fileselect(rnd_gui, "Load Vendor Resource File...",
-														"Picks a vendor resource file to load.\n"
-															"This file can contain drc settings for a\n"
-															"particular vendor as well as a list of\n"
-															"predefined drills which are allowed.", default_file, ".res", NULL, "vendor", RND_HID_FSD_READ, NULL);
-		if (fname == NULL) {
-			RND_ACT_IRES(1);
-			return 0;
-		}
-
-		free_fname = rnd_true;
-
-		free(default_file);
-		default_file = NULL;
-
-		if (fname && *fname)
-			default_file = rnd_strdup(fname);
+	if (root->type != LHT_HASH) {
+		rnd_hid_cfg_error(root, "vendor drill root node must be a hash\n");
+		return -1;
 	}
 
-	vendor_free_all();
-
-	/* load the resource file */
-	doc = rnd_hid_cfg_load_lht(&PCB->hidlib, fname);
-	if (doc == NULL) {
-		rnd_message(RND_MSG_ERROR, "Could not load vendor resource file \"%s\"\n", fname);
-		RND_ACT_IRES(1);
-		return 0;
-	}
-
-	/* figure out the vendor name, if specified */
-	vendor_name = (char *) RND_UNKNOWN(rnd_hid_cfg_text_value(doc, "vendor"));
+	if (!pure)
+		vendor_free_all();
 
 	/* figure out the units, if specified */
-	sval = rnd_hid_cfg_text_value(doc, "/units");
+	sval = lht_get_htext(root, "units");
 	if (sval == NULL) {
-		sf = RND_MIL_TO_COORD(1);
+		if (!pure)
+			sf = RND_MIL_TO_COORD(1);
 	}
 	else if ((RND_NSTRCMP(sval, "mil") == 0) || (RND_NSTRCMP(sval, "mils") == 0)) {
 		sf = RND_MIL_TO_COORD(1);
@@ -198,26 +230,32 @@ fgw_error_t pcb_act_LoadVendorFrom(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		sf = RND_INCH_TO_COORD(1);
 	}
 
-	/* default to ROUND_UP */
-	rounding_method = ROUND_UP;
-	sval = rnd_hid_cfg_text_value(doc, "/round");
+	if (!pure)
+		rounding_method = ROUND_UP; /* default to ROUND_UP */
+
+	sval = lht_get_htext(root, "round");
 	if (sval != NULL) {
 		if (RND_NSTRCMP(sval, "up") == 0) {
 			rounding_method = ROUND_UP;
 		}
-		else if (RND_NSTRCMP(sval, "nearest") == 0) {
-			rounding_method = CLOSEST;
+		else if (RND_NSTRCMP(sval, "down") == 0) {
+			rounding_method = ROUND_DOWN;
 		}
-		else {
+		else if (RND_NSTRCMP(sval, "nearest") == 0) {
+			rounding_method = ROUND_NEAREST;
+		}
+		else if (!pure) {
 			rnd_message(RND_MSG_ERROR, "\"%s\" is not a valid rounding type.  Defaulting to up\n", sval);
 			rounding_method = ROUND_UP;
 		}
 	}
 
-	process_skips(lht_tree_path(doc, "/", "/skips", 1, NULL));
+	num_skips = process_skips(lht_dom_hash_get(root, "skips"));
+	if (num_skips < 0)
+		res = -1;
 
 	/* extract the drillmap resource */
-	drlres = lht_tree_path(doc, "/", "/drillmap", 1, NULL);
+	drlres = lht_dom_hash_get(root, "drillmap");
 	if (drlres != NULL) {
 		if (drlres->type == LHT_LIST) {
 			lht_node_t *n;
@@ -234,52 +272,80 @@ fgw_error_t pcb_act_LoadVendorFrom(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	else
 		rnd_message(RND_MSG_ERROR, "No drillmap resource found\n");
 
-	sval = rnd_hid_cfg_text_value(doc, "/drc/copper_space");
-	if (sval != NULL) {
-		load_meta_coord("design/bloat", floor(sf * atof(sval) + 0.5));
-		rnd_message(RND_MSG_INFO, "Set DRC minimum copper spacing to %ml mils\n", conf_core.design.bloat);
-	}
-
-	sval = rnd_hid_cfg_text_value(doc, "/drc/copper_overlap");
-	if (sval != NULL) {
-		load_meta_coord("design/shrink", floor(sf * atof(sval) + 0.5));
-		rnd_message(RND_MSG_INFO, "Set DRC minimum copper overlap to %ml mils\n", conf_core.design.shrink);
-	}
-
-	sval = rnd_hid_cfg_text_value(doc, "/drc/copper_width");
-	if (sval != NULL) {
-		load_meta_coord("design/min_wid", floor(sf * atof(sval) + 0.5));
-		rnd_message(RND_MSG_INFO, "Set DRC minimum copper spacing to %ml mils\n", conf_core.design.min_wid);
-	}
-
-	sval = rnd_hid_cfg_text_value(doc, "/drc/silk_width");
-	if (sval != NULL) {
-		load_meta_coord("design/min_slk", floor(sf * atof(sval) + 0.5));
-		rnd_message(RND_MSG_INFO, "Set DRC minimum silk width to %ml mils\n", conf_core.design.min_slk);
-	}
-
-	sval = rnd_hid_cfg_text_value(doc, "/drc/min_drill");
-	if (sval != NULL) {
-		load_meta_coord("design/min_drill", floor(sf * atof(sval) + 0.5));
-		rnd_message(RND_MSG_INFO, "Set DRC minimum drill diameter to %ml mils\n", conf_core.design.min_drill);
-	}
-
-	sval = rnd_hid_cfg_text_value(doc, "/drc/min_ring");
-	if (sval != NULL) {
-		load_meta_coord("design/min_ring", floor(sf * atof(sval) + 0.5));
-		rnd_message(RND_MSG_INFO, "Set DRC minimum annular ring to %ml mils\n", conf_core.design.min_ring);
+	if (lht_dom_hash_get(root, "drc") != NULL) {
+		rnd_message(RND_MSG_ERROR, "Vendordrill: %s contains a drc subtree. The vendor drill plugin does not support DRC settings anymore\n", fname);
+		rnd_message(RND_MSG_ERROR, "Vendordrill: please refer to http://www.repo.hu/projects/pcb-rnd/help/err0003.html\n");
 	}
 
 	rnd_message(RND_MSG_INFO, "Loaded %d vendor drills from %s\n", n_vendor_drills, fname);
-	rnd_message(RND_MSG_INFO, "Loaded %d RefDes skips, %d Value skips, %d Descr skips\n", n_refdes, n_value, n_descr);
+	rnd_message(RND_MSG_INFO, "Loaded %ld skips for %d different attributes\n", num_skips, skips.used);
 
 	rnd_conf_set(RND_CFR_DESIGN, "plugins/vendor/enable", -1, "0", RND_POL_OVERWRITE);
 
-	apply_vendor_map();
+	if (!pure)
+		apply_vendor_map();
+
+	return res;
+}
+
+static const char pcb_acts_LoadVendorFrom[] = "LoadVendorFrom(filename, [yes|no])";
+static const char pcb_acth_LoadVendorFrom[] = "Loads the specified vendor lihata file. If second argument is \"yes\" or \"pure\", load in pure mode without side effects: do not reset or apply, only incrementally load.";
+/* DOC: loadvendorfrom.html */
+fgw_error_t pcb_act_LoadVendorFrom(fgw_arg_t *res, int argc, fgw_arg_t *argv)
+{
+	const char *fname = NULL, *spure = NULL;
+	static char *default_file = NULL;
+	int pure = 0;
+	lht_doc_t *doc;
+	rnd_bool free_fname = rnd_false;
+	int r;
+
+	cached_drill = -1;
+
+	RND_ACT_MAY_CONVARG(1, FGW_STR, LoadVendorFrom, fname = argv[1].val.str);
+	RND_ACT_MAY_CONVARG(2, FGW_STR, LoadVendorFrom, spure = argv[2].val.str);
+
+	if (!fname || !*fname) {
+		fname = rnd_gui->fileselect(rnd_gui, "Load Vendor Resource File...",
+			"Picks a vendor resource file to load.\n"
+			"This file contains a list of\n"
+			"predefined drills which are allowed.", default_file, ".res",
+			NULL, "vendor", RND_HID_FSD_READ, NULL);
+		if (fname == NULL) {
+			RND_ACT_IRES(1);
+			return 0;
+		}
+
+		free_fname = rnd_true;
+
+		free(default_file);
+		default_file = NULL;
+
+		if (fname && *fname)
+			default_file = rnd_strdup(fname);
+	}
+
+	if (spure != NULL) {
+		if (strcmp(spure, "pure") == 0)
+			pure = 1;
+		else
+			pure = rnd_istrue(spure);
+	}
+
+	/* load the resource file */
+	doc = rnd_hid_cfg_load_lht(&PCB->hidlib, fname);
+	if (doc == NULL) {
+		rnd_message(RND_MSG_ERROR, "Could not load vendor resource file \"%s\"\n", fname);
+		RND_ACT_IRES(1);
+		return 0;
+	}
+
+	r = vendor_load_root(fname, doc->root, pure);
+
 	if (free_fname)
 		free((char*)fname);
 	lht_dom_uninit(doc);
-	RND_ACT_IRES(0);
+	RND_ACT_IRES(r);
 	return 0;
 }
 
@@ -346,33 +412,38 @@ static void apply_vendor_map(void)
 		rnd_message(RND_MSG_INFO, "Updated %ld drill sizes out of %ld total\n", (long)changed, (long)tot);
 
 		/* Update the current Via */
-		if (conf_core.design.via_drilling_hole != vendorDrillMap(conf_core.design.via_drilling_hole)) {
-			changed++;
-			rnd_conf_setf(RND_CFR_DESIGN, "design/via_drilling_hole", -1, "%$mm", vendorDrillMap(conf_core.design.via_drilling_hole));
-			rnd_message(RND_MSG_INFO, "Adjusted active via hole size to be %ml mils\n", conf_core.design.via_drilling_hole);
+		{
+			pcb_pstk_proto_t *proto = pcb_pstk_get_proto_(PCB->Data, conf_core.design.via_proto);
+			rnd_coord_t ndia = (proto != NULL) ? vendorDrillMap(proto->hdia) : 0;
+
+			if ((proto != NULL) && (proto->hdia != ndia)) {
+				changed++;
+				pcb_pstk_proto_change_hole(proto, NULL, &ndia, NULL, NULL);
+				rnd_conf_setf(RND_CFR_DESIGN, "design/via_drilling_hole", -1, "%$mm", ndia);
+				rnd_message(RND_MSG_INFO, "Adjusted pen via hole size to be %ml mils\n", ndia);
+			}
 		}
 
 		/* and update the vias for the various routing styles */
 		for (i = 0; i < vtroutestyle_len(&PCB->RouteStyle); i++) {
-			if (PCB->RouteStyle.array[i].Hole != vendorDrillMap(PCB->RouteStyle.array[i].Hole)) {
+			pcb_pstk_proto_t *proto = pcb_pstk_get_proto_(PCB->Data, PCB->RouteStyle.array[i].via_proto);
+			rnd_coord_t ndia;
+
+			if (proto == NULL)
+				continue;
+
+			ndia = vendorDrillMap(proto->hdia);
+			if (proto->hdia != ndia) {
 				changed++;
-				PCB->RouteStyle.array[i].Hole = vendorDrillMap(PCB->RouteStyle.array[i].Hole);
+				pcb_pstk_proto_change_hole(proto, NULL, &ndia, NULL, NULL);
 				rnd_message(RND_MSG_INFO,
-								"Adjusted %s routing style hole size to be %ml mils\n",
-								PCB->RouteStyle.array[i].name, PCB->RouteStyle.array[i].Hole);
-				if (PCB->RouteStyle.array[i].Diameter < PCB->RouteStyle.array[i].Hole + PCB_MIN_PINORVIACOPPER) {
-					PCB->RouteStyle.array[i].Diameter = PCB->RouteStyle.array[i].Hole + PCB_MIN_PINORVIACOPPER;
-					rnd_message(RND_MSG_INFO,
-									"Increased %s routing style via diameter to %ml mils\n",
-									PCB->RouteStyle.array[i].name, PCB->RouteStyle.array[i].Diameter);
-				}
+					"Adjusted %s routing style hole size to be %ml mils\n",
+					PCB->RouteStyle.array[i].name, ndia);
 			}
 		}
 
-		/*
-		 * if we've changed anything, indicate that we need to save the
-		 * file, redraw things, and make sure we can undo.
-		 */
+		/* if we've changed anything, indicate that we need to save the
+		   file, redraw things, and make sure we can undo. */
 		if (changed) {
 			pcb_board_set_changed_flag(PCB, rnd_true);
 			rnd_hid_redraw(PCB);
@@ -384,7 +455,7 @@ static void apply_vendor_map(void)
 	rnd_conf_force_set_bool(conf_vendor.plugins.vendor.enable, state);
 }
 
-/* for a given drill size, find the closest vendor drill size */
+/* for a given drill size, find the nearest vendor drill size */
 rnd_coord_t vendorDrillMap(rnd_coord_t in)
 {
 	int i, min, max;
@@ -414,7 +485,7 @@ rnd_coord_t vendorDrillMap(rnd_coord_t in)
 		return vendor_drills[n_vendor_drills - 1];
 	}
 
-	/* figure out which 2 drills are closest in size */
+	/* figure out which 2 drills are nearest in size */
 	min = 0;
 	max = n_vendor_drills - 1;
 	while (max - min > 1) {
@@ -427,23 +498,31 @@ rnd_coord_t vendorDrillMap(rnd_coord_t in)
 	i = max;
 
 	/* now round per the rounding mode */
-	if (rounding_method == CLOSEST) {
-		/* find the closest drill size */
-		if ((in - vendor_drills[i - 1]) > (vendor_drills[i] - in)) {
-			cached_map = vendor_drills[i];
-			return vendor_drills[i];
-		}
-		else {
+	switch(rounding_method) {
+		case ROUND_NEAREST:
+			if ((in - vendor_drills[i - 1]) > (vendor_drills[i] - in)) {
+				cached_map = vendor_drills[i];
+				return vendor_drills[i];
+			}
 			cached_map = vendor_drills[i - 1];
 			return vendor_drills[i - 1];
-		}
-	}
-	else {
-		/* always round up */
-		cached_map = vendor_drills[i];
-		return vendor_drills[i];
+
+		case ROUND_UP:
+			goto round_up;
+
+		case ROUND_DOWN:
+			if (in == vendor_drills[max]) {
+				cached_map = vendor_drills[max];
+				return vendor_drills[max];
+			}
+			cached_map = vendor_drills[min];
+			return vendor_drills[min];
 	}
 
+	/* default, for invalid internal state */
+	round_up:;
+		cached_map = vendor_drills[i];
+		return vendor_drills[i];
 }
 
 /* add a drill size to the vendor drill list */
@@ -464,11 +543,9 @@ static void add_to_drills(char *sval)
 	tmpd = atof(sval);
 	val = floor(sf * tmpd + 0.5);
 
-	/*
-	 * We keep the array of vendor drills sorted to make it easier to
-	 * do the rounding later.  The algorithm used here is not so efficient,
-	 * but we're not dealing with much in the way of data.
-	 */
+	/* We keep the array of vendor drills sorted to make it easier to
+	   do the rounding later.  The algorithm used here is not so efficient,
+	   but we're not dealing with much in the way of data. */
 
 	/* figure out where to insert the value to keep the array sorted.  */
 	k = 0;
@@ -488,126 +565,85 @@ static void add_to_drills(char *sval)
 	}
 }
 
-/* deal with the "skip" subresource */
-static void process_skips(lht_node_t *res)
+/* deal with the "skip" subtree; returns number of skips loaded */
+static long process_skips(lht_node_t *res)
 {
-	char *sval;
-	int *cnt;
-	char ***lst = NULL;
 	lht_node_t *n;
+	const char *attr;
+	long cnt = 0;
 
 	if (res == NULL)
-		return;
+		return 0;
 
-	if (res->type != LHT_LIST)
+	if (res->type != LHT_LIST) {
 		rnd_hid_cfg_error(res, "skips must be a list.\n");
+		return -1;
+	}
 
 	for(n = res->data.list.first; n != NULL; n = n->next) {
 		if (n->type == LHT_TEXT) {
-			if (RND_NSTRCMP(n->name, "refdes") == 0) {
-				cnt = &n_refdes;
-				lst = &ignore_refdes;
-			}
-			else if (RND_NSTRCMP(n->name, "value") == 0) {
-				cnt = &n_value;
-				lst = &ignore_value;
+			if ((RND_NSTRCMP(n->name, "refdes") == 0) || (RND_NSTRCMP(n->name, "value") == 0)) {
+				attr = n->name;
 			}
 			else if (RND_NSTRCMP(n->name, "descr") == 0) {
-				cnt = &n_descr;
-				lst = &ignore_descr;
+				attr = "footprint";
+			}
+			else if ((n->name[0] == 'a') && (n->name[1] == '.')) {
+				attr = n->name+2;
 			}
 			else {
 				rnd_hid_cfg_error(n, "invalid skip name; must be one of refdes, value, descr");
 				continue;
 			}
-			/* add the entry to the appropriate list */
-			sval = n->data.text.value;
-			(*cnt)++;
-			if ((*lst = (char **) realloc(*lst, (*cnt) * sizeof(char *))) == NULL) {
-				fprintf(stderr, "realloc() failed\n");
-				exit(-1);
-			}
-			(*lst)[*cnt - 1] = rnd_strdup(sval);
+			skip_add(attr, n->data.text.value);
+			cnt++;
 		}
 		else
 			rnd_hid_cfg_error(n, "invalid skip type; must be text");
 	}
+	return cnt;
 }
 
 static rnd_bool vendorIsSubcMappable(pcb_subc_t *subc)
 {
-	int i;
-	int noskip;
+	int noskip = 1;
+	htsv_entry_t *e;
 
 	if (!conf_vendor.plugins.vendor.enable)
 		return rnd_false;
 
-TODO(": these 3 loops should be wrapped in a single loop that iterates over attribute keys")
-	noskip = 1;
-	for (i = 0; i < n_refdes; i++) {
-		if ((RND_NSTRCMP(RND_UNKNOWN(subc->refdes), ignore_refdes[i]) == 0)
-				|| rematch(ignore_refdes[i], RND_UNKNOWN(subc->refdes))) {
-			rnd_message(RND_MSG_INFO, "Vendor mapping skipped because refdes = %s matches %s\n", RND_UNKNOWN(subc->refdes), ignore_refdes[i]);
-			noskip = 0;
-		}
-	}
-	if (noskip) {
-		const char *vl = pcb_attribute_get(&subc->Attributes, "value");
-		for (i = 0; i < n_value; i++) {
-			if ((RND_NSTRCMP(RND_UNKNOWN(vl), ignore_value[i]) == 0)
-					|| rematch(ignore_value[i], RND_UNKNOWN(vl))) {
-				rnd_message(RND_MSG_INFO, "Vendor mapping skipped because value = %s matches %s\n", RND_UNKNOWN(vl), ignore_value[i]);
-				noskip = 0;
-			}
-		}
-	}
-
-	if (noskip) {
-		const char *fp = pcb_attribute_get(&subc->Attributes, "footprint");
-		for (i = 0; i < n_descr; i++) {
-			if ((RND_NSTRCMP(RND_UNKNOWN(fp), ignore_descr[i]) == 0)
-					|| rematch(ignore_descr[i], RND_UNKNOWN(fp))) {
-				rnd_message(RND_MSG_INFO,
-								"Vendor mapping skipped because descr = %s matches %s\n",
-								RND_UNKNOWN(fp), ignore_descr[i]);
-				noskip = 0;
-			}
-		}
-	}
-
-	if (noskip && PCB_FLAG_TEST(PCB_FLAG_LOCK, subc)) {
+	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, subc)) {
 		rnd_message(RND_MSG_INFO, "Vendor mapping skipped because element %s is locked\n", RND_UNKNOWN(subc->refdes));
 		noskip = 0;
+		goto done;
 	}
 
+	for(e = htsv_first(&skips); e != NULL; e = htsv_next(&skips, e)) {
+		long n;
+		vtp0_t *slot = &e->value;
+		const char *attr = e->key;
+		const char *vl = RND_UNKNOWN(pcb_attribute_get(&subc->Attributes, attr));
+
+		for(n = 0; n < slot->used; n+=2) {
+			void *rx  = slot->array[n];
+			char *str = slot->array[n+1];
+
+			if (re_sei_exec(rx, vl) != 0) {
+				rnd_message(RND_MSG_INFO, "Vendor mapping skipped because %s = %s matches %s\n", attr, vl, str);
+				noskip = 0;
+				goto done;
+			}
+		}
+	}
+
+	done:;
+
 	if (noskip)
 		return rnd_true;
 	else
 		return rnd_false;
 }
 
-static rnd_bool rematch(const char *re, const char *s)
-{
-	int result;
-	re_sei_t *regex;
-
-	/* compile the regular expression */
-	regex = re_sei_comp(re);
-	if (re_sei_errno(regex) != 0) {
-		rnd_message(RND_MSG_ERROR, "regexp error: %s\n", re_error_str(re_sei_errno(regex)));
-		re_sei_free(regex);
-		return rnd_false;
-	}
-
-	result = re_sei_exec(regex, s);
-	re_sei_free(regex);
-
-	if (result != 0)
-		return rnd_true;
-	else
-		return rnd_false;
-}
-
 static const char *vendor_cookie = "vendor drill mapping";
 
 rnd_action_t vendor_action_list[] = {
@@ -616,24 +652,9 @@ rnd_action_t vendor_action_list[] = {
 	{"LoadVendorFrom", pcb_act_LoadVendorFrom, pcb_acth_LoadVendorFrom, pcb_acts_LoadVendorFrom}
 };
 
-static char **vendor_free_vect(char **lst, int *len)
-{
-	if (lst != NULL) {
-		int n;
-		for(n = 0; n < *len; n++)
-			if (lst[n] != NULL)
-				free(lst[n]);
-		free(lst);
-	}
-	*len = 0;
-	return NULL;
-}
-
 static void vendor_free_all(void)
 {
-	ignore_refdes = vendor_free_vect(ignore_refdes, &n_refdes);
-	ignore_value = vendor_free_vect(ignore_value, &n_value);
-	ignore_descr = vendor_free_vect(ignore_descr, &n_descr);
+	skip_uninit();
 	if (vendor_drills != NULL) {
 		free(vendor_drills);
 		vendor_drills = NULL;
@@ -655,10 +676,18 @@ static void vendor_new_pstk(rnd_hidlib_t *hidlib, void *user_data, int argc, rnd
 	apply_vendor_pstk1(ps, &dummy);
 }
 
+static int vendor_anyload_subtree(const rnd_anyload_t *al, rnd_hidlib_t *hl, lht_node_t *root)
+{
+	return vendor_load_root(root->file_name, root, 0);
+}
+
+static rnd_anyload_t vendor_anyload = {0};
+
 int pplg_check_ver_vendordrill(int ver_needed) { return 0; }
 
 void pplg_uninit_vendordrill(void)
 {
+	rnd_anyload_unreg_by_cookie(vendor_cookie);
 	rnd_event_unbind_allcookie(vendor_cookie);
 	rnd_remove_actions_by_cookie(vendor_cookie);
 	vendor_free_all();
@@ -676,5 +705,10 @@ int pplg_init_vendordrill(void)
 	rnd_event_bind(PCB_EVENT_NEW_PSTK, vendor_new_pstk, NULL, vendor_cookie);
 	RND_REGISTER_ACTIONS(vendor_action_list, vendor_cookie)
 	rnd_hid_menu_load(rnd_gui, NULL, vendor_cookie, 110, NULL, 0, vendor_menu, "plugin: vendor drill mapping");
+
+	vendor_anyload.load_subtree = vendor_anyload_subtree;
+	vendor_anyload.cookie = vendor_cookie;
+	rnd_anyload_reg("^vendor_drill_map$", &vendor_anyload);
+
 	return 0;
 }
